/*

Copyright © David Tritscher. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:

   This product includes software developed by David Tritscher.

4. The name of the developer may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY DAVID TRITSCHER ''AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL DAVID TRITSCHER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/* 2009-03-07 04:21:25 */

const char *name = "cavegrab";

#ifdef WITH_IPF

#include "caps/capsimage.h"

#if (defined __linux)
#include <dlfcn.h>
void *dlhandle;
char *dlname = "libcapsimage.so";
#endif

#if (defined WIN32)
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#define dlopen(a,b) LoadLibrary(a)
#define dlsym(a,b) (void*)GetProcAddress(a,b)
#define dlclose(a) FreeLibrary(a)
HINSTANCE dlhandle;
char *dlname = "CAPSImg.dll";
#endif

#if (defined AMIGA)
#include <proto/capsimage.h>
#include <proto/exec.h>
struct MsgPort *CapsMsgPort;
struct IORequest *CapsIORequest;
struct Library *CapsImageBase;
char *dlname = "capsimage.device";
#endif

#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <strings.h>
#include <dirent.h>

char *in_name;
DIR *in_dirp;
FILE *in_fp;
char is_dir;

/* detect emerald mine cave version and pad to a common size. this preserves the
 * exact cave from each disk. converting the cave to a common format is done at
 * load time in emerald. by keeping the exact bytes of the original cave,
 * changes to the game wont require the original cave files to be re-converted.
 * it also compresses a tiny bit better.
 */

int filter(unsigned char buf[2172], unsigned int len)
{
	if(len >= 2172 && (buf[2106] == 255 && buf[2107] == 54 && buf[2108] == 48 && buf[2109] == 48)) {
		return(0);
	}
	if(len >= 2110 && (buf[2106] == 255 && buf[2107] == 53 && buf[2108] == 48 && buf[2109] == 48)) {
		memset(buf + 2110, 0, 62);
		return(0);
	}
	if(len >= 2106 && (buf[1983] == 116 || buf[2047] == 116)) {
		memset(buf + 2106, 0, 66);
		return(0);
	}
	if(len >= 2106 && (buf[1983] == 27 || buf[2047] == 219)) {
		unsigned int i;
		for(i = 2106; i < 2172; i++) buf[i] = (0 + 17) ^ (i * 7 + 101);
		return(0);
	}
	return(1);
}

/* create a file, and parent directories if necessary */

#ifdef __MINGW32__
#define mkdir(a,b) mkdir(a)
#endif

FILE *creat_mkdir(char *path)
{
	FILE *fp;
	fp = fopen(path, "wb");
	if(fp == 0) {
		int i;
		for(i = 0; path[i]; i++) {
			if(path[i] == '/') {
				path[i] = 0;
				mkdir(path, 511);
				path[i] = '/';
			}
		}
		fp = fopen(path, "wb");
	}
	return(fp);
}

/* create a string from a path name and file name */

char *alloc_strcat(char *path_name, char *file_name)
{
	int path_strlen = strlen(path_name), file_strlen = strlen(file_name);
	char *ptr = malloc(path_strlen + file_strlen + 1);
	if(ptr) sprintf(ptr, "%s%s", path_name, file_name);
	return(ptr);
}

/* maintain a vector of filenames for easy searching. a filename is
 * searched in this list, rather than trying to open directly, because the
 * amiga filesystem is case insensitive but the host filesystem may not be.
 */

struct fileinfo {
	char *name;
	unsigned int inode;
};

struct fileinfo *filelist;
int filelist_len;
int filelist_alloc;

int add_filelist(struct fileinfo *fileinfo)
{
	struct fileinfo *node;
	if(filelist_alloc <= filelist_len) {
		struct fileinfo *ptr = filelist;
		int size = 50;
		while(size <= filelist_len) size = size * 8 / 5;
		if(ptr = realloc(ptr, size * sizeof(*ptr)), ptr == 0) {
			fprintf(stderr, "%s: %s (%d): %s\n", name, "failed to allocate memory for file list", size * sizeof(*ptr), strerror(errno));
			return(1);
		}
		filelist = ptr;
		filelist_alloc = size;
	}
	node = &filelist[filelist_len++];
	memcpy(node, fileinfo, sizeof(*node));
	return(0);
}

#ifdef __SASC
#define strcasecmp(a,b) stricmp(a,b)
#endif

static int sort_func(const void *aa, const void *bb)
{
	const struct fileinfo *a = aa, *b = bb;
	return(strcasecmp(a->name, b->name));
}
void sort_filelist(void)
{
	qsort(filelist, filelist_len, sizeof(*filelist), sort_func);
}

static int search_func(const void *aa, const void *bb)
{
	const char *a = aa;
	const struct fileinfo *b = bb;
	return(strcasecmp(a, b->name));
}
struct fileinfo *search_filelist(char *key)
{
	return(bsearch(key, filelist, filelist_len, sizeof(*filelist), search_func));
}

/* create a deflated output stream */

struct outfile {
	char *name;
	FILE *fp;
	char is_open, is_err;
	struct zip *zip_state;
	unsigned char *outend, *out;
	unsigned char buf[4096];
};

extern const char *zip(struct zip *, unsigned char **indin, unsigned char **indout, unsigned char *inend, unsigned char *outend);
extern void zip_init(struct zip **);
extern void zip_free(struct zip **);

int zcreate(struct outfile *outfile)
{
	outfile->fp = creat_mkdir(outfile->name);
	if(outfile->fp == 0) {
		fprintf(stderr, "%s: %s: %s: %s\n", name, outfile->name, "failed to create file", strerror(errno));
		return(1);
	}
	zip_init(&outfile->zip_state);
	outfile->outend = outfile->buf + sizeof(outfile->buf);
	outfile->out = outfile->buf;
	return(0);
}

int zwrite(struct outfile *outfile, unsigned char *buf, unsigned int len)
{
	unsigned char *inend = 0, *in = 0;
	if(buf) { in = buf; inend = buf + len; }
	for(;;) {
		const char *err = zip(outfile->zip_state, &in, &outfile->out, inend, outfile->outend);
		if(err) {
			fprintf(stderr, "%s: %s: %s: %s\n", name, outfile->name, "failed to zip file", err);
			return(1);
		}
		if(in == inend && in) break;

		if(outfile->out == outfile->outend || in == 0) {
			int len = fwrite(outfile->buf, 1, outfile->out - outfile->buf, outfile->fp);
			if(ferror(outfile->fp)) {
				fprintf(stderr, "%s: %s: %s: %s\n", name, outfile->name, "failed to write file", strerror(errno));
				return(1);
			}
			outfile->out = outfile->buf; outfile->outend = outfile->buf + len;
		}
		if(outfile->out == outfile->outend) break;
	}
	return(0);
}

void zflush(struct outfile *outfile)
{
	if(outfile->is_open) zwrite(outfile, 0, 0);

	if(outfile->fp) if(fclose(outfile->fp)) {
		fprintf(stderr, "%s: %s: %s: %s\n", name, outfile->name, "failed to flush file", strerror(errno));
	}
	zip_free(&outfile->zip_state);
}

/* uncompress a gzip or dms file */

unsigned char *blk_buf;
unsigned long blk_len;
unsigned long blk_alloc;

struct unzip *unzip_state;
struct undms *undms_state;

extern const char *unzip(struct unzip *, unsigned char **indin, unsigned char **indout, unsigned char *inend, unsigned char *outend);
extern void unzip_init(struct unzip **);
extern void unzip_free(struct unzip **);

extern const char *undms(struct undms *, unsigned char **indin, unsigned char **indout, unsigned char *inend, unsigned char *outend);
extern void undms_init(struct undms **);
extern void undms_free(struct undms **);

unsigned char unzip_buf[4096];

void unzip_file(void)
{
	unsigned char *inend = unzip_buf + sizeof(unzip_buf), *in = inend;
	unsigned char *outend, *out;

	int type = 0;

	unzip_init(&unzip_state);
	undms_init(&undms_state);

	for(;;) {

/* create output buffer */

		if(blk_alloc <= blk_len) {
			unsigned char *ptr = blk_buf;
			unsigned long size = 50000;
			while(size <= blk_len) size = size * 8 / 5;
			if(ptr = realloc(ptr, size), ptr == 0) {
				fprintf(stderr, "%s: %s: %s: %s\n", name, in_name, "failed to allocate memory for adf file", strerror(errno));
				break;
			}
			blk_buf = ptr;
			blk_alloc = size;
		}
		out = blk_buf + blk_len; outend = blk_buf + blk_alloc;

/* read from a file or pipe */

		if(in == inend && in) {
			int len = fread(unzip_buf, 1, in - unzip_buf, in_fp);
			in = unzip_buf; inend = unzip_buf + len;

			if(ferror(in_fp)) {
				fprintf(stderr, "%s: %s: %s: %s\n", name, in_name, "failed to read file", strerror(errno));
				break;
			}
		}
		if(in == inend) { in = 0; inend = 0; }

/* try gzip */

		if(type == 0 || type == 3) {
			const char *err = unzip(unzip_state, &in, &out, inend, outend);
			if(type == 0) {
				if(err) {
					in = unzip_buf; /* rewind */
				} else {
					fputs(" [gzip]", stdout); fflush(stdout);
					type = 3;
				}
			} else if(err) {
				fprintf(stderr, "%s: %s: %s: %s\n", name, in_name, "failed to unzip file", err);
				break;
			}
		}

/* try dms */

		if(type == 0 || type == 2) {
			const char *err = undms(undms_state, &in, &out, inend, outend);
			if(type == 0) {
				if(err) {
					in = unzip_buf; /* rewind */
				} else {
					fputs(" [dms]", stdout); fflush(stdout);
					type = 2;
				}
			} else if(err) {
				fprintf(stderr, "%s: %s: %s: %s\n", name, in_name, "failed to undms file", err);
				break;
			}
		}

/* try copy */

		if(type == 0 || type == 1) {
			if(in) {
				int len = inend - in;
				if(len > outend - out) len = outend - out;
				memcpy(out, in, len);
				in += len; out += len;
			}
			type = 1;
		}

/* update output buffer position */

		blk_len = out - blk_buf;
		if(out != outend && in == 0) break;
	}

	unzip_free(&unzip_state);
	undms_free(&undms_state);
}

/* convert an ipf file to adf */

#ifdef WITH_IPF

#define TRACKSIZE 5632

void amigadosmfm(unsigned char track[TRACKSIZE], unsigned char *mfm, unsigned int len)
{
	unsigned int ptr = 0;
	unsigned int sector;
	for(;;) {
/* sync word */
		for(;;) {
			if(ptr + 2 > len) return;
			if(mfm[ptr] == 0x44 && mfm[ptr + 1] == 0x89) break;
			ptr += 2;
		}
		for(;;) {
			if(ptr + 2 > len) return;
			if(!(mfm[ptr] == 0x44 && mfm[ptr + 1] == 0x89)) break;
			ptr += 2;
		}
/* sector header */
		if(ptr + 56 > len) return;
		sector = (mfm[ptr + 2] & 0x55) << 1 | (mfm[ptr + 6] & 0x55);
		ptr += 56;
/* sector data */
		if(ptr + 1024 > len) return;
		if(sector < TRACKSIZE / 512) {
			unsigned char *out = track + sector * 512, *outend = out + 512;
			unsigned char *aa = mfm + ptr, *bb = aa + 512;
			while(out < outend) {
				*out++ = (*aa++ & 0x55) << 1 | (*bb++ & 0x55);
			}
		}
		ptr += 1024;
	}
}

unsigned char *ipf_buf;
unsigned long ipf_len;
unsigned long ipf_alloc;

void unipf_file(void)
{
#if (defined AMIGA)
	if(CapsMsgPort = CreateMsgPort(), CapsMsgPort) {
	if(CapsIORequest = CreateIORequest(CapsMsgPort, sizeof(*CapsIORequest)), CapsIORequest) {
	if(OpenDevice(dlname, 0, CapsIORequest, 0) == 0) {
	CapsImageBase = CapsIORequest->io_Device;
	{
#else
	if(dlhandle = dlopen(dlname, RTLD_NOW), dlhandle) {
	CapsLong(*CAPSInit)(void) = dlsym(dlhandle, "CAPSInit");
	CapsLong(*CAPSAddImage)(void) = dlsym(dlhandle, "CAPSAddImage");
	CapsLong(*CAPSLockImageMemory)(CapsLong, CapsUByte *, CapsULong, CapsULong) = dlsym(dlhandle, "CAPSLockImageMemory");
	CapsLong(*CAPSGetImageInfo)(struct CapsImageInfo *, CapsLong) = dlsym(dlhandle, "CAPSGetImageInfo");
	CapsLong(*CAPSLockTrack)(struct CapsTrackInfo *, CapsLong, CapsULong, CapsULong, CapsULong) = dlsym(dlhandle, "CAPSLockTrack");
	CapsLong(*CAPSUnlockTrack)(CapsLong, CapsULong, CapsULong) = dlsym(dlhandle, "CAPSUnlockTrack");
	CapsLong(*CAPSUnlockImage)(CapsLong) = dlsym(dlhandle, "CAPSUnlockImage");
	CapsLong(*CAPSRemImage)(CapsLong) = dlsym(dlhandle, "CAPSRemImage");
	CapsLong(*CAPSExit)(void) = dlsym(dlhandle, "CAPSExit");
	if(CAPSInit && CAPSAddImage && CAPSLockImageMemory && CAPSGetImageInfo && CAPSLockTrack && CAPSUnlockTrack && CAPSUnlockImage && CAPSRemImage && CAPSExit) {
#endif

	CapsLong id;
	struct CapsImageInfo cii;
	struct CapsTrackInfo cti;
	CapsULong cylinder;
	CapsULong head;

	if(CAPSInit() == imgeOk) {
	if(id = CAPSAddImage(), id >= 0) {
	if(CAPSLockImageMemory(id, blk_buf, blk_len, DI_LOCK_MEMREF) == imgeOk) {
	if(CAPSGetImageInfo(&cii, id) == imgeOk) {

	fputs(" [ipf]", stdout); fflush(stdout);
	for(cylinder = cii.mincylinder; cylinder <= cii.maxcylinder; cylinder++) {
		for(head = cii.minhead; head <= cii.maxhead; head++) {
			if(CAPSLockTrack(&cti, id, cylinder, head, DI_LOCK_INDEX) == imgeOk) {
				if(ipf_alloc <= ipf_len + TRACKSIZE) {
					unsigned char *ptr = ipf_buf;
					unsigned long size = 50000;
					while(size <= ipf_len + TRACKSIZE) size = size * 8 / 5;
					if(ptr = realloc(ptr, size), ptr == 0) {
						fprintf(stderr, "%s: %s: %s: %s\n", name, in_name, "failed to allocate memory for ipf file", strerror(errno));
						goto fail;
					}
					ipf_buf = ptr;
					ipf_alloc = size;
				}
				memset(ipf_buf + ipf_len, 0, TRACKSIZE);
				amigadosmfm(ipf_buf + ipf_len, cti.trackbuf, cti.tracklen);
				ipf_len += TRACKSIZE;
fail:
				CAPSUnlockTrack(id, cylinder, head);
			}
		}
	}

	}
	CAPSUnlockImage(id);
	}
	CAPSRemImage(id);
	}
	CAPSExit();
	}

#if ! (defined AMIGA)
	}
	dlclose(dlhandle);
	}
#else
	}
	CloseDevice(CapsIORequest);
	}
	DeleteIORequest(CapsIORequest);
	}
	DeleteMsgPort(CapsMsgPort);
	}
#endif

	if(ipf_buf) {
		if(blk_buf) free(blk_buf);
		blk_buf = ipf_buf;
		blk_len = ipf_len;
		blk_alloc = ipf_alloc;
	}
}

#endif /* WITH_IPF */

/* parse an amiga disk file */

#define BSIZE 512
#define NUMBLOCKS 1760

unsigned char *adf_blk[NUMBLOCKS];
char *adf_path[NUMBLOCKS]; /* also used as a bitmap check */
unsigned int adf_fifo[NUMBLOCKS];

enum { TYPE_ROOT, TYPE_DIR, TYPE_FILE, TYPE_LINK, };

#define BE32(x) (mem = (x), mem[0] << 24 | mem[1] << 16 | mem[2] << 8 | mem[3])

void parse_adf(void)
{
	unsigned int push = 0, pop = 0;

	adf_fifo[push++] = 880; adf_path[880] = ""; /* start at the root block */

	while(pop < push) {
		unsigned int self_num;
		unsigned char *self_blk;
		char *self_path;

		unsigned int cnt;
		unsigned char *mem;

		unsigned long t1, t2, sum;
		unsigned long x;
		int type;

		self_num = adf_fifo[pop++];

		if(self_num >= NUMBLOCKS) { fprintf(stderr, "%s: %s: %s (%u)\n", name, in_name, "bad block number", self_num); continue; }
		if(self_blk = adf_blk[self_num], self_blk == 0) { fprintf(stderr, "%s: %s: block %u: %s\n", name, in_name, self_num, "missing block"); continue; }
		if(self_path = adf_path[self_num], self_path == 0) { fprintf(stderr, "%s: %s: block %u: %s\n", name, in_name, self_num, "missing directory"); continue; }

		t1 = BE32(self_blk + 0);
		t2 = BE32(self_blk + (BSIZE - 4));
		if(t1 == 0x2 && t2 == 0x1) {
			type = TYPE_ROOT;
		} else if(t1 == 0x2 && t2 == 0x2) {
			type = TYPE_DIR;
		} else if(t1 == 0x2 && t2 == 0xfffffffd) {
			type = TYPE_FILE;
		} else if(t1 == 0x2 && (t2 == 0xfffffffc || t2 == 0x3 || t2 == 0x4)) {
			type = TYPE_LINK;
		} else {
			fprintf(stderr, "%s: %s: block %u: %s (0x%lx:0x%lx)\n", name, in_name, self_num, "bad block type", t1, t2);
			continue;
		}

		sum = 0;
		for(cnt = 0; cnt < BSIZE; cnt += 4) sum += BE32(self_blk + cnt);
		sum &= 0xffffffff;
		if(sum != 0) { fprintf(stderr, "%s: %s: block %u: %s (0x%lx)\n", name, in_name, self_num, "bad checksum", sum); }

		if(type == TYPE_DIR || type == TYPE_FILE || type == TYPE_LINK) {
			x = BE32(self_blk + (BSIZE - 16));
			if(x) {
				if(x < NUMBLOCKS && adf_path[x] == 0) {
					adf_fifo[push++] = x; adf_path[x] = self_path;
				} else {
					fprintf(stderr, "%s: %s: block %u: %s (0x%lx)\n", name, in_name, self_num, "bad hash chain", x);
				}
			}
		}
		if(type == TYPE_DIR || type == TYPE_FILE) {
			struct fileinfo fileinfo;
			char file_name[48];
			memset(file_name, 0, sizeof(file_name));

			cnt = self_blk[BSIZE - 80];
			if(cnt >= 1 && cnt <= 30) strncpy(file_name, (char *)(self_blk + (BSIZE - 79)), cnt);
			if(*file_name == 0) {
				sprintf(file_name, "%u~", self_num);
				fprintf(stderr, "%s: %s: block %u: %s (%s)\n", name, in_name, self_num, "bad name", self_path);
			}
			if(type == TYPE_DIR) strcat(file_name, "/");

			fileinfo.name = alloc_strcat(self_path, file_name);
			fileinfo.inode = self_num;
			if(fileinfo.name == 0) {
				perror(0);
			} else if(add_filelist(&fileinfo)) {
				free(fileinfo.name);
			} else {
				self_path = fileinfo.name;
			}
		}
		if(type == TYPE_DIR || type == TYPE_ROOT) {
			for(cnt = 24; cnt < (BSIZE - 200); cnt += 4) {
				x = BE32(self_blk + cnt);
				if(x) {
					if(x < NUMBLOCKS && adf_path[x] == 0) {
						adf_fifo[push++] = x; adf_path[x] = self_path;
					} else {
						fprintf(stderr, "%s: %s: block %u: %s (0x%lx)\n", name, in_name, self_num, "bad directory hash", x);
					}
				}
			}
		}
	}
}

/* dump a file from an amiga disk file */

void dump_adf(struct fileinfo *fileinfo, unsigned char **indout, unsigned char *outend)
{
	unsigned char *out = *indout;

	char is_ofs = 0; /* determined from the first data block */

	unsigned int self_num, ext_num, data_num;
	unsigned char *self_blk, *ext_blk, *data_blk;
	char *self_path;

	unsigned int cnt;
	unsigned char *mem;

	unsigned long remain;
	unsigned int index, hash;

	unsigned long t1, t2, sum;

	if(self_num = fileinfo->inode, self_num >= NUMBLOCKS) return;
	if(self_blk = adf_blk[self_num], self_blk == 0) return;
	if(self_path = adf_path[self_num], self_path == 0) return;

	t1 = BE32(self_blk + 0);
	t2 = BE32(self_blk + (BSIZE - 4));
	if(!(t1 == 0x2 && t2 == 0xfffffffd)) return; /* can only extract files */

	ext_num = self_num; ext_blk = self_blk; /* treat the file header as a file extension block */
	index = 1; hash = (BSIZE - 200);

	remain = BE32(self_blk + (BSIZE - 188));

	while(remain != 0 && out != outend) {

		if(hash <= 24) {
			ext_num = BE32(ext_blk + (BSIZE - 8));
			if(ext_num == 0) { fprintf(stderr, "%s: %s: %s: %s\n", name, in_name, fileinfo->name, "unexpected eof"); break; }

			if(ext_num >= NUMBLOCKS) { fprintf(stderr, "%s: %s: %s: %s (%u)\n", name, in_name, fileinfo->name, "bad file extension block number", ext_num); break; }
			if(ext_blk = adf_blk[ext_num], ext_blk == 0) { fprintf(stderr, "%s: %s: %s: block %u: %s\n", name, in_name, fileinfo->name, ext_num, "missing file extension block"); break; }

			if(adf_path[ext_num] != 0) { fprintf(stderr, "%s: %s: %s: block %u: %s\n", name, in_name, fileinfo->name, ext_num, "duplicate file extension block"); break; }
			adf_path[ext_num] = self_path;

			t1 = BE32(ext_blk + 0);
			t2 = BE32(ext_blk + (BSIZE - 4));
			if(!(t1 == 0x10 && t2 == 0xfffffffd)) { fprintf(stderr, "%s: %s: %s: block %u: %s (0x%lx:0x%lx)\n", name, in_name, fileinfo->name, ext_num, "bad file extension block type", t1, t2); break; }

			sum = 0;
			for(cnt = 0; cnt < BSIZE; cnt += 4) sum += BE32(ext_blk + cnt);
			sum &= 0xffffffff;
			if(sum != 0) { fprintf(stderr, "%s: %s: %s: block %u: %s (0x%lx)\n", name, in_name, fileinfo->name, ext_num, "bad file extension checksum", sum); }

			hash = (BSIZE - 200);
		}
		hash -= 4;

		if(data_num = BE32(ext_blk + hash), data_num >= NUMBLOCKS) { fprintf(stderr, "%s: %s: %s: %s (%u)\n", name, in_name, fileinfo->name, "bad file data block number", data_num); break; }
		if(data_blk = adf_blk[data_num], data_blk == 0) { fprintf(stderr, "%s: %s: %s: block %u: %s\n", name, in_name, fileinfo->name, data_num, "missing file data block"); break; }

		if(index == 1) {
			unsigned long type = BE32(data_blk + 0);
			unsigned long header_key = BE32(data_blk + 4);
			unsigned long seq_num = BE32(data_blk + 8);
			sum = 0;
			for(cnt = 0; cnt < BSIZE; cnt += 4) sum += BE32(data_blk + cnt);
			sum &= 0xffffffff;
			if(type == 0x8 && header_key == self_num && seq_num == index && sum == 0) is_ofs = 1;
		}
		index++;

		if(is_ofs) {
			mem = data_blk + 24; cnt = (BSIZE - 24);
		} else {
			mem = data_blk; cnt = BSIZE;
		}
		if(cnt > remain) cnt = remain;
		if(cnt > outend - out) cnt = outend - out;
		memcpy(out, mem, cnt);
		out += cnt; remain -= cnt;
	}

	*indout = out;
}

/* read filenames and caves from an amiga disk file */

int scan_file(void)
{
	int i;

	is_dir = 0;

	fputs(in_name, stdout); fflush(stdout);
	unzip_file();
#ifdef WITH_IPF
	unipf_file();
#endif
	for(i = 0; i < NUMBLOCKS && i < blk_len / BSIZE; i++) adf_blk[i] = &blk_buf[i * BSIZE];
	fputs(" [adf]", stdout); fflush(stdout);
	parse_adf();

	return(0);
}

int read_file(struct fileinfo *fileinfo, unsigned char buf[2172])
{
	unsigned char *out = buf, *outend = buf + 2172;
	dump_adf(fileinfo, &out, outend);
	return(filter(buf, out - buf));
}

/* read filenames and caves from a host directory */

char *dir_prefix;

int scan_dir(void)
{
	int in_strlen = strlen(in_name);
	struct dirent *dp;

	is_dir = 1;

	dir_prefix = malloc(in_strlen + 2);
	if(dir_prefix == 0) { perror(0); return(1); }

	strcpy(dir_prefix, in_name);
	if(in_strlen > 0 && in_name[in_strlen - 1] != '/') strcat(dir_prefix, "/");

	fputs(dir_prefix, stdout); fflush(stdout);
	fputs(" [dir]", stdout); fflush(stdout);

	while(dp = readdir(in_dirp), dp) {
		struct fileinfo fileinfo;
		fileinfo.name = strdup(dp->d_name);
		fileinfo.inode = -1;
		if(fileinfo.name == 0) {
			perror(0);
		} else if(add_filelist(&fileinfo)) {
			free(fileinfo.name);
		}
	}
	return(0);
}

int read_dir(struct fileinfo *fileinfo, unsigned char buf[2172])
{
	unsigned int len = 0;
	FILE *fp = 0;

	char *file_name = alloc_strcat(dir_prefix, fileinfo->name);
	if(file_name == 0) { perror(0); return(1); }

	if(fp = fopen(file_name, "rb"), fp == 0) {
		fprintf(stderr, "%s: %s: %s: %s\n", name, file_name, "failed to open file", strerror(errno));
	} else {
		len = fread(buf, 1, 2172, fp);
		if(ferror(fp)) {
			fprintf(stderr, "%s: %s: %s: %s\n", name, file_name, "failed to read file", strerror(errno));
		}
		if(fclose(fp)) {
			fprintf(stderr, "%s: %s: %s: %s\n", name, file_name, "failed to close file", strerror(errno));
		}
	}
	free(file_name);

	return(filter(buf, len));
}

/* read a cave from a host directory or an amiga disk file */

int read_cave(struct fileinfo *fileinfo, unsigned char buf[2172])
{
	return(is_dir ? read_dir(fileinfo, buf) : read_file(fileinfo, buf));
}

/* write a cave to a deflated stream */

int write_cave(struct outfile *outfile, unsigned char buf[2172])
{
	if(!outfile->is_err) {
		if(outfile->is_open == 0) if(zcreate(outfile) == 0) outfile->is_open = 1;
		if(outfile->is_open == 0) outfile->is_err = 1;
	}
	if(!outfile->is_err) {
		if(zwrite(outfile, buf, 2172)) outfile->is_err = 1;
	}
	return(outfile->is_err);
}

/* read and write a single player and teamwork cave */

struct outfile out_solo, out_team;

unsigned char solo_buf[2172], team_buf[2172];

int solo_cave_count, team_cave_count;

int copy_solo_team(struct fileinfo *soloinfo, struct fileinfo *teaminfo)
{
	char got_solo = 0, got_team = 0;
	if(soloinfo) if(read_cave(soloinfo, solo_buf) == 0) got_solo = 1;
	if(teaminfo) if(read_cave(teaminfo, team_buf) == 0) got_team = 1;

#if 0
	if(got_solo && !got_team) { memcpy(team_buf, solo_buf, 2172); got_team = 1; }
#endif

	if(got_solo) if(write_cave(&out_solo, solo_buf) == 0) solo_cave_count++;
	if(got_team) if(write_cave(&out_team, team_buf) == 0) team_cave_count++;

	return(!(got_solo || got_team));
}

/* search the file list for emerald mine cave files
 *
 * do a scan of all the possible filenames for each version of emerald mine.
 * if a filename matches, and it is a valid cave, it is assumed that the
 * rest of the files are all from the same version of emerald mine.
 *
 * eg. if 'pla/0' is a valid cave file, only 'pla/?' will be considered.
 *
 * the search order is 'cave/?', then 'pla/?', and finally 'a?'.
 */

int search_caves(void)
{
	char *path_name;
	char file_name[32];
	unsigned int level;
	char found;

	path_name = is_dir ? "" : "cave/";
	found = 0;
	for(level = 0; level <= 80; level++) {
		struct fileinfo *soloinfo = 0, *teaminfo = 0;
		sprintf(file_name, "%s%02u%c", path_name, level, 's');
		soloinfo = search_filelist(file_name);
		sprintf(file_name, "%s%02u%c", path_name, level, 't');
		teaminfo = search_filelist(file_name);
		if(found || soloinfo || teaminfo) {
			if(copy_solo_team(soloinfo, teaminfo) == 0) {
				if(found == 0) { fputs(" [cave/*]", stdout); fflush(stdout); }
				found = 1;
			}
		}
	}
	if(found) return(0);

	path_name = is_dir ? "" : "pla/";
	found = 0;
	for(level = 0; level <= 80; level++) {
		struct fileinfo *soloinfo = 0, *teaminfo = 0;
		sprintf(file_name, "%s%u", path_name, level);
		soloinfo = search_filelist(file_name);
		if(level % 4 == 0) {
			sprintf(file_name, "%s%u", path_name, level / 4 + 81);
			teaminfo = search_filelist(file_name);
		}
		if(found || soloinfo || teaminfo) {
			if(copy_solo_team(soloinfo, teaminfo) == 0) {
				if(found == 0) { fputs(" [pla/*]", stdout); fflush(stdout); }
				found = 1;
			}
		}
	}
	if(found) return(0);

	path_name = is_dir ? "" : "";
	found = 0;
	for(level = 0; level <= 80; level++) {
		struct fileinfo *soloinfo = 0, *teaminfo = 0;
		sprintf(file_name, "%s%c%c%c", path_name, 'a', 'a' + (level / 10), '0' + (level % 10));
		soloinfo = search_filelist(file_name);
		if(level % 4 == 0) {
			sprintf(file_name, "%s%c%c%c", path_name, 'a', 'a' + ((level / 4 + 81) / 10), '0' + ((level / 4 + 81) % 10));
			teaminfo = search_filelist(file_name);
		}
		if(found || soloinfo || teaminfo) {
			if(copy_solo_team(soloinfo, teaminfo) == 0) {
				if(found == 0) { fputs(" [a*]", stdout); fflush(stdout); }
				found = 1;
			}
		}
	}
	if(found) return(0);

	found = 0;
	for(level = 0; level <= 80; level++) {
		struct fileinfo *soloinfo = 0, *teaminfo = 0;
		sprintf(file_name, "part%c/%u", (level > 50 ? '2' : '1'), level);
		soloinfo = search_filelist(file_name);
		if(level % 4 == 0) {
			sprintf(file_name, "part%c/%u", '2', level / 4 + 81);
			teaminfo = search_filelist(file_name);
		}
		if(found || soloinfo || teaminfo) {
			if(copy_solo_team(soloinfo, teaminfo) == 0) {
				if(found == 0) { fputs(" [plaex]", stdout); fflush(stdout); }
				found = 1;
			}
		}
	}
	if(found) return(0);

	fputs(" no caves found", stdout); fflush(stdout);
	return(1);
}

int main(int argc, char **argv)
{
	if(argc < 2) {
		fputs("cavegrab 2.1 © David Tritscher"
		" [gzip]"
		" [dms]"
#ifdef WITH_IPF
		" [ipf]"
#endif
		"\n\n", stderr);
		fputs("usage: cavegrab file [dir/|file.adf]\n\n", stderr);
		fputs("create X11 Emerald Mine caves from a directory or amiga disk file\n", stderr);
		return(1);
	}

	if(argc > 1) {
		out_solo.name = alloc_strcat(argv[1], ".S");
		out_team.name = alloc_strcat(argv[1], ".T");
	}
	if(out_solo.name == 0 || out_team.name == 0) { perror(0); goto fail; }

	if(argc > 2) {
		in_name = argv[2];
		if(in_dirp = opendir(in_name), in_dirp) {
			scan_dir();
			closedir(in_dirp);
		} else if(in_fp = fopen(in_name, "rb"), in_fp) {
			scan_file();
			fclose(in_fp);
		} else {
			fprintf(stderr, "%s: %s: %s: %s\n", name, in_name, "failed to open file", strerror(errno));
			goto fail;
		}
	} else {
		in_name = "stdin";
		in_fp = stdin;
		scan_file();
	}

	sort_filelist();

	search_caves();

	if(solo_cave_count) { fprintf(stdout, " %s [%d]", out_solo.name, solo_cave_count); fflush(stdout); }
	if(team_cave_count) { fprintf(stdout, " %s [%d]", out_team.name, team_cave_count); fflush(stdout); }
	fputs("\n", stdout); fflush(stdout);

fail:
	zflush(&out_solo);
	zflush(&out_team);

	if(out_solo.name) free(out_solo.name);
	if(out_team.name) free(out_team.name);

	if(dir_prefix) free(dir_prefix);

	if(blk_buf) free(blk_buf);

	if(filelist) {
		int i;
		for(i = 0; i < filelist_len; i++) if(filelist[i].name) free(filelist[i].name);
		free(filelist);
	}

	return(0);
}
