/* unlimited distribution */

/* 2010-10-10 16:55:54 */

const char *name = "cavepack";

#define LENGTH 250000
#define SPLIT 250000
#define TRY 8

#include <zlib.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef unsigned long ulong;

/* http://www.cs.mu.oz.au/~alistair/abstracts/wads95.html
 *
 * Alistair Moffat, alistair@cs.mu.oz.au
 * Jyrki Katajainen, jyrki@diku.dk
 */
static void minimum_redundancy(long A[288], int size, int limit)
{
	int root, next, leaf, avbl, dpth, used;

	root = size - 1; next = size - 2; leaf = size - 3;
	A[root] += A[next];
	while(next > 0) {
		if(leaf < 0 || (root > next && A[root] < A[leaf])) {
			A[next] = A[root]; A[root] = next; root--;
		} else {
			A[next] = A[leaf]; leaf--;
		}
		if(leaf < 0 || (root > next && A[root] < A[leaf])) {
			A[next] += A[root]; A[root] = next; root--;
		} else {
			A[next] += A[leaf]; leaf--;
		}
		next--;
	}

	A[1] = 0;
	for(next = 2; next < size; next++) { A[next] = A[A[next]] + 1; }

	root = 1; next = 0; avbl = 1; dpth = 0;
	while(avbl > 0) {
		used = 0;
		while(root < size && A[root] == dpth) { used++; root++; }
		while(avbl > used) { A[next] = dpth; next++; avbl--; }
		dpth += 1; avbl <<= 1;
	}

	while(dpth = A[size - 1], dpth > limit) {
		for(leaf = size - 2; A[leaf - 1] == dpth; leaf--);
		for(next = leaf; A[next - 1] == dpth - 1; next--);
		A[leaf + 1]--; A[leaf] = A[next];
		A[next - 1]++; A[next] = A[next - 1];
	}
}

static void merge_sort(ulong sum[288], long in[288], long out[288], int size)
{
	int mid = size / 2;
	if(mid > 1) merge_sort(sum, out, in, mid);
	if(size - mid > 1) merge_sort(sum, out + mid, in + mid, size - mid);
	{
	int n = 0, left = 0, right = mid;
	while(left < mid && right < size) {
		if(sum[in[left]] < sum[in[right]]) {
			out[n++] = in[right++];
		} else {
			out[n++] = in[left++];
		}
	}
	while(left < mid) out[n++] = in[left++];
	while(right < size) out[n++] = in[right++];
	}
}

static int huffman(ulong sum[288], uchar bit[288], int length, int limit)
{
	long A[288];
	long B[288];
	int size = 0, used = 0;
	int n;

	for(n = 0; n < length; n++) if(sum[n] != 0) A[size++] = n;
	if(size > 0) used = A[size - 1];

	if(size < 2) {
		bit[used] = 0;
		bit[0] = 1; if(used == 0) used = 1; bit[used] = 1; /* special case */
	} else {
		for(n = 0; n < size; n++) B[n] = A[n];
		merge_sort(sum, A, B, size);
		for(n = 0; n < size; n++) A[n] = sum[B[n]];
		minimum_redundancy(A, size, limit);
		for(n = 0; n < size; n++) bit[B[n]] = A[n];
	}
	return(used + 1);
}

/* static huffman */

static ulong rev(ulong a)
{
	a = (a & 0x0000ffff) << 16 | (a >> 16 & 0x0000ffff);
	a = (a & 0x00ff00ff) << 8 | (a >> 8 & 0x00ff00ff);
	a = (a & 0x0f0f0f0f) << 4 | (a >> 4 & 0x0f0f0f0f);
	a = (a & 0x33333333) << 2 | (a >> 2 & 0x33333333);
	a = (a & 0x55555555) << 1 | (a >> 1 & 0x55555555);
	return(a);
}

int table(ulong *table, uchar *length, uint size)
{
	ulong avbl = 1, code = 0;
	uint dpth = 0, cnt;

	while(avbl > 0) {
		for(cnt = 0; cnt < size; cnt++) if(length[cnt] == dpth) {

			table[cnt] = rev(code); code++;
			avbl--;
		}
		code *= 2;
		dpth += 1; avbl <<= 1;
	}
	if(avbl != 0) return(1);

	return(0);
}

/* crc-32-ieee 802.3 */

ulong crc_table[256];

uchar len_table[256];
uchar ptr_table[32768];

void crc_calc(ulong *indcrc, uchar *in, uint size)
{
	ulong crc = *indcrc;
	crc = ~crc & 0xffffffff;
	while(size > 0) {
		crc = crc_table[*in++ ^ (crc & 255)] ^ (crc >> 8);
		size--;
	}
	crc = ~crc & 0xffffffff;
	*indcrc = crc;
}

void table_init(void)
{
	uint a, b;
	for(a = 0; a < 256; a++) {
		ulong crc = a;
		for(b = 0; b < 8; b++) {
			crc = ((crc & 1) == 0 ? 0 : 0xedb88320) ^ (crc >> 1);
		}
		crc_table[a] = crc;
	}

	for(a = 0; a < 256; a++) {
		uint bit = 0, mask = a;
		if(mask >= 64) { bit += 16; mask >>= 4; }
		if(mask >= 16) { bit += 8; mask >>= 2; }
		if(mask >= 8) { bit += 4; mask >>= 1; }
		len_table[a] = bit + mask;
	}
	for(a = 0; a < 32768; a++) {
		uint bit = 0, mask = a;
		if(mask >= 512) { bit += 16; mask >>= 8; }
		if(mask >= 32) { bit += 8; mask >>= 4; }
		if(mask >= 8) { bit += 4; mask >>= 2; }
		if(mask >= 4) { bit += 2; mask >>= 1; }
		ptr_table[a] = bit + mask;
	}
	len_table[255] = 28;
}

const uchar hdr_order[19] = { 16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15, };

/* original data */

uchar dictionary[LENGTH + 512];

ulong length;
ulong crc;

/* find strings */

ulong search_hash[65536];
ulong search_left[LENGTH];
ulong search_right[LENGTH];

ushort *search_strings[LENGTH];
ushort search_buffer[LENGTH * 12];

void search(void)
{
	memset(search_hash, -1, sizeof(search_hash));

	{ ushort *out = search_buffer;

	{ ulong n;
	for(n = 0; n < length; n++) {
		search_strings[n] = out;

		{ uint hash = (dictionary[n + 0] << 8) + (dictionary[n + 1]);
		ulong node = search_hash[hash];

		ulong *leftptr = search_right + n, *rightptr = search_left + n;
		uint leftlen = 2, rightlen = 2, best = 2;
		for(;;) {
			uint len = (leftlen < rightlen ? leftlen : rightlen);
			ulong ptr = n - node - 1;
			if(node >= LENGTH || ptr >= 32768) break;
			{ uchar *str_a = dictionary + n, *str_b = dictionary + node;
			while(str_a[len] == str_b[len] && len < 258) len++;
			if(best < len) {
				best = len;
				*out++ = best;
				*out++ = ptr;
			}
			if(str_a[len] < str_b[len]) {
				*leftptr = node;
				leftlen = len;
				leftptr = search_left + node;
				node = *leftptr;
			} else {
				*rightptr = node;
				rightlen = len;
				rightptr = search_right + node;
				node = *rightptr;
			} }
		}
		*leftptr = node;
		*rightptr = node;

		search_hash[hash] = n; }

		*out++ = 0;
	} }

	if(out > search_buffer + LENGTH * 12) abort(); }
}

/* cache prices */

uchar body_bit[339];

ulong body_cost;

uchar cache_len[256], cache_ptr[32];

void cache(void)
{
	uint a;

	body_cost = 3;

	memcpy(cache_ptr, body_bit + 257, 31);
	for(a = 4; a < 24; a++) {
		cache_ptr[a + 4] += (a >> 2);
	}
	for(a = 0; a < 256; a++) {
		cache_len[a] = cache_ptr[len_table[a]];
	}

	memcpy(cache_ptr, body_bit + 288, 32);
	for(a = 2; a < 28; a++) {
		cache_ptr[a + 2] += (a >> 1);
	}
}

/* find path */

uint block_left, block_right;

ulong string_left, string_right;

ulong string_cost[LENGTH + 512];
ushort string_len[LENGTH + 512];
ushort string_ptr[LENGTH + 512];

void string(void)
{
	string_left = block_left * SPLIT;
	string_right = block_right * SPLIT;
	if(string_right > length) string_right = length;

	memset(string_cost, -1, sizeof(string_cost));
	string_cost[string_left] = 0;

	{ ulong n;
	for(n = string_left; n < string_right; n++) {
		ulong chrcost = string_cost[n] + body_bit[dictionary[n]];
		if(string_cost[n + 1] > chrcost) {
			string_cost[n + 1] = chrcost;
			string_len[n + 1] = 1;
		}
		{ uint len = 2;
		ushort *in = search_strings[n];
		while(*in != 0) {
			uint best = *in++;
			uint ptr = *in++;
			ulong ptrcost = string_cost[n] + cache_ptr[ptr_table[ptr]];
			while(len < best) {
				len++;
				{ ulong strcost = ptrcost + cache_len[len - 3];
				if(string_cost[n + len] > strcost) {
					string_cost[n + len] = strcost;
					string_len[n + len] = len;
					string_ptr[n + len] = ptr;
				} }
			}
		} }
	} }

	{ ulong n = string_right;
	uint len = string_len[n];
	uint ptr = string_ptr[n];
	while(n > string_left) {
		if(len < 1 || len > n) abort();
		n -= len;
		{ uint next_len = string_len[n]; string_len[n] = len; len = next_len; }
		{ uint next_ptr = string_ptr[n]; string_ptr[n] = ptr; ptr = next_ptr; }
	} }
}

/* count body */

ulong body_sum[339];

void body(void)
{
	memset(body_sum, 0, sizeof(body_sum));

	{ ulong n;
	for(n = string_left; n < string_right; ) {
		uint len = string_len[n];
		uint ptr = string_ptr[n];
		if(len >= 3) {
			len -= 3;
			body_sum[len_table[len] + 257]++;
			body_sum[ptr_table[ptr] + 288]++;
			len += 3;
		} else {
			body_sum[dictionary[n]]++;
		}
		n += len;
	} }
	body_sum[256]++;
}

/* find block price */

void price(void)
{
	uint a;
	for(a = 0; a < 339; a++) {
		body_cost += body_sum[a] * body_bit[a];
	}
	for(a = 4; a < 24; a++) {
		body_cost += body_sum[a + 261] * (a >> 2);
	}
	for(a = 2; a < 28; a++) {
		body_cost += body_sum[a + 290] * (a >> 1);
	}
}

/* dynamic header */

uchar header_buf[320];

uint len_size, ptr_size, hdr_size;

void header(void)
{
	memset(body_bit, 32, 339);

	len_size = huffman(body_sum + 0, body_bit + 0, 288, 15);
	ptr_size = huffman(body_sum + 288, body_bit + 288, 32, 15);

	if(len_size < 257) len_size = 257;
	if(ptr_size < 1) ptr_size = 1;

	body_cost += 5 + 5 + 4;

	memcpy(header_buf, body_bit + 0, len_size);
	memcpy(header_buf + len_size, body_bit + 288, ptr_size);

	{ uint a;
	for(a = 0; a < len_size + ptr_size; ) {
		uint bit = header_buf[a];
		uint rep;
		for(rep = 1; a + rep < len_size + ptr_size; rep++) if(header_buf[a + rep] != bit) break;
		if(bit == 32) {
			bit = 0;
			while(rep >= 3) {
				uint len = (rep > 138 ? 138 : rep);
				if(len >= 11) {
					body_sum[338]++;
					body_cost += 7;
				} else {
					body_sum[337]++;
					body_cost += 3;
				}
				a += len; rep -= len;
			}
		}
		while(rep > 0) {
			body_sum[bit + 320]++;
			a += 1; rep -= 1;
			while(rep >= 3) {
				uint len = (rep > 6 ? 6 : rep);
				body_sum[336]++;
				body_cost += 2;
				a += len; rep -= len;
			}
		}
	} }

	hdr_size = huffman(body_sum + 320, body_bit + 320, 19, 7);

	for(hdr_size = 19; hdr_size > 4; hdr_size--) if(body_bit[hdr_order[hdr_size - 1] + 320] != 32) break;

	body_cost += hdr_size * 3;
}

/* merge blocks */

uint block_size;

ulong block_cost[(LENGTH - 1) / SPLIT + 2];
uint block_path[(LENGTH - 1) / SPLIT + 2];
uchar block_data[(LENGTH - 1) / SPLIT + 2][321];

void pack(void)
{
	block_size = (length + SPLIT - 1) / SPLIT;

	search();

	memset(block_cost, -1, sizeof(block_cost));
	block_cost[0] = 0;

	for(block_left = 0; block_left < block_size; block_left++) {
		for(block_right = block_left; block_right < block_size; ) {
			block_right++;

			block_data[0][320] = 1; /* fixed tables */

			memset(block_data[0] + 0, 8, 144);
			memset(block_data[0] + 144, 9, 112);
			memset(block_data[0] + 256, 7, 24);
			memset(block_data[0] + 280, 8, 8);
			memset(block_data[0] + 288, 5, 32);

			{ uint rep;
			for(rep = 0; rep < TRY; rep++) {

				memcpy(body_bit, block_data[0], 320);

				cache();
				string();
				body();
				if(block_data[0][320] == 2) header();
				price();

				body_cost += block_cost[block_left];
				if(block_cost[block_right] > body_cost) {
					block_cost[block_right] = body_cost;
					block_path[block_right] = block_left;
					memcpy(block_data[block_right], block_data[0], 321);
				}

				block_data[0][320] = 2; /* dynamic tables */

				huffman(body_sum + 0, block_data[0] + 0, 288, 15);
				huffman(body_sum + 288, block_data[0] + 288, 32, 15);
			} }

		}
	}

	{ uint b = block_size;
	uint a = block_path[b];
	while(b > 0) {
		uint c = b; b = a;
		a = block_path[b];
		block_path[b] = c;
	} }
}

/* write blocks */

#define RPOKE() { while(shift <= 24) { ch = code >> shift; shift += 8; if(putc(ch, fp) == EOF) return "write error"; } }
#define RSHIFT(x) { code >>= (x); shift -= (x); }

const char *dump(FILE *fp)
{
	uchar ch;
	uint a;

	{ uchar header_buf[10] = { 31, 139, 8, 0, 0, 0, 0, 0, 0, 255, };
	for(a = 0; a < 10; a++) {
		ch = header_buf[a];
		if(putc(ch, fp) == EOF) return "write error";
	} }

	{ ulong code = 0;
	uint shift = 32;

	for(block_left = 0; block_left < block_size; block_left = block_right) {
		block_right = block_path[block_left];

		memcpy(body_bit, block_data[block_right], 321);

		RSHIFT(1); code += (block_right == block_size ? 1 : 0) << 31;
		RSHIFT(2); code += body_bit[320] << 30;
		RPOKE();

		cache();
		string();
		body();

		if(body_bit[320] == 2) {
			header();

			RSHIFT(5); code += (len_size - 257) << 27;
			RSHIFT(5); code += (ptr_size - 1) << 27;
			RSHIFT(4); code += (hdr_size - 4) << 28;
			RPOKE();

			for(a = 0; a < hdr_size; a++) {
				uint bit = body_bit[hdr_order[a] + 320];
				if(bit == 32) bit = 0;
				RSHIFT(3); code += bit << 29;
				RPOKE();
			}

			if(table(body_sum + 320, body_bit + 320, 19)) abort();

			{ ulong huf;
			uint hufb;
			for(a = 0; a < len_size + ptr_size; ) {
				uint bit = header_buf[a];
				uint rep;
				for(rep = 1; a + rep < len_size + ptr_size; rep++) if(header_buf[a + rep] != bit) break;
				if(bit == 32) {
					bit = 0;
					while(rep >= 3) {
						uint len = (rep > 138 ? 138 : rep);
						if(len >= 11) {
							huf = body_sum[338]; hufb = body_bit[338];
							RSHIFT(hufb); code += huf;
							RSHIFT(7); code += (len - 11) << 25;
						} else {
							huf = body_sum[337]; hufb = body_bit[337];
							RSHIFT(hufb); code += huf;
							RSHIFT(3); code += (len - 3) << 29;
						}
						RPOKE();
						a += len; rep -= len;
					}
				}
				while(rep > 0) {
					huf = body_sum[bit + 320]; hufb = body_bit[bit + 320];
					RSHIFT(hufb); code += huf;
					RPOKE();
					a += 1; rep -= 1;
					while(rep >= 3) {
						uint len = (rep > 6 ? 6 : rep);
						huf = body_sum[336]; hufb = body_bit[336];
						RSHIFT(hufb); code += huf;
						RSHIFT(2); code += (len - 3) << 30;
						RPOKE();
						a += len; rep -= len;
					}
				}
			} }
		}

		if(table(body_sum + 0, body_bit + 0, 288)) abort();
		if(table(body_sum + 288, body_bit + 288, 32)) abort();

		{ ulong huf;
		uint hufb;

		ulong n;
		for(n = string_left; n < string_right; ) {
			uint len = string_len[n];
			uint ptr = string_ptr[n];
			if(len >= 3) {
				len -= 3;

				{ uint lenc = len_table[len];
				huf = body_sum[lenc + 257]; hufb = body_bit[lenc + 257];
				RSHIFT(hufb); code += huf;
				RPOKE();
				if(lenc >= 8 && lenc <= 27) {
					uint lenb = (lenc >> 2) - 1;
					RSHIFT(lenb); code += len << (32 - lenb) & 0xffffffff;
					RPOKE();
				} }

				{ uint ptrc = ptr_table[ptr];
				huf = body_sum[ptrc + 288]; hufb = body_bit[ptrc + 288];
				RSHIFT(hufb); code += huf;
				RPOKE();
				if(ptrc >= 4) {
					uint ptrb = (ptrc >> 1) - 1;
					RSHIFT(ptrb); code += ptr << (32 - ptrb) & 0xffffffff;
					RPOKE();
				} }

				len += 3;
			} else {
				huf = body_sum[dictionary[n]]; hufb = body_bit[dictionary[n]];
				RSHIFT(hufb); code += huf;
				RPOKE();
			}
			n += len;
		}
		huf = body_sum[256]; hufb = body_bit[256];
		RSHIFT(hufb); code += huf;
		RPOKE(); }
	}
	RSHIFT(7); RPOKE(); }

	for(a = 0; a < 32; a += 8) {
		ch = crc >> a;
		if(putc(ch, fp) == EOF) return "write error";
	}
	for(a = 0; a < 32; a += 8) {
		ch = length >> a;
		if(putc(ch, fp) == EOF) return "write error";
	}

	return(0);
}

int main(int argc, char **argv)
{
	int i;

	table_init();

	for(i = 1; i < argc; i++) {
		char *cave_name = argv[i];
		int size = -1;

		fputs(cave_name, stderr); putc(' ', stderr); fflush(stderr);

		memset(dictionary, 0, sizeof(dictionary));

		{ gzFile cave_in;
		if(cave_in = gzopen(cave_name, "rb"), cave_in == 0) perror("gzopen error");
		if(cave_in) if(size = gzread(cave_in, dictionary, LENGTH), size == -1) perror("gzread error");
		if(cave_in) if(gzclose(cave_in)) perror("gzclose error"); }

		if(size <= 0) continue;

		length = size;
		crc = 0;

		crc_calc(&crc, dictionary, length);

		pack();

		{ FILE *cave_out;
		if(cave_out = fopen(cave_name, "wb"), cave_out == 0) perror("create error");
		if(cave_out) if(dump(cave_out)) perror("write error");
		if(cave_out) if(fclose(cave_out)) perror("flush error"); }
	}
	if(argc < 2) {
		fputs("cavepack 0.4 © David Tritscher"
#ifdef ZLIB_VERSION
		" [zlib]"
#endif
		"\n\n", stderr);
		fputs("usage: cavepack [file...]\n\n", stderr);
		fputs("compress X11 Emerald Mine caves\n", stderr);
	}
	return(0);
}
