/* 2009-03-13 01:52:48 */

#define DICMASK 32767

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef unsigned long ulong;

struct undms {
	int state;

/* metadata */

	uchar info_header[4];
	uchar archive_header[52];
	uchar track_header[20];

/* track */

	uchar *pack_ptr;
	uchar *unpack_ptr;

	uint pack_size;
	uint simple_size;
	uint unpack_size;

	uint track;

/* flags */

	uint pack_flag;
	uint pack_mode;

	char clear_flag;
	char header_flag;
	char simple_flag;

	ulong id;
	uint sum1, sum2;

	uint cnt, len;

/* lzhuf */

	uint quick_dic; /* 256 byte dictionary */
	char quick_clear_flag;

	uint medium_dic; /* 16384 byte dictionary */
	char medium_clear_flag;

	uint deep_dic; /* 16384 byte dictionary */
	char deep_clear_flag;

	ushort deep_weight[628];
	ushort deep_child[628];
	ushort deep_parent[628];

	uint heavy_dic; /* 8192 byte dictionary */
	uint heavy_ptr;
	char heavy_clear_flag;
	char heavy_header_flag;

	ulong heavy_huf_len[2048];
	ulong heavy_huf_ptr[512];
	uchar heavy_bit[512];

/* buffers */

	uchar dictionary[DICMASK + 1];

	uchar buffer_one[32000];
	uchar buffer_two[32000];
};

#include <string.h>

/* crc-16-ibm */

static ushort crc_table[256];

static void crc_init(void)
{
static char build;
if(build) return;
{
	uint a, b;
	for(a = 0; a < 256; a++) {
		ushort crc = a;
		for(b = 0; b < 8; b++) {
			crc = ((crc & 1) == 0 ? 0 : 0xa001) ^ (crc >> 1);
		}
		crc_table[a] = crc;
	}
}
build = 1;
}

static uint crc_calc(uchar *in, uint size)
{
	uint crc = 0;
	while(size > 0) {
		crc = crc_table[*in++ ^ (crc & 255)] ^ (crc >> 8);
		size--;
	}
	return(crc);
}

static uint sum_calc(uchar *in, uint size)
{
	uint sum = 0;
	while(size > 0) {
		sum += *in++;
		size--;
	}
	return(sum & 65535);
}

#define IN() { if(in == inend) return "end of file"; }

static const char *simple_decode(struct undms *S, uchar *in, uchar *out, uchar *inend, uchar *outend)
{
	uint len, ch;

	while(out < outend) {
		IN(); ch = *in++; if(ch != 144) { *out++ = ch; continue; }
		IN(); len = *in++; if(len == 0) { *out++ = ch; continue; }
		IN(); ch = *in++;
		if(len == 255) {
			IN(); len = *in++;
			IN(); len = (len << 8) + *in++;
		}
		if(len > outend - out) return "bad repeat length";
		while(len > 0) {
			*out++ = ch;
			len--;
		}
	}

#if 0
	if(inend != in + 4) fprintf(stderr, "[simple: %u extra bytes]", inend - (in + 4));
#endif
	return(0);
}

#define PEEK() { while(shift >= 8) { IN(); shift -= 8; code += *in++ << shift; } }
#define SHIFT(x) { shift += (x); code <<= (x); code &= 0xffffffff; }

static const char *quick_decode(struct undms *S, uchar *in, uchar *out, uchar *inend, uchar *outend, char clear_flag)
{
	register ulong code = 0;
	register uint shift = 32;

	uint dic, str, len, ptr, bit, ch;

	if(clear_flag) {
		S->quick_clear_flag = 0;
		S->quick_dic = 0;
		memset(S->dictionary, 0, DICMASK + 1);
		S->quick_clear_flag = 1;
	}
	if(!S->quick_clear_flag) return "not initialized";

	dic = S->quick_dic;
	while(out < outend) {
		PEEK();
		bit = code >> 31; SHIFT(1);
		if(bit == 0) {
			len = (code >> 30) + 2; SHIFT(2);
			ptr = (code >> 24) + 1; SHIFT(8);

			if(len > outend - out) return "bad string length";
			str = dic - ptr;
			while(len > 0) {
				ch = S->dictionary[str++ & DICMASK];
				S->dictionary[dic++ & DICMASK] = ch;
				*out++ = ch;
				len--;
			}
		} else {
			ch = code >> 24; SHIFT(8);
			S->dictionary[dic++ & DICMASK] = ch;
			*out++ = ch;
		}
	}
	len = 5; ptr = 256; /* why this string? */
	str = dic - ptr;
	while(len > 0) {
		ch = S->dictionary[str++ & DICMASK];
		S->dictionary[dic++ & DICMASK] = ch;
		len--;
	}
	S->quick_dic = dic;

#if 0
	in -= (32 - shift) / 8;
	if(inend != in + 4) fprintf(stderr, "[quick: %u extra bytes]", inend - (in + 4));
#endif
	return(0);
}

/* static huffman */

static const uint medium_huf[16] = {
	0x0000 + 7, 0x0080 + 7, 0x0100 + 8, 0x0200 + 8,
	0x0300 + 8, 0x0400 + 9, 0x0600 + 9, 0x0800 + 9,
	0x0a00 + 9, 0x0c00 +10, 0x1000 +10, 0x1400 +10,
	0x1800 +11, 0x2000 +11, 0x2800 +11, 0x3000 +12,
};

static const char *medium_decode(struct undms *S, uchar *in, uchar *out, uchar *inend, uchar *outend, char clear_flag)
{
	register ulong code = 0;
	register uint shift = 32;

	uint dic, str, len, ptr, bit, ch;

	if(clear_flag) {
		S->medium_clear_flag = 0;
		S->medium_dic = 0;
		memset(S->dictionary, 0, DICMASK + 1);
		S->medium_clear_flag = 1;
	}
	if(!S->medium_clear_flag) return "not initialized";

	dic = S->medium_dic;
	while(out < outend) {
		PEEK();
		bit = code >> 31; SHIFT(1);
		if(bit == 0) {
			len = medium_huf[code >> 28];
			bit = (len & 15) - 4;
			len = (len >> 8) + 3;
			if(bit > 4) {
				SHIFT(4);
				bit -= 4;
				len += code >> (32 - bit);
			}
			SHIFT(bit);

			PEEK();
			ptr = medium_huf[code >> 28];
			SHIFT(4);
			bit = ptr & 15;
			ptr = (ptr & -16) + (code >> (32 - bit)) + 1;
			SHIFT(bit);

			if(len > outend - out) return "bad string length";
			str = dic - ptr;
			while(len > 0) {
				ch = S->dictionary[str++ & DICMASK];
				S->dictionary[dic++ & DICMASK] = ch;
				*out++ = ch;
				len--;
			}
		} else {
			ch = code >> 24; SHIFT(8);
			S->dictionary[dic++ & DICMASK] = ch;
			*out++ = ch;
		}
	}
	len = 66; ptr = 16384; /* why this string? */
	str = dic - ptr;
	while(len > 0) {
		ch = S->dictionary[str++ & DICMASK];
		S->dictionary[dic++ & DICMASK] = ch;
		len--;
	}
	S->medium_dic = dic;

#if 0
	in -= (32 - shift) / 8;
	if(inend != in + 4) fprintf(stderr, "[medium: %u extra bytes]", inend - (in + 4));
#endif
	return(0);
}

/* adaptive huffman */

static void deep_init(ushort weight[628], ushort child[628], ushort parent[628])
{
	uint a, b;
	/* leaves */
	for(a = 0, b = 1; a < 314; a++, b += 2) {
		child[a] = b;
		weight[a] = 1;
	}
	/* nodes */
	for(a = 314, b = 0; a < 627; a++, b += 2) {
		child[a] = b;
		weight[a] = weight[b] + weight[b + 1];
	}
	weight[627] = -1;
	/* parents */
	for(a = 0; a < 627; a++) parent[child[a]] = a;
}

static void deep_scale(ushort weight[628], ushort child[628], ushort parent[628])
{
	uint a, b, c, d, e;
	/* leaves */
	for(a = 627, b = 627; a > 0; ) {
		a--;
		if(child[a] & 1) {
			b--;
			child[b] = child[a];
			weight[b] = (weight[a] + 1) / 2;
		}
	}
	/* nodes */
	c = 314; d = 313;
	for(a = 314, b = 0; a < 627; a++, b += 2) {
		for(e = 0; e < 2; e++) {
			if(c < a && weight[c] < weight[d]) {
				child[b + e] = child[c];
				weight[b + e] = weight[c];
				c++;
			} else {
				child[b + e] = child[d];
				weight[b + e] = weight[d];
				d++;
			}
		}
		child[a] = b;
		weight[a] = weight[b] + weight[b + 1];
	}
	/* parents */
	for(a = 0; a < 627; a++) parent[child[a]] = a;
}

static void deep_update(ushort weight[628], ushort child[628], ushort parent[628], uint node)
{
	for(node = parent[node]; node != 626; node = parent[node & -2]) {
		if(weight[node] == weight[node + 1]) {
			uint swap = node;
			uint w = weight[node];
			do node++; while(w == weight[node + 1]); /* bottleneck */
			{
				uint childswap = child[swap];
				uint childnode = child[node];
				child[swap] = childnode;
				child[node] = childswap;
				parent[childswap] = node;
				parent[childnode] = swap;
			}
		}
		weight[node]++;
	}
	weight[626]++;
}

static const char *deep_decode(struct undms *S, uchar *in, uchar *out, uchar *inend, uchar *outend, char clear_flag)
{
	register ulong code = 0;
	register uint shift = 32;

	uint dic, str, len, ptr, bit, ch;
	uint node;

	if(clear_flag) {
		S->deep_clear_flag = 0;
		S->deep_dic = 0;
		memset(S->dictionary, 0, DICMASK + 1);
		deep_init(S->deep_weight, S->deep_child, S->deep_parent);
		S->deep_clear_flag = 1;
	}
	if(!S->deep_clear_flag) return "not initialized";

	dic = S->deep_dic;
	while(out < outend) {
		PEEK();
		for(node = S->deep_child[626]; (node & 1) == 0; node = S->deep_child[node]) {
			node += code >> 31; SHIFT(1);
		}
		if(S->deep_weight[626] == 32768) deep_scale(S->deep_weight, S->deep_child, S->deep_parent);
		deep_update(S->deep_weight, S->deep_child, S->deep_parent, node);

		node >>= 1;
		if(node >= 256) {
			len = node - 253;

			PEEK();
			ptr = medium_huf[code >> 28];
			SHIFT(4);
			bit = ptr & 15;
			ptr = (ptr & -16) + (code >> (32 - bit)) + 1;
			SHIFT(bit);

			if(len > outend - out) return "bad string length";
			str = dic - ptr;
			while(len > 0) {
				ch = S->dictionary[str++ & DICMASK];
				S->dictionary[dic++ & DICMASK] = ch;
				*out++ = ch;
				len--;
			}
		} else {
			ch = node;
			S->dictionary[dic++ & DICMASK] = ch;
			*out++ = ch;
		}
	}
	len = 60; ptr = 16384; /* why this string? */
	str = dic - ptr;
	while(len > 0) {
		ch = S->dictionary[str++ & DICMASK];
		S->dictionary[dic++ & DICMASK] = ch;
		len--;
	}
	S->deep_dic = dic;

#if 0
	in -= (32 - shift) / 8;
	if(inend != in + 4) fprintf(stderr, "[deep: %u extra bytes]", inend - (in + 4));
#endif
	return(0);
}

/* static huffman */

#define INDIRECT 0x80000000

static const char *heavy_table(ulong *table, uint table_size, uchar *length, uint size, uint pow)
{
	ulong avbl = 1, code = 0, table_alloc = 1 << pow;

	uint cnt, bit;

/* radix sort the lengths */

	uint radix[33] = { 0 };
	ushort sort[512];
	cnt = 0;
	for(bit = 0; bit < size; bit++) { radix[length[bit]]++; }
	for(bit = 0; bit <= 32; bit++) { uint sum = radix[bit]; radix[bit] = cnt; cnt += sum; }
	for(bit = 0; bit < size; bit++) { sort[radix[length[bit]]++] = bit; }

	cnt = 0;
	for(bit = 0; bit < 32; bit++) {
		for(; cnt < radix[bit]; cnt++) {
			ulong huf = (sort[cnt] << 8) + bit; /* pack the code and length into a single word */
			uint base = 0;

/* make indirect hash tables */

			ulong c = code;
			uint b = bit, p = pow;
			while(b > p) {
				b -= p;
				base += c >> b;
				c &= (1 << b) - 1;
				p = 2;
				if(c == 0) {
					table[base] = table_alloc + INDIRECT;
					table_alloc += 1 << p;
					if(table_alloc > table_size) return "*** BUG: huffman table";
				}
				base = table[base] - INDIRECT;
			}
			p -= b;
			base += c << p;

/* fill in hash tables */

			{
				uint rep = 1 << p;
				while(rep > 0) { table[base++] = huf; rep--; }
			}

			if(avbl == 0) return "huffman tree overflow";
			avbl--; code++;
		}
		avbl *= 2; code *= 2;
	}
	if(avbl != 0) return "huffman tree underflow";

	return(0);
}

static const char *heavy_decode(struct undms *S, uchar *in, uchar *out, uchar *inend, uchar *outend, uint repeat_code, char clear_flag, char header_flag)
{
	register ulong code = 0;
	register uint shift = 32;

	uint dic, str, len, ptr, bit, ch;
	ulong huf;

	if(clear_flag) {
		S->heavy_clear_flag = 0;
		S->heavy_dic = 0;
		S->heavy_ptr = 1;
		memset(S->dictionary, 0, DICMASK + 1);
		S->heavy_clear_flag = 1;
	}
	if(header_flag) {
		S->heavy_header_flag = 0;

/* length huffman tree */

		memset(S->heavy_bit, 32, 512);
		PEEK();
		len = code >> 23; SHIFT(9);
		if(len == 0) {
			ch = code >> 23; SHIFT(9);
			if(ch >= 511) return "bad character tree code";
			S->heavy_bit[ch] = 0;
		} else {
			if(len > 511) return "bad character tree length";
			for(bit = 0; bit < len; bit++) {
				PEEK();
				ch = code >> 27; SHIFT(5);
				if(ch > 20) return "bad character tree code";
				if(ch) S->heavy_bit[bit] = ch;
			}
		}
		if(heavy_table(S->heavy_huf_len, 2048, S->heavy_bit, 512, 10)) return "bad character tree";

/* pointer huffman tree */

		memset(S->heavy_bit, 32, 32);
		PEEK();
		len = code >> 27; SHIFT(5);
		if(len == 0) {
			ch = code >> 27; SHIFT(5);
			if(ch >= repeat_code + 1) return "bad string tree code";
			S->heavy_bit[ch] = 0;
		} else {
			if(len > repeat_code + 1) return "bad string tree length";
			for(bit = 0; bit < len; bit++) {
				PEEK();
				ch = code >> 28; SHIFT(4);
				if(ch > 16) return "bad string tree code";
				if(ch) S->heavy_bit[bit] = ch;
			}
		}
		if(heavy_table(S->heavy_huf_ptr, 512, S->heavy_bit, 32, 8)) return "bad string tree";

		S->heavy_header_flag = 1;
	}
	if(!S->heavy_clear_flag || !S->heavy_header_flag) return "not initialized";

	dic = S->heavy_dic;
	ptr = S->heavy_ptr;
	while(out < outend) {

/* length huffman code */

		PEEK(); huf = S->heavy_huf_len[code >> 22];
		if(huf >= INDIRECT) {
			huf = S->heavy_huf_len[huf + (code >> 20 & 3) - INDIRECT];
			if(huf >= INDIRECT) {
				huf = S->heavy_huf_len[huf + (code >> 18 & 3) - INDIRECT];
				if(huf >= INDIRECT) {
					huf = S->heavy_huf_len[huf + (code >> 16 & 3) - INDIRECT];
					if(huf >= INDIRECT) {
						huf = S->heavy_huf_len[huf + (code >> 14 & 3) - INDIRECT];
						if(huf >= INDIRECT) {
							huf = S->heavy_huf_len[huf + (code >> 12 & 3) - INDIRECT];
						}
					}
				}
			}
		}
		bit = huf & 255; SHIFT(bit); huf >>= 8;

		if(huf >= 256) {
			len = huf - 253;

/* pointer huffman code */

			PEEK(); huf = S->heavy_huf_ptr[code >> 24];
			if(huf >= INDIRECT) {
				huf = S->heavy_huf_ptr[huf + (code >> 22 & 3) - INDIRECT];
				if(huf >= INDIRECT) {
					huf = S->heavy_huf_ptr[huf + (code >> 20 & 3) - INDIRECT];
					if(huf >= INDIRECT) {
						huf = S->heavy_huf_ptr[huf + (code >> 18 & 3) - INDIRECT];
						if(huf >= INDIRECT) {
							huf = S->heavy_huf_ptr[huf + (code >> 16 & 3) - INDIRECT];
						}
					}
				}
			}
			bit = huf & 255; SHIFT(bit); huf >>= 8;

			if(huf != repeat_code) {
				ptr = huf;
				if(ptr >= 2) {
					bit = ptr - 1;
					PEEK();
					ptr = (1 << bit) + (code >> (32 - bit)); SHIFT(bit);
				}
				ptr += 1;
			}

			if(len > outend - out) return "bad string length";
			str = dic - ptr;
			while(len > 0) {
				ch = S->dictionary[str++ & DICMASK];
				S->dictionary[dic++ & DICMASK] = ch;
				*out++ = ch;
				len--;
			}
		} else {
			ch = huf;
			S->dictionary[dic++ & DICMASK] = ch;
			*out++ = ch;
		}
	}
	S->heavy_dic = dic;
	S->heavy_ptr = ptr;

#if 0
	in -= (32 - shift) / 8;
	if(inend != in + 4) fprintf(stderr, "[heavy: %u extra bytes]", inend - (in + 4));
#endif
	return(0);
}

#undef IN

#define IN(st) { case st: if(in == inend && inend != 0) { S->state = st; goto ret; } }
#define OUT(st) { case st: if(out == outend) { S->state = st; goto ret; } }

#define BE16(x) (mem = (x), mem[0] << 8 | mem[1])
#define BE32(x) (mem = (x), mem[0] << 24 | mem[1] << 16 | mem[2] << 8 | mem[3])

const char *undms(struct undms *S, uchar **indin, uchar **indout, uchar *inend, uchar *outend)
{
	uchar *in = *indin;
	uchar *out = *indout;
	uchar *mem;

	if(S == 0) return "failed to allocate state";

switch(S->state) {
default: if(in == inend) break;

/* read info header */

	for(S->cnt = 0; S->cnt < sizeof(S->info_header); S->cnt++) {
		IN(1); if(in == 0) return "failed to read info_header";
		S->info_header[S->cnt] = *in++;
	}
	S->id = BE32(S->info_header + 0);
	if(S->id != 0x444d5321) return "unknown info_header";

/* read archive header */

	for(S->cnt = 0; S->cnt < sizeof(S->archive_header); S->cnt++) {
		IN(2); if(in == 0) return "failed to read archive_header";
		S->archive_header[S->cnt] = *in++;
	}
	S->sum1 = BE16(S->archive_header + 50);
	S->sum2 = crc_calc(S->archive_header, 50);
	if(S->sum1 != S->sum2) return "bad archive_header checksum";

	S->quick_clear_flag = 0;
	S->medium_clear_flag = 0;
	S->deep_clear_flag = 0;
	S->heavy_clear_flag = 0;
	S->heavy_header_flag = 0;

	S->clear_flag = 1;
	for(;;) {
		IN(3); if(in == 0) break;

/* read track header */

		for(S->cnt = 0; S->cnt < sizeof(S->track_header); S->cnt++) {
			IN(4); if(in == 0) return "failed to read track_header";
			S->track_header[S->cnt] = *in++;
		}
		S->id = BE16(S->track_header + 0);
		if(S->id != 0x5452) return "unknown track_header";
		S->sum1 = BE16(S->track_header + 18);
		S->sum2 = crc_calc(S->track_header, 18);
		if(S->sum1 != S->sum2) return "bad track_header checksum";

		S->track = BE16(S->track_header + 2);

		S->pack_size = BE16(S->track_header + 6);
		S->simple_size = BE16(S->track_header + 8);
		S->unpack_size = BE16(S->track_header + 10);

		S->pack_flag = S->track_header[12];
		S->pack_mode = S->track_header[13];

/* read compressed data */

		if(S->pack_size + 4 > 32000 || S->simple_size + 4 > 32000 || S->unpack_size + 4 > 32000) return "*** BUG: compress buffers";

		S->pack_ptr = S->buffer_one;
		S->unpack_ptr = S->buffer_two;

		for(S->cnt = 0; S->cnt < S->pack_size; S->cnt += S->len) {
			IN(5); if(in == 0) return "failed to read compress_data";
			S->len = S->pack_size - S->cnt;
			if(S->len > inend - in) S->len = inend - in;
			memcpy(S->pack_ptr + S->cnt, in, S->len);
			in += S->len;
		}
		S->sum1 = BE16(S->track_header + 16);
		S->sum2 = crc_calc(S->pack_ptr, S->pack_size);
		if(S->sum1 != S->sum2) return "bad compress_data checksum";

		if(S->track > 1000 || S->unpack_size < 8192) continue; /* try to skip bad tracks */

/* uncompress */

		S->header_flag = (S->pack_flag & 2) ? 1 : 0;
		S->simple_flag = (S->pack_flag & 4) ? 1 : 0;
		switch(S->pack_mode) {
		case 0: /* none */
			S->unpack_ptr = S->buffer_one;
			break;
		case 1: /* simple */
			if(simple_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->pack_size + 4, S->unpack_ptr + S->unpack_size)) return "failed to uncompress [simple]";
			break;
		case 2: /* quick */
			if(quick_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->pack_size + 4, S->unpack_ptr + S->simple_size, S->clear_flag)) return "failed to uncompress [quick]";
			S->pack_ptr = S->buffer_two;
			S->unpack_ptr = S->buffer_one;
			if(simple_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->simple_size + 4, S->unpack_ptr + S->unpack_size)) return "failed to uncompress [quick simple]";
			break;
		case 3: /* medium */
			if(medium_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->pack_size + 4, S->unpack_ptr + S->simple_size, S->clear_flag)) return "failed to uncompress [medium]";
			S->pack_ptr = S->buffer_two;
			S->unpack_ptr = S->buffer_one;
			if(simple_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->simple_size + 4, S->unpack_ptr + S->unpack_size)) return "failed to uncompress [medium simple]";
			break;
		case 4: /* deep */
			if(deep_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->pack_size + 4, S->unpack_ptr + S->simple_size, S->clear_flag)) return "failed to uncompress [deep]";
			S->pack_ptr = S->buffer_two;
			S->unpack_ptr = S->buffer_one;
			if(simple_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->simple_size + 4, S->unpack_ptr + S->unpack_size)) return "failed to uncompress [deep simple]";
			break;
		case 5: /* heavy1 */
			if(heavy_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->pack_size + 4, S->unpack_ptr + S->simple_size, 13, S->clear_flag, S->header_flag)) return "failed to uncompress [heavy1]";
			if(S->simple_flag) {
				S->pack_ptr = S->buffer_two;
				S->unpack_ptr = S->buffer_one;
				if(simple_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->simple_size + 4, S->unpack_ptr + S->unpack_size)) return "failed to uncompress [heavy1 simple]";
			}
			break;
		case 6: /* heavy2 */
			if(heavy_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->pack_size + 4, S->unpack_ptr + S->simple_size, 14, S->clear_flag, S->header_flag)) return "failed to uncompress [heavy2]";
			if(S->simple_flag) {
				S->pack_ptr = S->buffer_two;
				S->unpack_ptr = S->buffer_one;
				if(simple_decode(S, S->pack_ptr, S->unpack_ptr, S->pack_ptr + S->simple_size + 4, S->unpack_ptr + S->unpack_size)) return "failed to uncompress [heavy2 simple]";
			}
			break;
		default:
			return "unknown compress_mode";
		}
		S->clear_flag = (S->pack_flag & 1) ? 0 : 1;

/* write uncompressed data */

		S->sum1 = BE16(S->track_header + 14);
		S->sum2 = sum_calc(S->unpack_ptr, S->unpack_size);
		if(S->sum1 != S->sum2) return "bad uncompress_data checksum";

		for(S->cnt = 0; S->cnt < S->unpack_size; S->cnt += S->len) {
			OUT(6);
			S->len = S->unpack_size - S->cnt;
			if(S->len > outend - out) S->len = outend - out;
			memcpy(out, S->unpack_ptr + S->cnt, S->len);
			out += S->len;
		}
	}
}
S->state = 0;

ret:
	*indin = in;
	*indout = out;
	return(0);
}

#include <stdlib.h>
void undms_init(struct undms **indS) { if(*indS == 0) (*indS) = malloc(sizeof(struct undms)); if(*indS) (*indS)->state = 0; crc_init(); }
void undms_free(struct undms **indS) { if(*indS) free(*indS); (*indS) = 0; }
