/* 2010-08-31 01:52:20 */

#define META

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef unsigned long ulong;

struct zip {
	int state;

/* strings */

	uchar dictionary[32768 + 256 * 2];
	ulong pointer[4096];

	uchar *dicptr;

	uint zero;
	ulong size;
	ulong hash;

/* bitstream */

	ulong code;
	uint shift;

	ulong huf;
	uint bit;

	ulong len, ptr;
	uint lenb, ptrb;
	uint lenc, ptrc;

/* metadata */

#ifdef META
	ulong crc;
	uchar *crcptr;

	uchar meta[10];
	uint cnt;
#endif
};

#include <string.h>

/* static huffman */

static uint rev(uint a)
{
	a = (a & 0x00ff) << 8 | (a >> 8 & 0x00ff);
	a = (a & 0x0f0f) << 4 | (a >> 4 & 0x0f0f);
	a = (a & 0x3333) << 2 | (a >> 2 & 0x3333);
	a = (a & 0x5555) << 1 | (a >> 1 & 0x5555);
	return(a);
}

static ulong huf_table[320];

static void huf_init(void)
{
static char build;
if(build) return;
{
	uint a;
	for(a = 0; a < 144; a++) huf_table[a + 0] = (rev(a + 48) << 16) + 8;
	for(a = 0; a < 112; a++) huf_table[a + 144] = (rev(a + 400) << 16) + 9;
	for(a = 0; a < 24; a++) huf_table[a + 256] = (rev(a + 0) << 16) + 7;
	for(a = 0; a < 8; a++) huf_table[a + 280] = (rev(a + 192) << 16) + 8;
	for(a = 0; a < 32; a++) huf_table[a + 288] = (rev(a) << 16) + 5;
}
build = 1;
}

/* crc-32-ieee 802.3 */

static ulong crc_table[256];

static void crc_calc(ulong *indcrc, uchar *in, uint size)
{
	ulong crc = *indcrc;
	crc = ~crc & 0xffffffff;
	while(size > 0) {
		crc = crc_table[*in++ ^ (crc & 255)] ^ (crc >> 8);
		size--;
	}
	crc = ~crc & 0xffffffff;
	*indcrc = crc;
}

static void crc_init(void)
{
static char build;
if(build) return;
{
	uint a, b;
	for(a = 0; a < 256; a++) {
		ulong crc = a;
		for(b = 0; b < 8; b++) {
			crc = ((crc & 1) == 0 ? 0 : 0xedb88320) ^ (crc >> 1);
		}
		crc_table[a] = crc;
	}
}
build = 1;
}

static uchar bit_table[256];

#define LOG(x) ((x) < 256 ? bit_table[x] : bit_table[(x) >> 8 & 255] + 8)

static void bit_init(void)
{
static char build;
if(build) return;
{
	uint a;
	for(a = 0; a < 256; a++) {
		uint bit = 0, mask = a;
		if(mask >= 16) { bit += 4; mask >>= 4; }
		if(mask >= 4) { bit += 2; mask >>= 2; }
		if(mask >= 2) { bit += 1; mask >>= 1; }
		bit_table[a] = bit + mask;
	}
}
build = 1;
}

#define IN(st) { case st: if(in == inend && inend != 0) { S->state = st; goto ret; } }
#define OUT(st) { case st: if(out == outend) { S->state = st; goto ret; } }

#define RPOKE(st) { while(S->shift <= 24) { OUT(st); *out++ = S->code >> S->shift; S->shift += 8; } }
#define RSHIFT(x) { S->code >>= (x); S->shift -= (x); }

const char *zip(struct zip *S, uchar **indin, uchar **indout, uchar *inend, uchar *outend)
{
	uchar *in = *indin;
	uchar *out = *indout;

	if(S == 0) return "failed to allocate state";

switch(S->state) {
default: if(in == inend) break;

	S->dicptr = S->dictionary;

	S->zero = 256;
	S->size = 32768;

	memset(S->pointer, 0, sizeof(S->pointer));

	for(S->len = 256; S->len > 0; S->len--) {
		IN(1);
		if(in) {
			S->dicptr[256] = *in++;
		} else {
			S->dicptr[256] = 0;
			S->zero--;
		}
		S->dicptr++;
	}

#ifdef META
	S->crc = 0;
	S->crcptr = S->dicptr;

	S->meta[0] = 31; S->meta[1] = 139; S->meta[2] = 8; S->meta[3] = 0;
	S->meta[4] = 0; S->meta[5] = 0; S->meta[6] = 0; S->meta[7] = 0;
	S->meta[8] = 0; S->meta[9] = 255;

	for(S->cnt = 0; S->cnt < 10; S->cnt++) {
		OUT(9); *out++ = S->meta[S->cnt];
	}
#endif

	S->code = 0; S->shift = 32;

	RSHIFT(3); S->code += 3 << 29;
	RPOKE(2);

	S->hash = S->dicptr[0] << 24 | S->dicptr[1] << 16 | S->dicptr[2] << 8 | S->dicptr[3];

	while(S->zero > 0) {

/* find string */

		S->len = 1;
		S->ptr = S->size - S->pointer[(S->hash & 0xffffffff) % 4093];
		if(S->ptr >= 1 && S->ptr <= 32767) {
			uint len = 0;
			uchar *str_a = S->dicptr - S->ptr + (S->dicptr < S->dictionary + S->ptr ? 32768 + 256 : 0);
			uchar *str_b = S->dicptr;
			while(len < 256) {
				if(str_a[len] != str_b[len]) break;
				len++;
			}
			S->len = len;
			if(S->len > S->zero) S->len = S->zero;
		}

/* write string */

		if(S->len >= 3) {
			S->len -= 3; S->ptr -= 1;

			S->lenc = LOG(S->len >> 2);
			S->lenb = S->lenc > 0 ? S->lenc - 1 : 0;
			S->lenc = (S->lenc << 2) + (S->len >> S->lenb & 3);

			S->huf = huf_table[S->lenc + 257]; S->bit = S->huf & 255;
			RSHIFT(S->bit); S->code += S->huf & -256;
			if(S->lenb > 0) {
				RSHIFT(S->lenb); S->code += S->len << (32 - S->lenb) & 0xffffffff;
			}
			RPOKE(3);

			S->ptrc = LOG(S->ptr >> 1);
			S->ptrb = S->ptrc > 0 ? S->ptrc - 1 : 0;
			S->ptrc = (S->ptrc << 1) + (S->ptr >> S->ptrb & 1);

			S->huf = huf_table[S->ptrc + 288]; S->bit = S->huf & 255;
			RSHIFT(S->bit); S->code += S->huf & -256;
			if(S->ptrb > 0) {
				RSHIFT(S->ptrb); S->code += S->ptr << (32 - S->ptrb) & 0xffffffff;
			}
			RPOKE(4);

			S->len += 3; S->ptr += 1;
		} else {
			S->huf = huf_table[S->dicptr[0]]; S->bit = S->huf & 255;
			RSHIFT(S->bit); S->code += S->huf & -256;
			RPOKE(5);

			S->len = 1;
		}

/* skip string */

		for(; S->len > 0; S->len--) {
			IN(6);
			if(in) {
				S->dicptr[256] = *in++;
			} else {
				S->dicptr[256] = 0;
				S->zero--;
			}
			S->dicptr++;

			if(S->dicptr == S->dictionary + 32768 + 256) {
#ifdef META
				crc_calc(&S->crc, S->crcptr, S->dictionary + 32768 + 256 - S->crcptr);
				S->crcptr = S->dictionary;
#endif
				memcpy(S->dictionary, S->dictionary + 32768 + 256, 256);
				S->dicptr = S->dictionary;
			}

			S->pointer[(S->hash & 0xffffffff) % 4093] = S->size;
			S->hash = S->hash << 8 | S->dicptr[3];
			S->size++;
		}
	}

	S->huf = huf_table[256]; S->bit = S->huf & 255;
	RSHIFT(S->bit); S->code += S->huf & -256;

	RSHIFT(7);
	RPOKE(7);

#ifdef META
	crc_calc(&S->crc, S->crcptr, S->dicptr - S->crcptr);

	S->meta[0] = S->crc; S->meta[1] = S->crc >> 8; S->meta[2] = S->crc >> 16; S->meta[3] = S->crc >> 24;
	S->crc = S->size - 32768;
	S->meta[4] = S->crc; S->meta[5] = S->crc >> 8; S->meta[6] = S->crc >> 16; S->meta[7] = S->crc >> 24;

	for(S->cnt = 0; S->cnt < 8; S->cnt++) {
		OUT(8); *out++ = S->meta[S->cnt];
	}
#endif
}
S->state = 0;

ret:
	*indin = in;
	*indout = out;
	return(0);
}

#include <stdlib.h>
void zip_init(struct zip **indS) { if(*indS == 0) (*indS) = malloc(sizeof(struct zip)); if(*indS) (*indS)->state = 0; huf_init(); crc_init(); bit_init(); }
void zip_free(struct zip **indS) { if(*indS) free(*indS); (*indS) = 0; }
