/* handle display, keyboard, joystick, and audio in emerald mine */

/* uses amigaos intuition */

extern char *global;
static char *local = "amiga";

#include "proto.h"

#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <proto/console.h>

#include <devices/console.h>
#include <devices/timer.h>
#include <devices/gameport.h>
#include <devices/audio.h>

#include <exec/errors.h>
#include <devices/conunit.h>

#include <stdio.h>
#include <string.h>

/* ---------------------------------------------------------------------- */

#define PLANES 6

/* colourmap for amiga ocs extra halfbrite */

static UWORD privateColours[32] = {
	0x000,0xfff,0xf00,0xd00,0xa00,0xea6,0xb85,0x963,
	0xfe0,0xfd0,0xfb0,0xe90,0xb60,0x0f0,0x0c0,0x0a0,
	0xcff,0x8ad,0x477,0x245,0x58f,0x25d,0x129,0x0dd,
	0x388,0xe0b,0x534,0xc00,0xc31,0x832,0x74a,0xccc,
};

static char paletteHash[128] = {
	            ['!'] = 32,             ['#'] = 1,  ['$'] = 1,  ['%'] = 31, ['&'] = 31,
	['('] = 31, [')'] = 31, ['*'] = 2,  ['+'] = 3,  [','] = 4,  ['-'] = 4,  ['.'] = 34, ['/'] = 36,
	['0'] = 5,  ['1'] = 6,  ['2'] = 6,  ['3'] = 7,  ['4'] = 37, ['5'] = 38, ['6'] = 8,  ['7'] = 9,
	['8'] = 10, ['9'] = 11, [':'] = 12, [';'] = 12, ['<'] = 13, ['='] = 14, ['>'] = 14,
	['@'] = 15, ['A'] = 45, ['B'] = 47, ['C'] = 16, ['D'] = 31, ['E'] = 17, ['F'] = 48, ['G'] = 18,
	['H'] = 19, ['I'] = 17, ['J'] = 20, ['K'] = 21, ['L'] = 21, ['M'] = 22, ['N'] = 22, ['O'] = 23,
	['P'] = 23, ['Q'] = 23, ['R'] = 24, ['S'] = 55, ['T'] = 55, ['U'] = 25, ['V'] = 25, ['W'] = 25,
	['X'] = 57, ['Y'] = 57, ['Z'] = 57, ['['] = 33,             [']'] = 48, ['^'] = 49, ['_'] = 49,
	['`'] = 26, ['a'] = 62, ['b'] = 16, ['c'] = 31, ['d'] = 5,  ['e'] = 7,  ['f'] = 9,  ['g'] = 11,
	['h'] = 20, ['i'] = 20, ['j'] = 21, ['k'] = 25, ['l'] = 4,  ['m'] = 20, ['n'] = 21, ['o'] = 21,
	['p'] = 21, ['q'] = 21, ['r'] = 22, ['s'] = 22, ['t'] = 22, ['u'] = 27, ['v'] = 4,  ['w'] = 34,
	['x'] = 34,
};

static char *keyboardString[128] = {
	"grave", "1", "2", "3", "4", "5", "6", "7",
	"8", "9", "0", "minus", "equal", "backslash", "<0e>", "KP_0",
	"q", "w", "e", "r", "t", "y", "u", "i",
	"o", "p", "bracketleft", "bracketright", "<1c>", "KP_1", "KP_2", "KP_3",
	"a", "s", "d", "f", "g", "h", "j", "k", "l",
	"semicolon", "apostrophe", "<2b>", "<2c>", "KP_4", "KP_5", "KP_6",
	"<30>", "z", "x", "c", "v", "b", "n", "m",
	"comma", "period", "slash", "<3b>", "KP_Decimal", "KP_7", "KP_8", "KP_9",
	"space", "BackSpace", "Tab", "KP_Enter", "Return", "Escape", "Delete", "<47>",
	"<48>", "<49>", "KP_Subtract", "<4b>", "Up", "Down", "Right", "Left",
	"F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8",
	"F9", "F10", "<5a>", "<5b>", "<5c>", "<5d>", "<5e>", "Help",
	"Shift_L", "Shift_R", "Caps_Lock", "Control", "Alt_L", "Alt_R", "Amiga_L", "Amiga_R",
	"<68>", "<69>", "<6a>", "<6b>", "<6c>", "<6d>", "<6e>", "<6f>",
	"<70>", "<71>", "<72>", "<73>", "<74>", "<75>", "<76>", "<77>",
	"<78>", "<79>", "<7a>", "<7b>", "<7c>", "<7d>", "<7e>", "<7f>",
};

/* ---------------------------------------------------------------------- */

static void InitList(struct List *list) /* amiga.lib/NewList */
{
	list->lh_Head = (void *)(&list->lh_Tail);
	list->lh_TailPred = (void *)(&list->lh_Head);
}
static void InitMsgPort(struct MsgPort *msgPort, UBYTE signalNum) /* exec.library/CreateMsgPort */
{
	msgPort->mp_Node.ln_Type = NT_MSGPORT;
	msgPort->mp_Flags = PA_SIGNAL;
	msgPort->mp_SigBit = signalNum;
	msgPort->mp_SigTask = FindTask(0);
	InitList(&msgPort->mp_MsgList);
}
static void InitIORequest(struct IORequest *ioReq, struct MsgPort *replyPort, ULONG size) /* exec.library/CreateIORequest */
{
	ioReq->io_Message.mn_Node.ln_Type = NT_REPLYMSG;
	ioReq->io_Message.mn_ReplyPort = replyPort;
	ioReq->io_Message.mn_Length = size;
	ioReq->io_Flags = IOF_QUICK;
}
static char *OpenScreenFault(ULONG errorCode)
{
	switch(errorCode) {
	case OSERR_NOMONITOR: return "named monitor spec not available";
	case OSERR_NOCHIPS: return "you need newer custom chips";
	case OSERR_NOMEM: return "couldn't get normal memory";
	case OSERR_NOCHIPMEM: return "couldn't get chipmem";
	case OSERR_PUBNOTUNIQUE: return "public screen name already used";
	case OSERR_UNKNOWNMODE: return "don't recognize mode asked for";
	case OSERR_TOODEEP: return "screen deeper than hw supports";
	case OSERR_ATTACHFAIL: return "failed to attach screens";
	case OSERR_NOTAVAILABLE: return "mode not available for other reason";
	}
	return "";
}
static char *DeviceFault(BYTE ioError)
{
	switch(ioError) {
	case IOERR_OPENFAIL: return "device/unit failed to open";
	case IOERR_ABORTED: return "request terminated early";
	case IOERR_NOCMD: return "command not supported by device";
	case IOERR_BADLENGTH: return "not a valid length";
	case IOERR_BADADDRESS: return "invalid address";
	case IOERR_UNITBUSY: return "requested unit is busy";
	case IOERR_SELFTEST: return "hardware failed self-test";
	}
	return "";
}
static char *AudioFault(BYTE ioError)
{
	switch(ioError) {
	case ADIOERR_NOALLOCATION: return "no allocation key";
	case ADIOERR_ALLOCFAILED: return "allocation failed";
	case ADIOERR_CHANNELSTOLEN: return "channel stolen";
	}
	return(DeviceFault(ioError));
}

/* ---------------------------------------------------------------------- */

/* create attached screens, create window, create palette, create cursor */

static struct MsgPort ioMsgPort;

static struct IOStdReq consoleIOReq;
struct Device *ConsoleDevice;

static struct timerequest timerIOReq;
struct Device *TimerBase;

static struct RastPort screenPixmap;
static struct BitMap screenBM;

static struct Window *window;
static UWORD *cursor;

static struct ViewPort *viewport;

static ULONG osErrorCode;

static struct TagItem nsExtension[] = {
	{ SA_Parent, 0 },
	{ SA_Draggable, 0 },
	{ SA_ErrorCode, (ULONG)(&osErrorCode) },
	{ TAG_DONE, 0 },
};
static struct ExtNewScreen newScreen = {
	0, 12, VIEW_WIDTH, VIEW_HEIGHT, PLANES, 0, 0,
	EXTRA_HALFBRITE,
	CUSTOMSCREEN | CUSTOMBITMAP | NS_EXTENDED,
	0, 0, 0, 0,
	nsExtension,
};
static struct NewWindow newWindow = {
	0, 0, VIEW_WIDTH, VIEW_HEIGHT, 0, 0,
	IDCMP_RAWKEY,
	WFLG_SIMPLE_REFRESH | WFLG_BORDERLESS | WFLG_ACTIVATE | WFLG_RMBTRAP | WFLG_NOCAREREFRESH,
	0, 0, 0, 0, 0,
	0, 0, 0, 0,
	CUSTOMSCREEN,
};

static struct TagItem scoreNSExtension[] = { { SA_ErrorCode, (ULONG)(&osErrorCode) } , { TAG_DONE, 0 } };
static struct ExtNewScreen scoreNewScreen = {
	0, 0, SCORE_WIDTH, 8, 1, 0, 0,
	0,
	CUSTOMSCREEN | NS_EXTENDED,
	0, "X11 Emerald Mine", 0, 0,
	scoreNSExtension,
};
static struct Screen *scoreScreen;

int display_open(void)
{
	int a;

	InitMsgPort(&ioMsgPort, SIGB_SINGLE);

	InitIORequest((void *)(&consoleIOReq), &ioMsgPort, sizeof(consoleIOReq));
	if(OpenDevice("console.device", CONU_LIBRARY, (void *)(&consoleIOReq), 0)) {
		fprintf(stderr, "%s[%s]: %s: %s\n", global, local, "failed to open console device", DeviceFault(consoleIOReq.io_Error));
		return(1);
	}
	ConsoleDevice = consoleIOReq.io_Device;

	InitIORequest((void *)(&timerIOReq), &ioMsgPort, sizeof(timerIOReq));
	if(OpenDevice("timer.device", UNIT_MICROHZ, (void *)(&timerIOReq), 0)) {
		fprintf(stderr, "%s[%s]: %s: %s\n", global, local, "failed to open timer device", DeviceFault(timerIOReq.tr_node.io_Error));
		return(1);
	}
	TimerBase = timerIOReq.tr_node.io_Device;

	if(scoreScreen = OpenScreen((void *)(&scoreNewScreen)), scoreScreen == 0) {
		fprintf(stderr, "%s[%s]: %s: %s\n", global, local, "failed to open score screen", OpenScreenFault(osErrorCode));
		return(1);
	}
	LoadRGB4(&scoreScreen->ViewPort, privateColours, 2);

	nsExtension[0].ti_Data = (ULONG)(scoreScreen); /* attach screen */

	InitRastPort(&screenPixmap);
	screenPixmap.BitMap = &screenBM;
	InitBitMap(&screenBM, PLANES, VIEW_WIDTH + SCROLL_WIDTH, VIEW_HEIGHT + SCROLL_HEIGHT);
	for(a = 0; a < PLANES; a++) if(screenBM.Planes[a] = AllocRaster(VIEW_WIDTH + SCROLL_WIDTH, VIEW_HEIGHT + SCROLL_HEIGHT), screenBM.Planes[a] == 0) {
		fprintf(stderr, "%s[%s]: %s (%dx%dx%d)\n", global, local, "failed to allocate screen pixmap", VIEW_WIDTH + SCROLL_WIDTH, VIEW_HEIGHT + SCROLL_HEIGHT, PLANES);
		return(1);
	}

	newScreen.CustomBitMap = screenPixmap.BitMap;
	if(newWindow.Screen = OpenScreen((void *)(&newScreen)), newWindow.Screen == 0) {
		fprintf(stderr, "%s[%s]: %s: %s\n", global, local, "failed to open screen", OpenScreenFault(osErrorCode));
		return(1);
	}
	if(window = OpenWindow(&newWindow), window == 0) {
		fprintf(stderr, "%s[%s]: %s\n", global, local, "failed to open window");
		return(1);
	}
	viewport = &window->WScreen->ViewPort;

	LoadRGB4(viewport, privateColours, 32);

	if(cursor = AllocMem(128, MEMF_CHIP | MEMF_CLEAR), cursor) SetPointer(window, cursor, 1, 1, 0, 0);

	return(0);
}

/* handle events, frame sync */

int keyboard_raw = -1;
int keyboard_cook = -1;

static char keyboardVector[16];

static long frame_usec;

static struct InputEvent consoleEvent;

void display_read(void)
{
	long usec;
	int kc;
	char buf[3];
	struct IntuiMessage *event;

#ifdef __libnix
	__chkabort();
#endif

	while(event = (void *)(GetMsg(window->UserPort)), event) {
		switch(event->Class) {
		case IDCMP_RAWKEY: /* KeyPress */ /* KeyRelease */
			kc = event->Code & 127;
			if((event->Code & IECODE_UP_PREFIX) == 0) {
				keyboardVector[kc >> 3] |= (1 << (kc & 7));
				keyboard_raw = kc;
				buf[0] = 0; buf[1] = 0; buf[2] = 0;
				consoleEvent.ie_Class = IECLASS_RAWKEY;
				consoleEvent.ie_Code = event->Code;
				consoleEvent.ie_Qualifier = event->Qualifier;
				RawKeyConvert(&consoleEvent, buf, 3, 0);
				keyboard_cook = (buf[0] & 255) | (buf[1] & 255) << 8 | (buf[2] & 255) << 16;
			} else {
				keyboardVector[kc >> 3] &= ~(1 << (kc & 7));
				keyboard_raw = -1;
				keyboard_cook = -1;
			}
			break;
		}
		ReplyMsg((void *)(event));
	}

	timerIOReq.tr_node.io_Command = TR_GETSYSTIME;
	DoIO((void *)(&timerIOReq));
	usec = frame_usec + 1000000;
	frame_usec = (timerIOReq.tr_time.tv_micro + timerIOReq.tr_time.tv_secs * 1000000) * 60;
	usec -= frame_usec;
	if(usec >= 0 && usec <= 1500000) {
		frame_usec += usec;
		timerIOReq.tr_node.io_Command = TR_ADDREQUEST;
		timerIOReq.tr_time.tv_micro = usec / 60 + 1; timerIOReq.tr_time.tv_secs = 0;
		DoIO((void *)(&timerIOReq));
	}
}

int display_close(void)
{
	int a;
	if(cursor) { ClearPointer(window); FreeMem(cursor, 128); cursor = 0; }
	if(window) { CloseWindow(window); window = 0; }
	if(newWindow.Screen) { CloseScreen(newWindow.Screen); newWindow.Screen = 0; }
	for(a = 0; a < PLANES; a++) if(screenBM.Planes[a]) { FreeRaster(screenBM.Planes[a], VIEW_WIDTH + SCROLL_WIDTH, VIEW_HEIGHT + SCROLL_HEIGHT); screenBM.Planes[a] = 0; }
	if(scoreScreen) { CloseScreen(scoreScreen); scoreScreen = 0; }
	if(TimerBase) { CloseDevice((void *)(&timerIOReq)); TimerBase = 0; }
	if(ConsoleDevice) { CloseDevice((void *)(&consoleIOReq)); ConsoleDevice = 0; }
	return(0);
}

/* ---------------------------------------------------------------------- */

/* handle keyboard, key names */

char keyboard_input[2][INPUT_MAX];

int keyboard_keycode[2][INPUT_MAX];

static const int defaultKeyCode[INPUT_MAX][2] = {
	[INPUT_north] = { 0x4c, 0x11 }, /* Up, w */
	[INPUT_east] = { 0x4e, 0x22 }, /* Right, d */
	[INPUT_south] = { 0x4d, 0x21 }, /* Down, s */
	[INPUT_west] = { 0x4f, 0x20 }, /* Left, a */
	[INPUT_shoot] = { 0x65, 0x64 }, /* Alt_R, Alt_L */
	[INPUT_abort] = { 0x45, 0x45 }, /* Escape, Escape */
};

int keyboard_open(void)
{
	int ply, button;
	for(ply = 0; ply < 2; ply++) {
		for(button = 0; button < INPUT_MAX; button++) {
			keyboard_keycode[ply][button] = defaultKeyCode[button][ply];
		}
	}
	return(0);
}

void keyboard_read(void)
{
	int ply, button;
	for(ply = 0; ply < 2; ply++) {
		for(button = 0; button < INPUT_MAX; button++) {
			int kc = keyboard_keycode[ply][button] & 127;
			keyboard_input[ply][button] = keyboardVector[kc >> 3] >> (kc & 7) & 1;
		}
	}
}

int keyboard_close(void)
{
	return(0);
}

char *keyboard_to_name(int kc)
{
	return(keyboardString[kc & 127]);
}

int keyboard_to_keycode(char *kn)
{
	int kc = 128;
	if(*kn) for(kc = 0; kc < 128; kc++) if(strcmp(kn, keyboardString[kc]) == 0) break;
	if(kc >= 128) kc = -1;
	return(kc);
}

/* ---------------------------------------------------------------------- */

/* handle joysticks */

char joystick_input[2][INPUT_MAX];

static struct IOStdReq gameportIOReq[2];
static struct Device *GamePortBase[2];

static BYTE gameportAskCType[2] = { GPCT_NOCONTROLLER, GPCT_NOCONTROLLER, };
static BYTE gameportSetCType[2] = { GPCT_ABSJOYSTICK, GPCT_ABSJOYSTICK, };

static struct GamePortTrigger gameportTrigger = { GPTF_DOWNKEYS | GPTF_UPKEYS, -1, 1, 1 };

static struct InputEvent gameportEvent[2];

int joystick_open(void)
{
	int ply;
	for(ply = 0; ply < 2; ply++) {
		InitIORequest((void *)(&gameportIOReq[ply]), &ioMsgPort, sizeof(gameportIOReq[ply]));
		if(OpenDevice("gameport.device", ply, (void *)(&gameportIOReq[ply]), 0)) {
			fprintf(stderr, "%s[%s]: %s: %s\n", global, local, "failed to open gameport device", DeviceFault(gameportIOReq[ply].io_Error));
			continue;
		}
		GamePortBase[ply] = gameportIOReq[ply].io_Device;

		Forbid();
		gameportIOReq[ply].io_Command = GPD_ASKCTYPE;
		gameportIOReq[ply].io_Data = &gameportAskCType[ply];
		gameportIOReq[ply].io_Length = sizeof(gameportAskCType[ply]);
		DoIO((void *)(&gameportIOReq[ply]));
		if(gameportAskCType[ply] == GPCT_NOCONTROLLER) {
			gameportIOReq[ply].io_Command = GPD_SETCTYPE;
			gameportIOReq[ply].io_Data = &gameportSetCType[ply];
			gameportIOReq[ply].io_Length = sizeof(gameportSetCType[ply]);
			DoIO((void *)(&gameportIOReq[ply]));
			gameportIOReq[ply].io_Command = GPD_SETTRIGGER;
			gameportIOReq[ply].io_Data = &gameportTrigger;
			gameportIOReq[ply].io_Length = sizeof(gameportTrigger);
			DoIO((void *)(&gameportIOReq[ply]));
		}
		Permit();
	}
	return(0);
}

void joystick_read(void)
{
	int ply;
	for(ply = 0; ply < 2; ply++) if(GamePortBase[ply]) {
		char *joy = joystick_input[ply];

		while(CheckIO((void *)(&gameportIOReq[ply]))) {
			WaitIO((void *)(&gameportIOReq[ply]));

			joy[INPUT_north] = (gameportEvent[ply].ie_Y < 0);
			joy[INPUT_east] = (gameportEvent[ply].ie_X > 0);
			joy[INPUT_south] = (gameportEvent[ply].ie_Y > 0);
			joy[INPUT_west] = (gameportEvent[ply].ie_X < 0);
			if(gameportEvent[ply].ie_Code != IECODE_NOBUTTON) {
				joy[INPUT_shoot] = (gameportEvent[ply].ie_Code & IECODE_UP_PREFIX) ? 0 : 1;
			}

			gameportIOReq[ply].io_Command = GPD_READEVENT;
			gameportIOReq[ply].io_Data = &gameportEvent[ply];
			gameportIOReq[ply].io_Length = sizeof(gameportEvent[ply]);
			SendIO((void *)(&gameportIOReq[ply]));
		}
	}
}

int joystick_close(void)
{
	int ply;
	for(ply = 0; ply < 2; ply++) if(GamePortBase[ply]) {
		AbortIO((void *)(&gameportIOReq[ply]));
		WaitIO((void *)(&gameportIOReq[ply]));
		if(gameportAskCType[ply] == GPCT_NOCONTROLLER) {
			gameportIOReq[ply].io_Command = GPD_SETCTYPE;
			gameportIOReq[ply].io_Data = &gameportAskCType[ply];
			gameportIOReq[ply].io_Length = sizeof(gameportAskCType[ply]);
			DoIO((void *)(&gameportIOReq[ply]));
		}
		CloseDevice((void *)(&gameportIOReq[ply]));
		GamePortBase[ply] = 0;
	}
	return(0);
}

/* ---------------------------------------------------------------------- */

/* handle audio */

static char sound_play[SOUND_MAX]; /* should probably be protected by a semaphore */
static int sound_ptr[SOUND_MAX]; /* play position in the sound */

static UBYTE *sounds_chip[SOUND_MAX]; /* sounds are copied to chipmem */

static struct IOAudio audioIOReq;
static struct Device *AudioBase;

static UBYTE audioAlloc[1] = { (1 << ADHARD_CHANNELS) - 1 };

static struct Task audioTask;
static ULONG audioStack[256];

static void audio_thread(void);

int audio_open(void)
{
	int i;

	InitIORequest((void *)(&audioIOReq), &ioMsgPort, sizeof(audioIOReq));
	audioIOReq.ioa_Length = 0;
	if(OpenDevice("audio.device", 0, (void *)(&audioIOReq), 0)) {
		fprintf(stderr, "%s[%s]: %s: %s\n", global, local, "failed to open audio device", AudioFault(audioIOReq.ioa_Request.io_Error));
		return(1);
	}
	AudioBase = audioIOReq.ioa_Request.io_Device;

	audioIOReq.ioa_Request.io_Message.mn_Node.ln_Pri = ADALLOC_MAXPREC;
	audioIOReq.ioa_Request.io_Command = ADCMD_ALLOCATE;
	audioIOReq.ioa_Request.io_Flags = ADIOF_NOWAIT;
	audioIOReq.ioa_Data = audioAlloc;
	audioIOReq.ioa_Length = sizeof(audioAlloc);
	if(BeginIO((void *)(&audioIOReq)), WaitIO((void *)(&audioIOReq))) {
		fprintf(stderr, "%s[%s]: %s: %s\n", global, local, "failed to allocate audio channels", AudioFault(audioIOReq.ioa_Request.io_Error));
		return(1);
	}

	for(i = 0; i < SOUND_MAX; i++) {
		if(sounds_chip[i] = AllocMem(sounds_length[i], MEMF_CHIP), sounds_chip[i] == 0) {
			fprintf(stderr, "%s[%s]: %s (%d)\n", global, local, "failed to allocate sounds", sounds_length[i]);
			return(1);
		}
	}
	for(i = 0; i < SOUND_MAX; i++) {
		register const unsigned char *in = sounds_data[i];
		register UBYTE *out = sounds_chip[i];
		register int len = sounds_length[i];
		do *out++ = ulaw_to_linear[*in++] >> 8; while(--len);
	}

	audioTask.tc_Node.ln_Type = NT_TASK;
	audioTask.tc_Node.ln_Name = "emerald audio thread";
	audioTask.tc_SigAlloc = SIGF_ABORT | SIGF_SINGLE;
	audioTask.tc_SPReg = audioStack + 254;
	audioTask.tc_SPLower = audioStack + 1;
	audioTask.tc_SPUpper = audioStack + 255;
	audioTask.tc_UserData = FindTask(0);
	InitList(&audioTask.tc_MemEntry);
	AddTask(&audioTask, audio_thread, 0);

	return(0);
}

void audio_write(struct LOGIC *lev)
{
	int i;
	for(i = 0; i < SOUND_MAX; i++) if(lev->sound[i]) {
		sound_play[i] = 1;
	}
	if(audioTask.tc_UserData) Signal(&audioTask, SIGF_SINGLE);
}

int audio_close(void)
{
	int i;
	if(audioTask.tc_UserData) {
		Signal(&audioTask, SIGF_ABORT); Wait(SIGF_CHILD);
		RemTask(&audioTask);
		audioTask.tc_UserData = 0;
	}
	if(AudioBase) {
		CloseDevice((void *)(&audioIOReq));
		AudioBase = 0;
	}
	for(i = 0; i < SOUND_MAX; i++) {
		if(sounds_chip[i]) { FreeMem(sounds_chip[i], sounds_length[i]); sounds_chip[i] = 0; }
	}
	return(0);
}

/* audio thread */

static int mix_play[MIXER_MAX]; /* sounds we have chosen to play */
static int mix_count;

#define BUFFER_SIZE 256

static struct MsgPort channelMsgPort;
static struct IOAudio channelIOReq[2][ADHARD_CHANNELS]; /* double buffer */

static void audio_thread(void)
{
	ULONG signals = 0;
	int dbl;
	int i;

	InitMsgPort(&channelMsgPort, SIGB_SINGLE);
	for(dbl = 0; dbl < 2; dbl++) for(i = 0; i < ADHARD_CHANNELS; i++) {
		channelIOReq[dbl][i] = audioIOReq;
		InitIORequest((void *)(&channelIOReq[dbl][i]), &channelMsgPort, sizeof(audioIOReq));
	}

	memset(sound_ptr, -1, sizeof(sound_ptr));

	dbl = 0;
	while((signals & SIGF_ABORT) == 0) {
		dbl = 1 - dbl;

		mix_count = 0;

		for(i = 0; i < SOUND_MAX; i++) if(sound_play[i]) {
			sound_play[i] = 0;
			sound_ptr[i] = 0;
		}
		for(i = 0; i < SOUND_MAX; i++) if(sound_ptr[i] != -1) {
			while(mix_count < MIXER_MAX) {
				mix_play[mix_count++] = i;
				if(i >= SOUND_boom) break;
			}
		}

		if(mix_count == 0) {
			signals = Wait(SIGF_ABORT | SIGF_SINGLE);
		} else {
			signals = SetSignal(0, SIGF_ABORT | SIGF_SINGLE);
		}

		for(i = 0; i < ADHARD_CHANNELS && i < mix_count; i++) {
			UBYTE *in = sounds_chip[mix_play[i]] + sound_ptr[mix_play[i]];
			int len = sounds_length[mix_play[i]] - sound_ptr[mix_play[i]];
			if(len > BUFFER_SIZE) len = BUFFER_SIZE;

			WaitIO((void *)(&channelIOReq[dbl][i]));
			channelIOReq[dbl][i].ioa_Request.io_Unit = (void *)(1 << i);
			channelIOReq[dbl][i].ioa_Request.io_Command = CMD_WRITE;
			channelIOReq[dbl][i].ioa_Request.io_Flags = ADIOF_PERVOL;
			channelIOReq[dbl][i].ioa_Data = in;
			channelIOReq[dbl][i].ioa_Length = len;
			channelIOReq[dbl][i].ioa_Period = 3579545 / 8000;
			channelIOReq[dbl][i].ioa_Volume = 64;
			channelIOReq[dbl][i].ioa_Cycles = 1;
			BeginIO((void *)(&channelIOReq[dbl][i]));
		}
		for(i = 0; i < SOUND_MAX; i++) if(sound_ptr[i] != -1) {
			sound_ptr[i] += BUFFER_SIZE;
			if(sound_ptr[i] >= sounds_length[i]) sound_ptr[i] = -1;
		}
	}
	for(dbl = 0; dbl < 2; dbl++) for(i = 0; i < ADHARD_CHANNELS; i++) {
		AbortIO((void *)(&channelIOReq[dbl][i]));
		WaitIO((void *)(&channelIOReq[dbl][i]));
	}
	Signal(audioTask.tc_UserData, SIGF_CHILD); Wait(0);
}

/* ---------------------------------------------------------------------- */

/* create rastports, create bitmaps */

static struct RastPort objectPixmap;
static struct RastPort hiddenPixmap;
static struct RastPort spritePixmap;
static struct RastPort spriteBitmap;

static struct BitMap objectPM;
static struct BitMap hiddenPM;
static struct BitMap spritePM;
static struct BitMap spriteBM;

int screen_open(void)
{
	int a;

	InitRastPort(&objectPixmap);
	objectPixmap.BitMap = &objectPM;
	InitBitMap(&objectPM, PLANES, GFX_WIDTH, GFX_HEIGHT);
	for(a = 0; a < PLANES; a++) if(objectPM.Planes[a] = AllocRaster(GFX_WIDTH, GFX_HEIGHT), objectPM.Planes[a] == 0) {
		fprintf(stderr, "%s[%s]: %s (%dx%dx%d)\n", global, local, "failed to allocate graphics pixmap", GFX_WIDTH, GFX_HEIGHT, PLANES);
		return(1);
	}

	InitRastPort(&hiddenPixmap);
	hiddenPixmap.BitMap = &hiddenPM;
	InitBitMap(&hiddenPM, PLANES, 32, 32);
	if(hiddenPM.Planes[0] = AllocRaster(32, 32 * PLANES), hiddenPM.Planes[0] == 0) {
		fprintf(stderr, "%s[%s]: %s (%dx%dx%d)\n", global, local, "failed to allocate hidden pixmap", 32, 32, PLANES);
		return(1);
	}
	for(a = 1; a < PLANES; a++) hiddenPM.Planes[a] = hiddenPM.Planes[0] + hiddenPM.BytesPerRow * hiddenPM.Rows * a;

	InitRastPort(&spritePixmap);
	spritePixmap.BitMap = &spritePM;
	InitBitMap(&spritePM, PLANES, 16, 16);
	if(spritePM.Planes[0] = AllocRaster(16, 16 * PLANES), spritePM.Planes[0] == 0) {
		fprintf(stderr, "%s[%s]: %s (%dx%dx%d)\n", global, local, "failed to allocate sprite pixmap", 16, 16, PLANES);
		return(1);
	}
	for(a = 1; a < PLANES; a++) spritePM.Planes[a] = spritePM.Planes[0] + spritePM.BytesPerRow * spritePM.Rows * a;

	InitRastPort(&spriteBitmap);
	spriteBitmap.BitMap = &spriteBM;
	InitBitMap(&spriteBM, PLANES, 16, 16);
	if(spriteBM.Planes[0] = AllocRaster(16, 16), spriteBM.Planes[0] == 0) {
		fprintf(stderr, "%s[%s]: %s (%dx%dx%d)\n", global, local, "failed to allocate sprite bitmap", 16, 16, 1);
		return(1);
	}
	for(a = 1; a < PLANES; a++) spriteBM.Planes[a] = spriteBM.Planes[0];

	SetAPen(&spriteBitmap, 0);

	return(screen_write());
}

/* render graphics */

static struct BitMap pixmapImage;

int screen_write(void)
{
	int in_x, out_x, y;

	InitBitMap(&pixmapImage, 8, GFX_WIDTH, 1);
	if(pixmapImage.Planes[0] = AllocRaster(GFX_WIDTH, 8), pixmapImage.Planes[0] == 0) {
		fprintf(stderr, "%s[%s]: %s (%dx%dx%d)\n", global, local, "failed to allocate pixmap image", GFX_WIDTH, 1, 8);
		return(1);
	}
	for(y = 1; y < 8; y++) pixmapImage.Planes[y] = pixmapImage.Planes[0] + pixmapImage.BytesPerRow * pixmapImage.Rows * y;

	for(y = 0; y < GFX_HEIGHT; y++) {
		out_x = 0;
		for(in_x = 0; in_x < GFX_WIDTH; in_x += 8) {
			char bits[8]; /* graphics.library/WritePixelLine8 */
			char mask;

			bits[0] = paletteHash[graphics_xpm[in_x + 0][y] & 127];
			bits[1] = paletteHash[graphics_xpm[in_x + 1][y] & 127];
			bits[2] = paletteHash[graphics_xpm[in_x + 2][y] & 127];
			bits[3] = paletteHash[graphics_xpm[in_x + 3][y] & 127];
			bits[4] = paletteHash[graphics_xpm[in_x + 4][y] & 127];
			bits[5] = paletteHash[graphics_xpm[in_x + 5][y] & 127];
			bits[6] = paletteHash[graphics_xpm[in_x + 6][y] & 127];
			bits[7] = paletteHash[graphics_xpm[in_x + 7][y] & 127];

			mask = (bits[0] ^ bits[1] >> 1) & 0x55; bits[0] ^= mask; bits[1] ^= mask << 1;
			mask = (bits[2] ^ bits[3] >> 1) & 0x55; bits[2] ^= mask; bits[3] ^= mask << 1;
			mask = (bits[4] ^ bits[5] >> 1) & 0x55; bits[4] ^= mask; bits[5] ^= mask << 1;
			mask = (bits[6] ^ bits[7] >> 1) & 0x55; bits[6] ^= mask; bits[7] ^= mask << 1;

			mask = (bits[0] ^ bits[2] >> 2) & 0x33; bits[0] ^= mask; bits[2] ^= mask << 2;
			mask = (bits[1] ^ bits[3] >> 2) & 0x33; bits[1] ^= mask; bits[3] ^= mask << 2;
			mask = (bits[4] ^ bits[6] >> 2) & 0x33; bits[4] ^= mask; bits[6] ^= mask << 2;
			mask = (bits[5] ^ bits[7] >> 2) & 0x33; bits[5] ^= mask; bits[7] ^= mask << 2;

			mask = (bits[0] ^ bits[4] >> 4) & 0x0f; bits[0] ^= mask; bits[4] ^= mask << 4;
			mask = (bits[1] ^ bits[5] >> 4) & 0x0f; bits[1] ^= mask; bits[5] ^= mask << 4;
			mask = (bits[2] ^ bits[6] >> 4) & 0x0f; bits[2] ^= mask; bits[6] ^= mask << 4;
			mask = (bits[3] ^ bits[7] >> 4) & 0x0f; bits[3] ^= mask; bits[7] ^= mask << 4;

			pixmapImage.Planes[0][out_x] = bits[7];
			pixmapImage.Planes[1][out_x] = bits[6];
			pixmapImage.Planes[2][out_x] = bits[5];
			pixmapImage.Planes[3][out_x] = bits[4];
			pixmapImage.Planes[4][out_x] = bits[3];
			pixmapImage.Planes[5][out_x] = bits[2];
			pixmapImage.Planes[6][out_x] = bits[1];
			pixmapImage.Planes[7][out_x] = bits[0];

			out_x++;
		}
		BltBitMapRastPort(&pixmapImage, 0, 0, &objectPixmap, 0, y, GFX_WIDTH, 1, ABC|ABNC);
		WaitBlit();
	}

	FreeRaster(pixmapImage.Planes[0], GFX_WIDTH, 8); pixmapImage.Planes[0] = 0;
	return(0);
}

int screen_close(void)
{
	int a;
	for(a = 0; a < PLANES; a++) if(objectPM.Planes[a]) { FreeRaster(objectPM.Planes[a], GFX_WIDTH, GFX_HEIGHT); objectPM.Planes[a] = 0; }
	if(hiddenPM.Planes[0]) { FreeRaster(hiddenPM.Planes[0], 32, 32 * PLANES); hiddenPM.Planes[0] = 0; }
	if(spritePM.Planes[0]) { FreeRaster(spritePM.Planes[0], 16, 16 * PLANES); spritePM.Planes[0] = 0; }
	if(spriteBM.Planes[0]) { FreeRaster(spriteBM.Planes[0], 16, 16); spriteBM.Planes[0] = 0; }
	if(pixmapImage.Planes[0]) { FreeRaster(pixmapImage.Planes[0], GFX_WIDTH, 8); pixmapImage.Planes[0] = 0; }

	return(0);
}

/* ---------------------------------------------------------------------- */

/* game drawing functions */

static short screen_cache[(VIEW_WIDTH + SCROLL_WIDTH) / 16][(VIEW_HEIGHT + SCROLL_HEIGHT) / 16];

static int scn_pixel_left;
static int scn_pixel_top;

void x11_game_clear(void)
{
	SetAPen(&screenPixmap, COLOUR_black);
	RectFill(&screenPixmap, 0, 0, VIEW_WIDTH + SCROLL_WIDTH - 1, VIEW_HEIGHT + SCROLL_HEIGHT - 1);
	memset(screen_cache, -1, sizeof(screen_cache)); /* mark entire screen as dirty */
}

static void render_update(struct LOGIC *lev)
{
	int tile;
	int ply;
	int dx = (lev->pixel_left - scn_pixel_left);
	int dy = (lev->pixel_top - scn_pixel_top);
	int x0, x1;
	int y0, y1;

/* do not draw background over player */

	for(ply = 0; ply < 2; ply++) {
		if(lev->hide[ply]) continue;

		x0 = lev->pl_pixel_left[ply];
		x1 = lev->pl_pixel_right[ply];
		y0 = lev->pl_pixel_top[ply];
		y1 = lev->pl_pixel_bottom[ply];
		if(!(x0 >= lev->pixel_left && x1 <= lev->pixel_right && y0 >= lev->pixel_top && y1 <= lev->pixel_bottom)) continue;

		tile = object_map[lev->cave[x0 / 16][y0 / 16]][lev->frame];
		screen_cache[(x0 - dx) / 16][(y0 - dy) / 16] = tile;

		tile = object_map[lev->cave[x1 / 16][y1 / 16]][lev->frame];
		screen_cache[(x1 - dx) / 16][(y1 - dy) / 16] = tile;
	}
}

static void render_object(struct LOGIC *lev)
{
	int tile, tile_x, tile_y;
	int scn_x, lev_x;
	int scn_y, lev_y;
	scn_x = scn_pixel_left / 16;
	for(lev_x = lev->view_left; lev_x <= lev->view_right; lev_x++) {
		scn_y = scn_pixel_top / 16;
		for(lev_y = lev->view_top; lev_y <= lev->view_bottom; lev_y++) {
			tile = object_map[lev->cave[lev_x][lev_y]][lev->frame];
			if(screen_cache[scn_x][scn_y] != tile) {
				screen_cache[scn_x][scn_y] = tile;
				tile_x = (tile / GFX_HEIGHT) * 16;
				tile_y = (tile % GFX_HEIGHT);
				ClipBlit(&objectPixmap, tile_x, tile_y, &screenPixmap, scn_x * 16, scn_y * 16, 16, 16, ABC|ABNC);
			}
			scn_y++;
		}
		scn_x++;
	}
}

static void render_player(struct LOGIC *lev)
{
	int tile, tile_x, tile_y;
	int ply;
	int dx = (lev->pixel_left - scn_pixel_left);
	int dy = (lev->pixel_top - scn_pixel_top);
	int x0, x1;
	int y0, y1;
	for(ply = 0; ply < 2; ply++) {
		if(lev->hide[ply]) continue;

		x0 = lev->pl_pixel_left[ply];
		x1 = lev->pl_pixel_right[ply];
		y0 = lev->pl_pixel_top[ply];
		y1 = lev->pl_pixel_bottom[ply];
		if(!(x0 >= lev->pixel_left && x1 <= lev->pixel_right && y0 >= lev->pixel_top && y1 <= lev->pixel_bottom)) continue;

		x0 -= dx; x1 -= dx;
		y0 -= dy; y1 -= dy;

		tile = screen_cache[x0 / 16][y0 / 16];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		ClipBlit(&objectPixmap, tile_x, tile_y, &hiddenPixmap, 0, 0, 16, 16, ABC|ABNC);
		tile = screen_cache[x1 / 16][y1 / 16];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		ClipBlit(&objectPixmap, tile_x, tile_y, &hiddenPixmap, (x0 % 16 == 0 ? 0 : 16), (y0 % 16 == 0 ? 0 : 16), 16, 16, ABC|ABNC);

		tile = sprite_map[lev->pl_anim[ply]][ply][lev->frame];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		ClipBlit(&objectPixmap, tile_x, tile_y, &spritePixmap, 0, 0, 16, 16, ABC|ABNC);

		RectFill(&spriteBitmap, 0, 0, 16 - 1, 16 - 1);
		ClipBlit(&hiddenPixmap, x0 % 16, y0 % 16, &spriteBitmap, 0, 0, 16, 16, ABC|ABNC|ANBC);
		ClipBlit(&spriteBitmap, 0, 0, &spritePixmap, 0, 0, 16, 16, ANBC);
		ClipBlit(&spritePixmap, 0, 0, &hiddenPixmap, x0 % 16, y0 % 16, 16, 16, ABC|ABNC|ANBC);

		x0 /= 16; x1 /= 16;
		y0 /= 16; y1 /= 16;
		ClipBlit(&hiddenPixmap, 0, 0, &screenPixmap, x0 * 16, y0 * 16, (x1 - x0 + 1) * 16, (y1 - y0 + 1) * 16, ABC|ABNC);
	}
}

static void render_damage(struct LOGIC *lev)
{
	int ply;
	int dx = (lev->pixel_left - scn_pixel_left);
	int dy = (lev->pixel_top - scn_pixel_top);
	int x0, x1;
	int y0, y1;

/* mark screen over player as dirty */

	for(ply = 0; ply < 2; ply++) {
		if(lev->hide[ply]) continue;

		x0 = lev->pl_pixel_left[ply];
		x1 = lev->pl_pixel_right[ply];
		y0 = lev->pl_pixel_top[ply];
		y1 = lev->pl_pixel_bottom[ply];
		if(!(x0 >= lev->pixel_left && x1 <= lev->pixel_right && y0 >= lev->pixel_top && y1 <= lev->pixel_bottom)) continue;

		screen_cache[(x0 - dx) / 16][(y0 - dy) / 16] = -1;
		screen_cache[(x1 - dx) / 16][(y1 - dy) / 16] = -1;
	}
}

void x11_game_scroll(struct LOGIC *lev)
{
	scn_pixel_left = lev->pixel_left % SCROLL_WIDTH;
	scn_pixel_top = lev->pixel_top % SCROLL_HEIGHT;

	render_update(lev);
	render_object(lev);
	render_player(lev);
	render_damage(lev);

	viewport->RasInfo->RxOffset = scn_pixel_left;
	viewport->RasInfo->RyOffset = scn_pixel_top;
	ScrollVPort(viewport);
}

void x11_game_object(struct LOGIC *lev) { }
void x11_game_player(struct LOGIC *lev) { }
void x11_game_score(struct LOGIC *lev) { }
void x11_game_to_screen(void) { }

/* ---------------------------------------------------------------------- */

/* score drawing functions */

static short score_cache[SCORE_WIDTH / 8];

void x11_score_clear(void)
{
	SetAPen(&scoreScreen->RastPort, COLOUR_black);
	RectFill(&scoreScreen->RastPort, 0, 0, SCORE_WIDTH - 1, 8 - 1);
	memset(score_cache, -1, sizeof(score_cache));
}

void x11_score_print(char str[40])
{
	int tile, tile_x, tile_y;
	int x;
	for(x = 0; x < SCORE_WIDTH / 8; x++) {
		tile = score_map[str[x] & 127];
		if(score_cache[x] != tile) {
			score_cache[x] = tile;
			tile_x = (tile / GFX_HEIGHT) * 16;
			tile_y = (tile % GFX_HEIGHT);
			ClipBlit(&objectPixmap, tile_x, tile_y, &scoreScreen->RastPort, x * 8, 0, 8, 8, ABC|ABNC);
		}
	}
}

/* ---------------------------------------------------------------------- */

/* menu drawing functions */

static int antsPos;

void x11_menu_clear(void)
{
	SetAPen(&screenPixmap, COLOUR_black);
	RectFill(&screenPixmap, 0, 0, VIEW_WIDTH - 1, VIEW_HEIGHT - 1);
}

void x11_menu_print(char str[26], int top)
{
	int tile, tile_x, tile_y;
	int x;
	SetAPen(&screenPixmap, COLOUR_black);
	RectFill(&screenPixmap, 0, top * 16, VIEW_WIDTH - 1, (top + 1) * 16 - 1);
	for(x = 0; x < 26; x++) {
		tile = font_map[str[x] & 127];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		ClipBlit(&objectPixmap, tile_x, tile_y, &screenPixmap, x * 12 + 4, top * 16, 16, 16, ABC|ABNC);
	}
}

void x11_menu_ants(int left, int right, int top)
{
	int x, x0 = left, x1 = (right + 1);
	int y, y0 = top * 16, y1 = (top + 1) * 16;
	int pos, inc;

	antsPos = (antsPos + 1) % 16;
	pos = antsPos;

	for(x = x0; x < x1; x += inc) {
		inc = 8 - pos % 8; if(inc > x1 - x) inc = x1 - x;
		SetAPen(&screenPixmap, pos / 8 % 2 ? COLOUR_white : COLOUR_red);
		RectFill(&screenPixmap, x, y0, x + inc - 1, y0);
		pos += inc;
	}
	for(y = y0; y < y1; y += inc) {
		inc = 8 - pos % 8; if(inc > y1 - y) inc = y1 - y;
		SetAPen(&screenPixmap, pos / 8 % 2 ? COLOUR_white : COLOUR_red);
		RectFill(&screenPixmap, x1 - 1, y, x1 - 1, y + inc - 1);
		pos += inc;
	}
	for(x = x1; x > x0; x -= inc) {
		inc = 8 - pos % 8; if(inc > x - x0) inc = x - x0;
		SetAPen(&screenPixmap, pos / 8 % 2 ? COLOUR_white : COLOUR_red);
		RectFill(&screenPixmap, x - inc, y1 - 1, x - 1, y1 - 1);
		pos += inc;
	}
	for(y = y1; y > y0; y -= inc) {
		inc = 8 - pos % 8; if(inc > y - y0) inc = y - y0;
		SetAPen(&screenPixmap, pos / 8 % 2 ? COLOUR_white : COLOUR_red);
		RectFill(&screenPixmap, x0, y - inc, x0, y - 1);
		pos += inc;
	}
}

void x11_menu_to_screen(void)
{
	viewport->RasInfo->RxOffset = 0;
	viewport->RasInfo->RyOffset = 0;
	ScrollVPort(viewport);
}
