/* ulaw<->linear conversion tables for emerald mine */

#include "proto.h"

#if 0

int ulaw_to_linear(unsigned char);
unsigned char linear_to_ulaw(int);

void generate(short utol[256], unsigned char ltou[16384])
{
	int i;
	for(i = 0; i < 256; i++) {
		utol[i] = ulaw_to_linear(i);
	}
	for(i = -8192; i < 8192; i++) {
		ltou[i & 16383] = linear_to_ulaw(i * 4 + 2);
	}
}

/*
** This routine converts from ulaw to 16 bit linear.
**
** Craig Reese: IDA/Supercomputing Research Center
** 29 September 1989
**
** References:
** 1) CCITT Recommendation G.711  (very difficult to follow)
** 2) MIL-STD-188-113,"Interoperability and Performance Standards
**     for Analog-to_Digital Conversion Techniques,"
**     17 February 1987
**
** Input: 8 bit ulaw sample
** Output: signed 16 bit linear sample
*/

int ulaw_to_linear(unsigned char ulawbyte)
{
	static int exp_lut[8] = { 0, 132, 396, 924, 1980, 4092, 8316, 16764 };
	int sign, exponent, mantissa, sample;

	ulawbyte = ~ ulawbyte;
	sign = ( ulawbyte & 0x80 );
	exponent = ( ulawbyte >> 4 ) & 0x07;
	mantissa = ulawbyte & 0x0F;
	sample = exp_lut[exponent] + ( mantissa << ( exponent + 3 ) );
	if (sign != 0)
		sample = -sample;

	return(sample);
}

/*
** This routine converts from linear to ulaw.
**
** Craig Reese: IDA/Supercomputing Research Center
** Joe Campbell: Department of Defense
** 29 September 1989
**
** References:
** 1) CCITT Recommendation G.711  (very difficult to follow)
** 2) "A New Digital Technique for Implementation of Any
**     Continuous PCM Companding Law," Villeret, Michel,
**     et al. 1973 IEEE Int. Conf. on Communications, Vol 1,
**     1973, pg. 11.12-11.17
** 3) MIL-STD-188-113,"Interoperability and Performance Standards
**     for Analog-to_Digital Conversion Techniques,"
**     17 February 1987
**
** Input: Signed 16 bit linear sample
** Output: 8 bit ulaw sample
*/

#define ZEROTRAP    /* turn on the trap as per the MIL-STD */
#define BIAS 0x84   /* define the add-in bias for 16 bit samples */
#define CLIP 32635

unsigned char linear_to_ulaw(int sample)
{
	static int exp_lut[256] =
	{
		0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
		4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
		5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
		5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7
	};

	int sign, exponent, mantissa;
	unsigned char ulawbyte;

/* Get the sample into sign-magnitude. */
	sign = (sample >> 8) & 0x80; /* set aside the sign */
	if (sign != 0)
		sample = -sample; /* get magnitude */
	if (sample > CLIP)
		sample = CLIP; /* clip the magnitude */

/* Convert from 16 bit linear to ulaw. */
	sample = sample + BIAS;
	exponent = exp_lut[( sample >> 7 ) & 0xFF];
	mantissa = ( sample >> ( exponent + 3 ) ) & 0x0F;
	ulawbyte = ~ ( sign | ( exponent << 4 ) | mantissa );
#ifdef ZEROTRAP
	if (ulawbyte == 0)
		ulawbyte = 0x02; /* optional CCITT trap */
#endif

	return(ulawbyte);
}

#endif

const short ulaw_to_linear[256] = {
	-32124,-31100,-30076,-29052,-28028,-27004,-25980,-24956,
	-23932,-22908,-21884,-20860,-19836,-18812,-17788,-16764,
	-15996,-15484,-14972,-14460,-13948,-13436,-12924,-12412,
	-11900,-11388,-10876,-10364,-9852,-9340,-8828,-8316,
	-7932,-7676,-7420,-7164,-6908,-6652,-6396,-6140,
	-5884,-5628,-5372,-5116,-4860,-4604,-4348,-4092,
	-3900,-3772,-3644,-3516,-3388,-3260,-3132,-3004,
	-2876,-2748,-2620,-2492,-2364,-2236,-2108,-1980,
	-1884,-1820,-1756,-1692,-1628,-1564,-1500,-1436,
	-1372,-1308,-1244,-1180,-1116,-1052,-988,-924,
	-876,-844,-812,-780,-748,-716,-684,-652,
	-620,-588,-556,-524,-492,-460,-428,-396,
	-372,-356,-340,-324,-308,-292,-276,-260,
	-244,-228,-212,-196,-180,-164,-148,-132,
	-120,-112,-104,-96,-88,-80,-72,-64,
	-56,-48,-40,-32,-24,-16,-8,0,
	32124,31100,30076,29052,28028,27004,25980,24956,
	23932,22908,21884,20860,19836,18812,17788,16764,
	15996,15484,14972,14460,13948,13436,12924,12412,
	11900,11388,10876,10364,9852,9340,8828,8316,
	7932,7676,7420,7164,6908,6652,6396,6140,
	5884,5628,5372,5116,4860,4604,4348,4092,
	3900,3772,3644,3516,3388,3260,3132,3004,
	2876,2748,2620,2492,2364,2236,2108,1980,
	1884,1820,1756,1692,1628,1564,1500,1436,
	1372,1308,1244,1180,1116,1052,988,924,
	876,844,812,780,748,716,684,652,
	620,588,556,524,492,460,428,396,
	372,356,340,324,308,292,276,260,
	244,228,212,196,180,164,148,132,
	120,112,104,96,88,80,72,64,
	56,48,40,32,24,16,8,0,
};

const unsigned char linear_to_ulaw[16384] = {
	255,254,254,253,253,252,252,251,251,250,250,249,249,248,248,247,
	247,246,246,245,245,244,244,243,243,242,242,241,241,240,240,239,
	239,239,239,238,238,238,238,237,237,237,237,236,236,236,236,235,
	235,235,235,234,234,234,234,233,233,233,233,232,232,232,232,231,
	231,231,231,230,230,230,230,229,229,229,229,228,228,228,228,227,
	227,227,227,226,226,226,226,225,225,225,225,224,224,224,224,223,
	223,223,223,223,223,223,223,222,222,222,222,222,222,222,222,221,
	221,221,221,221,221,221,221,220,220,220,220,220,220,220,220,219,
	219,219,219,219,219,219,219,218,218,218,218,218,218,218,218,217,
	217,217,217,217,217,217,217,216,216,216,216,216,216,216,216,215,
	215,215,215,215,215,215,215,214,214,214,214,214,214,214,214,213,
	213,213,213,213,213,213,213,212,212,212,212,212,212,212,212,211,
	211,211,211,211,211,211,211,210,210,210,210,210,210,210,210,209,
	209,209,209,209,209,209,209,208,208,208,208,208,208,208,208,207,
	207,207,207,207,207,207,207,207,207,207,207,207,207,207,207,206,
	206,206,206,206,206,206,206,206,206,206,206,206,206,206,206,205,
	205,205,205,205,205,205,205,205,205,205,205,205,205,205,205,204,
	204,204,204,204,204,204,204,204,204,204,204,204,204,204,204,203,
	203,203,203,203,203,203,203,203,203,203,203,203,203,203,203,202,
	202,202,202,202,202,202,202,202,202,202,202,202,202,202,202,201,
	201,201,201,201,201,201,201,201,201,201,201,201,201,201,201,200,
	200,200,200,200,200,200,200,200,200,200,200,200,200,200,200,199,
	199,199,199,199,199,199,199,199,199,199,199,199,199,199,199,198,
	198,198,198,198,198,198,198,198,198,198,198,198,198,198,198,197,
	197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,196,
	196,196,196,196,196,196,196,196,196,196,196,196,196,196,196,195,
	195,195,195,195,195,195,195,195,195,195,195,195,195,195,195,194,
	194,194,194,194,194,194,194,194,194,194,194,194,194,194,194,193,
	193,193,193,193,193,193,193,193,193,193,193,193,193,193,193,192,
	192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,191,
	191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,
	191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,190,
	190,190,190,190,190,190,190,190,190,190,190,190,190,190,190,190,
	190,190,190,190,190,190,190,190,190,190,190,190,190,190,190,189,
	189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,
	189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,188,
	188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
	188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,187,
	187,187,187,187,187,187,187,187,187,187,187,187,187,187,187,187,
	187,187,187,187,187,187,187,187,187,187,187,187,187,187,187,186,
	186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,
	186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,185,
	185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
	185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,184,
	184,184,184,184,184,184,184,184,184,184,184,184,184,184,184,184,
	184,184,184,184,184,184,184,184,184,184,184,184,184,184,184,183,
	183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,
	183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,182,
	182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
	182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,181,
	181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,
	181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,180,
	180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,
	180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,179,
	179,179,179,179,179,179,179,179,179,179,179,179,179,179,179,179,
	179,179,179,179,179,179,179,179,179,179,179,179,179,179,179,178,
	178,178,178,178,178,178,178,178,178,178,178,178,178,178,178,178,
	178,178,178,178,178,178,178,178,178,178,178,178,178,178,178,177,
	177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
	177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,176,
	176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,
	176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,175,
	175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,
	175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,
	175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,
	175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,174,
	174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
	174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
	174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
	174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,173,
	173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,
	173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,
	173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,
	173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,172,
	172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,
	172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,
	172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,
	172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,171,
	171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
	171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
	171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
	171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,170,
	170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,
	170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,
	170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,
	170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,169,
	169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,
	169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,
	169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,
	169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,168,
	168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,
	168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,
	168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,
	168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,167,
	167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
	167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
	167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
	167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,166,
	166,166,166,166,166,166,166,166,166,166,166,166,166,166,166,166,
	166,166,166,166,166,166,166,166,166,166,166,166,166,166,166,166,
	166,166,166,166,166,166,166,166,166,166,166,166,166,166,166,166,
	166,166,166,166,166,166,166,166,166,166,166,166,166,166,166,165,
	165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,
	165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,
	165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,
	165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,164,
	164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,
	164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,
	164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,
	164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,163,
	163,163,163,163,163,163,163,163,163,163,163,163,163,163,163,163,
	163,163,163,163,163,163,163,163,163,163,163,163,163,163,163,163,
	163,163,163,163,163,163,163,163,163,163,163,163,163,163,163,163,
	163,163,163,163,163,163,163,163,163,163,163,163,163,163,163,162,
	162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,
	162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,
	162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,
	162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,161,
	161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,
	161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,
	161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,
	161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,160,
	160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,
	160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,
	160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,
	160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
	159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
	158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
	157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
	156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
	155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
	154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
	153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
	152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
	151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
	149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
	148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
	147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,
	146,146,146,146,146,146,146,146,146,146,146,146,146,146,146,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
	145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
	144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
	143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
	142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
	141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
	140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
	139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,
	138,138,138,138,138,138,138,138,138,138,138,138,138,138,138,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,
	137,137,137,137,137,137,137,137,137,137,137,137,137,137,137,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
	136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
	135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
	134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
	133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
	132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
	131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
	130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
	129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	9,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
	10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	11,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	12,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	13,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	14,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	15,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
	16,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,
	17,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,
	18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
	19,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
	20,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
	21,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,
	22,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,23,
	23,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,24,
	24,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,
	25,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,
	26,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,
	27,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,
	28,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,
	29,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,
	30,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,
	31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
	32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
	32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
	32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
	32,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,
	33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,
	33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,
	33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,
	33,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,
	34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,
	34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,
	34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,
	34,35,35,35,35,35,35,35,35,35,35,35,35,35,35,35,
	35,35,35,35,35,35,35,35,35,35,35,35,35,35,35,35,
	35,35,35,35,35,35,35,35,35,35,35,35,35,35,35,35,
	35,35,35,35,35,35,35,35,35,35,35,35,35,35,35,35,
	35,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,
	36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,
	36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,
	36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,
	36,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,
	37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,
	37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,
	37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,
	37,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,
	38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,
	38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,
	38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,
	38,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,
	39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,
	39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,
	39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,
	39,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,
	40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,
	40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,
	40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,
	40,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,
	41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,
	41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,
	41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,
	41,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,
	42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,
	42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,
	42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,
	42,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,
	43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,
	43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,
	43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,
	43,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,
	44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,
	44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,
	44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,
	44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,
	45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,
	45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,
	45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,
	45,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,
	46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,
	46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,
	46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,
	46,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
	47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
	47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
	47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
	47,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,
	48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,
	48,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,
	49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,
	49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,
	50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,
	50,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,
	51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,
	51,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,
	52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,
	52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,
	53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,
	53,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,
	54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,
	54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,
	55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,
	55,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,
	56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,
	56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,
	57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,
	57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,
	58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,
	58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,
	59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,
	59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,
	60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,
	60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,
	61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,
	61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,
	62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,
	62,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,
	63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,
	63,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
	64,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,
	65,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,
	66,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,
	67,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,
	68,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,
	69,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,
	70,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,
	71,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,
	72,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,
	73,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,
	74,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,
	75,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,
	76,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,
	77,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,
	78,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,
	79,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,
	81,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,
	83,84,84,84,84,84,84,84,84,85,85,85,85,85,85,85,
	85,86,86,86,86,86,86,86,86,87,87,87,87,87,87,87,
	87,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,
	89,90,90,90,90,90,90,90,90,91,91,91,91,91,91,91,
	91,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,
	93,94,94,94,94,94,94,94,94,95,95,95,95,95,95,95,
	95,96,96,96,96,97,97,97,97,98,98,98,98,99,99,99,
	99,100,100,100,100,101,101,101,101,102,102,102,102,103,103,103,
	103,104,104,104,104,105,105,105,105,106,106,106,106,107,107,107,
	107,108,108,108,108,109,109,109,109,110,110,110,110,111,111,111,
	111,112,112,113,113,114,114,115,115,116,116,117,117,118,118,119,
	119,120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,
};
