/* gunzip for emerald mine caves */

#include <stdio.h>
#include <string.h>

static int canonical_huffman_tree(short tree[576], int ntrees, const unsigned char bit[288], int nbits)
{
	int avbl = 2, dpth = 1, root = 0, next = 2;

	while(avbl > 0) {
		int n;
		for(n = 0; n < nbits; n++) if(bit[n] == dpth) {

			if(root >= ntrees) return(1);
			tree[root++] = ~n;
			avbl--;
		}
		for(n = 0; n < avbl; n++) {

			if(root >= ntrees) return(1);
			tree[root++] = next;
			next += 2;
		}
		avbl *= 2; dpth += 1;
	}
	if(avbl != 0) return(1);

	return(0);
}

static unsigned char dictionary[32768];
static short length_tree[576];
static short pointer_tree[64];
static unsigned char huffman_bit[320];

enum {
	FHCRC = 2,
	FEXTRA = 4,
	FNAME = 8,
	FCOMMENT = 16,

	BSTORED = 0,
	BFIXED = 2,
	BDYNAMIC = 4,
};

static const unsigned char order[19] = { 16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15, };

#define RPEEK(x) { while(shift < (x)) { if(ch = getc(in_file), ch == EOF) return "read error"; code += ch << shift; shift += 8; } }
#define RSHIFT(x) { code >>= (x); shift -= (x); }

char *gunzip(FILE *in_file, FILE *out_file)
{
	register int code = 0, shift = 0;

	int n, ch, huf, rep;

	int btype = 0;

	int dic = 0;

	memset(dictionary, 0, sizeof(dictionary));

	/* gzip header */

	for(n = 0; n < 10; n++) {
		RPEEK(8); huf = code & 255; RSHIFT(8);
		dictionary[n] = huf;
	}
	if(!(dictionary[0] == 0x1f && dictionary[1] == 0x8b && dictionary[2] == 0x08)) return "not a deflated gzip file";

	if((dictionary[3] & FEXTRA) != 0) {
		RPEEK(16); rep = code & 65535; RSHIFT(16);
		for(n = 0; n < rep; n++) {
			RPEEK(8); RSHIFT(8);
		}
	}
	if((dictionary[3] & FNAME) != 0) {
		for(n = 0; n < 4096; n++) {
			RPEEK(8); huf = code & 255; RSHIFT(8);
			if(huf == 0) break;
		}
		if(n >= 4096) return "name too long";
	}
	if((dictionary[3] & FCOMMENT) != 0) {
		for(n = 0; n < 65536; n++) {
			RPEEK(8); huf = code & 255; RSHIFT(8);
			if(huf == 0) break;
		}
		if(n >= 65536) return "comment too long";
	}
	if((dictionary[3] & FHCRC) != 0) {
		RPEEK(16); RSHIFT(16);
	}

	/* block loop */

	while((btype & 1) == 0) {

		int bit, len, ptr;

		int nlen, nptr, nhdr;

		RPEEK(3); btype = code & 7; RSHIFT(3);

		if((btype & 6) == BSTORED) {

			bit = shift & 7; RSHIFT(bit);

			RPEEK(16); rep = code & 65535; RSHIFT(16);
			RPEEK(16); RSHIFT(16);

			for(n = 0; n < rep; n++) {
				RPEEK(8); huf = code & 255; RSHIFT(8);
				dictionary[(dic++) & 32767] = huf;

				if((dic & 32767) == 0) if(fwrite(dictionary, 32768, 1, out_file) != 1) return "write error";
			}
			continue;

		} else if((btype & 6) == BFIXED) {

			for(n = 0; n < 144; n++) huffman_bit[n] = 8;
			for(n = 144; n < 256; n++) huffman_bit[n] = 9;
			for(n = 256; n < 280; n++) huffman_bit[n] = 7;
			for(n = 280; n < 288; n++) huffman_bit[n] = 8;
			for(n = 288; n < 320; n++) huffman_bit[n] = 5;

			nlen = 288; nptr = 32; nhdr = 0;

		} else if((btype & 6) == BDYNAMIC) {

			for(n = 0; n < 320; n++) huffman_bit[n] = 0;

			RPEEK(5); nlen = (code & 31) + 257; RSHIFT(5);
			RPEEK(5); nptr = (code & 31) + 1; RSHIFT(5);
			RPEEK(4); nhdr = (code & 15) + 4; RSHIFT(4);

			for(n = 0; n < nhdr; n++) {
				RPEEK(3); huf = code & 7; RSHIFT(3);
				huffman_bit[order[n]] = huf;
			}
			if(canonical_huffman_tree(length_tree, 38, huffman_bit, 19)) return "bad header tree";

			rep = 0;
			for(n = 0; n < nlen + nptr; ) {
				RPEEK(16);
				for(huf = 0; huf >= 0; huf = length_tree[huf + bit]) {
					bit = code & 1; RSHIFT(1);
				}
				huf = ~huf;

				bit = 1;
				switch(huf) {
				case 16: huf = rep; bit = (code & 3) + 3; RSHIFT(2); break;
				case 17: huf = 0; bit = (code & 7) + 3; RSHIFT(3); break;
				case 18: huf = 0; bit = (code & 127) + 11; RSHIFT(7); break;
				}
				if(n + bit > nlen + nptr) return "bad tree length";

				for(; bit > 0; bit--) {
					huffman_bit[n] = huf;
					n++;
				}
				rep = huf;
			}

		} else return "bad block type";

		/* codes */

		if(canonical_huffman_tree(length_tree, 576, huffman_bit, nlen)) return "bad character tree";
		if(canonical_huffman_tree(pointer_tree, 64, huffman_bit + nlen, nptr)) return "bad string tree";

		for(rep = 100000; rep > 0; rep--) {

			RPEEK(16);
			for(huf = 0; huf >= 0; huf = length_tree[huf + bit]) {
				bit = code & 1; RSHIFT(1);
			}
			huf = ~huf;

			if(huf > 256) {

				len = huf - 257;
				if(len >= 8) {
					if(len < 28) {
						bit = (len >> 2) - 1;
						len = (4 + (len & 3)) << bit;
						RPEEK(bit); len += code & ((1 << bit) - 1); RSHIFT(bit);
					} else if(len == 28) {
						len = 255;
					} else return "bad string length code";
				}
				len += 3;

				RPEEK(16);
				for(huf = 0; huf >= 0; huf = pointer_tree[huf + bit]) {
					bit = code & 1; RSHIFT(1);
				}
				huf = ~huf;

				ptr = huf;
				if(ptr >= 4) {
					if(ptr < 30) {
						bit = (ptr >> 1) - 1;
						ptr = (2 + (ptr & 1)) << bit;
						RPEEK(bit); ptr += code & ((1 << bit) - 1); RSHIFT(bit);
					} else return "bad string pointer code";
				}
				ptr += 1;

				for(; len > 0; len--) {
					huf = dictionary[(dic - ptr) & 32767];
					dictionary[(dic++) & 32767] = huf;

					if((dic & 32767) == 0) if(fwrite(dictionary, 32768, 1, out_file) != 1) return "write error";
				}

			} else if(huf < 256) {

				dictionary[(dic++) & 32767] = huf;

				if((dic & 32767) == 0) if(fwrite(dictionary, 32768, 1, out_file) != 1) return "write error";

			} else {
				break; /* end of block */
			}
		}
	}

	if((dic & 32767) != 0) if(fwrite(dictionary, (dic & 32767), 1, out_file) != 1) return "write error";

	/* crc not checked */

	if(fflush(out_file) != 0) return "flush error";
	return(0);
}
