/* handle display, keyboard, joystick, and audio in emerald mine */

/* uses win32 graphics device interface and multimedia system */

extern char *global;
static char *local = "mingw";

#include "proto.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <mmsystem.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* ---------------------------------------------------------------------- */

static void parseGeometry(char *parsestring, int *x_return, int *y_return, unsigned int *width_return, unsigned int *height_return)
{
	int x = 0, y = 0;
	unsigned int width = 0, height = 0;
	char gotxy = 0, gotwh = 0;
	if(*parsestring == '=') parsestring++;
	if(*parsestring != '+' && *parsestring != '-') {
		char *endptr;
		width = strtoul(parsestring, &endptr, 10);
		if(parsestring == endptr) return;
		parsestring = endptr;
		if(*parsestring != 'x' && *parsestring != 'X') return;
		parsestring++;
		height = strtoul(parsestring, &endptr, 10);
		if(parsestring == endptr) return;
		parsestring = endptr;
		gotwh = 1;
	}
	if(*parsestring == '+' || *parsestring == '-') {
		char *endptr;
		parsestring++;
		x = strtol(parsestring, &endptr, 10);
		if(parsestring == endptr) return;
		parsestring = endptr;
		if(*parsestring != '+' && *parsestring != '-') return;
		parsestring++;
		y = strtol(parsestring, &endptr, 10);
		if(parsestring == endptr) return;
		parsestring = endptr;
		gotxy = 1;
	}
	if(*parsestring != 0) return;

	if(gotxy) {
		if(x_return) *x_return = x;
		if(y_return) *y_return = y;
	}
	if(gotwh) {
		if(width_return) *width_return = width;
		if(height_return) *height_return = height;
	}
}

/* ---------------------------------------------------------------------- */

/* create window, create palette, create image */

static int scale_x = 2; /* default to double size because it looks better on most displays */
static int scale_y = 2;

static int center_dx, center_dy;

static HWND windowHWND;
static HDC window;

static unsigned long paletteHash[128];
static struct {
	BITMAPINFO x;
	RGBQUAD c[COLOUR_MAX];
} drawImage;

static HBRUSH blackPixel;
static HBRUSH whitePixel;
static HBRUSH redPixel;

static WNDCLASS wndClass;
static int wndClassFlag;
static RECT sizeHints;

int display_open(void)
{
	char buf[256];
	int i;

	for(i = 0; i < 128; i++) paletteHash[i] = COLOUR_black;

	for(i = 0; i < COLOUR_MAX; i++) {
		unsigned long rgbc = colours_xpm[i];
		drawImage.x.bmiColors[i].rgbRed = (rgbc >> 24) & 255;
		drawImage.x.bmiColors[i].rgbGreen = (rgbc >> 16) & 255;
		drawImage.x.bmiColors[i].rgbBlue = (rgbc >> 8) & 255;
		paletteHash[rgbc & 127] = i;
	}

	blackPixel = CreateSolidBrush(RGB(drawImage.x.bmiColors[COLOUR_black].rgbRed, drawImage.x.bmiColors[COLOUR_black].rgbGreen, drawImage.x.bmiColors[COLOUR_black].rgbBlue));
	whitePixel = CreateSolidBrush(RGB(drawImage.x.bmiColors[COLOUR_white].rgbRed, drawImage.x.bmiColors[COLOUR_white].rgbGreen, drawImage.x.bmiColors[COLOUR_white].rgbBlue));
	redPixel = CreateSolidBrush(RGB(drawImage.x.bmiColors[COLOUR_red].rgbRed, drawImage.x.bmiColors[COLOUR_red].rgbGreen, drawImage.x.bmiColors[COLOUR_red].rgbBlue));
	if(blackPixel == 0 || whitePixel == 0 || redPixel == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create brushes");
		goto fail;
	}

	sizeHints.left = 0;
	sizeHints.top = 0;
	if(arg_fullscreen) {
		sizeHints.right = GetSystemMetrics(SM_CXSCREEN);
		sizeHints.bottom = GetSystemMetrics(SM_CYSCREEN);
		scale_x = sizeHints.right / VIEW_WIDTH;
		scale_y = sizeHints.bottom / VIEW_HEIGHT;
		if(scale_x < scale_y) scale_y = scale_x; else scale_x = scale_y; /* keep it square */
	}
	if(arg_geometry) {
		int dummy;
		unsigned int w = scale_x;
		unsigned int h = scale_y;
		parseGeometry(arg_geometry, &dummy, &dummy, &w, &h);
		scale_x = w;
		scale_y = h;
	}
	if(scale_x < 1 || scale_x > 9 || scale_y < 1 || scale_y > 9) {
		scale_x = 1;
		scale_y = 1;
	}
	if(!arg_fullscreen) {
		sizeHints.right = VIEW_WIDTH * scale_x;
		sizeHints.bottom = VIEW_HEIGHT * scale_y;
	}
	center_dx = (sizeHints.right - VIEW_WIDTH * scale_x) / 2;
	center_dy = (sizeHints.bottom - VIEW_HEIGHT * scale_y) / 2;

	if(!arg_fullscreen) {
		AdjustWindowRect(&sizeHints, WS_CAPTION, 0);
		sizeHints.right -= sizeHints.left;
		sizeHints.bottom -= sizeHints.top;
		sizeHints.left = CW_USEDEFAULT;
		sizeHints.top = CW_USEDEFAULT;
		if(arg_geometry) {
			unsigned int dummy;
			int x = sizeHints.left;
			int y = sizeHints.top;
			parseGeometry(arg_geometry, &x, &y, &dummy, &dummy);
			sizeHints.left = x;
			sizeHints.top = y;
		}
	}

	wndClass.lpfnWndProc = DefWindowProc;
	wndClass.lpszClassName = global;
	wndClass.hbrBackground = blackPixel;
	wndClassFlag = RegisterClass(&wndClass);
	if(wndClassFlag == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to RegisterClass");
		goto fail;
	}
	windowHWND = CreateWindow(wndClass.lpszClassName, "X11 Emerald Mine", (arg_fullscreen ? WS_POPUP | WS_VISIBLE : WS_OVERLAPPED | WS_CAPTION | WS_VISIBLE), sizeHints.left, sizeHints.top, sizeHints.right, sizeHints.bottom, 0, 0, 0, 0);
	if(windowHWND == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to open window");
		goto fail;
	}
	window = GetDC(windowHWND);
	if(window == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to get window device context");
		goto fail;
	}

	return(0);
fail:
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, GetLastError(), 0, buf + strlen(buf), 128, 0);
	MessageBox(0, buf, 0, 0);
	return(1);
}

/* handle frame sync, events */

int keyboard_raw = -1;
int keyboard_cook = -1;

static char keyboardVector[512];

static long frame_msec;

static int cursor_timer;
static int cursor_hidden;

void display_read(void)
{
	long msec;
	MSG event;

	while(PeekMessage(&event, 0, 0, 0, PM_REMOVE)) {
		int kc;
		switch(event.message) {
		case WM_KEYDOWN: /* KeyPress */
		case WM_SYSKEYDOWN:
			if(event.hwnd == windowHWND) {
				kc = (event.lParam >> 16) & 511; /* keycode */
				keyboardVector[kc] = 1;
				keyboard_raw = kc;
			}
			TranslateMessage(&event);
			break;
		case WM_CHAR:
			if(event.hwnd == windowHWND) keyboard_cook = event.wParam;
			break;
		case WM_KEYUP: /* KeyRelease */
		case WM_SYSKEYUP:
			if(event.hwnd == windowHWND) {
				kc = (event.lParam >> 16) & 511; /* keycode */
				keyboardVector[kc] = 0;
				keyboard_raw = -1;
				keyboard_cook = -1;
			}
			break;
		case WM_MOUSEMOVE: /* MotionNotify */
			if(event.hwnd == windowHWND) cursor_timer = 32;
			break;
		default:
			DispatchMessage(&event);
			break;
		}
	}
	if(cursor_timer == 0 && cursor_hidden == 0) {
		ShowCursor(0);
		cursor_hidden = 1;
	}
	if(cursor_timer && cursor_hidden) {
		ShowCursor(1);
		cursor_hidden = 0;
	}
	if(cursor_timer) cursor_timer--;

	msec = frame_msec + 1000;
	frame_msec = timeGetTime() * 60;
	msec -= frame_msec;
	if(msec >= 0 && msec <= 1500) {
		frame_msec += msec;
		Sleep(msec / 60);
	}
}

int display_close(void)
{
	if(window) { ReleaseDC(windowHWND, window); window = 0; }
	if(windowHWND) { DestroyWindow(windowHWND); windowHWND = 0; }
	if(wndClassFlag) { UnregisterClass(wndClass.lpszClassName, 0); wndClassFlag = 0; }
	if(blackPixel) { DeleteObject(blackPixel); blackPixel = 0; }
	if(whitePixel) { DeleteObject(whitePixel); whitePixel = 0; }
	if(redPixel) { DeleteObject(redPixel); redPixel = 0; }
	return(0);
}

/* ---------------------------------------------------------------------- */

/* handle keyboard, key names */

char keyboard_input[2][INPUT_MAX];

int keyboard_keycode[2][INPUT_MAX];

static const int defaultKeyCode[INPUT_MAX][2] = {
	[INPUT_north] = { 0x148, 0x11 }, /* VK_UP, 'W' */
	[INPUT_east] = { 0x14d, 0x20 }, /* VK_RIGHT, 'D' */
	[INPUT_south] = { 0x150, 0x1f }, /* VK_DOWN, 'S' */
	[INPUT_west] = { 0x14b, 0x1e }, /* VK_LEFT, 'A' */
	[INPUT_shoot] = { 0x11d, 0x1d }, /* VK_RCONTROL, VK_LCONTROL */
	[INPUT_abort] = { 0x1, 0x1 }, /* VK_ESCAPE, VK_ESCAPE */
};
static char keyboardString[512][12];

int keyboard_open(void)
{
	int ply, button, kc;
	for(ply = 0; ply < 2; ply++) {
		for(button = 0; button < INPUT_MAX; button++) {
			keyboard_keycode[ply][button] = defaultKeyCode[button][ply];
		}
	}
	memset(keyboardString, 0, sizeof(keyboardString));
	for(kc = 0; kc < 512; kc++) {
		GetKeyNameText(kc << 16, keyboardString[kc], sizeof(keyboardString[kc]));
	}
	for(kc = 0; kc < 512; kc++) {
		char *ptr;
		for(ptr = keyboardString[kc]; *ptr; ptr++) if(*ptr == ' ') *ptr = '_';
	}
	return(0);
}

void keyboard_read(void)
{
	int ply, button;
	for(ply = 0; ply < 2; ply++) {
		for(button = 0; button < INPUT_MAX; button++) {
			int kc = keyboard_keycode[ply][button] & 511;
			keyboard_input[ply][button] = keyboardVector[kc];
		}
	}
}

int keyboard_close(void)
{
	return(0);
}

char *keyboard_to_name(int kc)
{
	return(keyboardString[kc & 511]);
}

int keyboard_to_keycode(char *kn)
{
	int kc = 512;
	if(*kn) for(kc = 0; kc < 512; kc++) if(strcmp(kn, keyboardString[kc]) == 0) break;
	if(kc >= 512) kc = -1;
	return(kc);
}

/* ---------------------------------------------------------------------- */

/* handle joysticks */

char joystick_input[2][INPUT_MAX];

static int joy_fd[2] = { -1, -1 };
static time_t joy_delay;

static JOYINFO joy_event[1];

int joystick_open(void)
{
	return(0);
}

void joystick_read(void)
{
	int ply;
	int len, i;
	for(ply = 0; ply < 2; ply++) {

		if(joy_fd[ply] == -1) {
			time_t joy_time = time(0);
			if((unsigned)(joy_time - joy_delay) > 2) {
				joy_delay = joy_time;
				for(i = 0; i < joyGetNumDevs(); i++) {
					if(joy_fd[1-ply] != i) { /* test for same device */
						if(joyGetPos(i, joy_event) == 0) joy_fd[ply] = i;
					}
					if(joy_fd[ply] != -1) break;
				}
			}
		}

		len = -1;
		if(joy_fd[ply] != -1) {
			len = joyGetPos(joy_fd[ply], joy_event);
			if(len) joy_fd[ply] = -1;
		}
		if(len == 0) {
			char *joy = joystick_input[ply];

			joy[INPUT_north] = (joy_event->wYpos < 0x4000);
			joy[INPUT_east] = (joy_event->wXpos > 0xc000);
			joy[INPUT_south] = (joy_event->wYpos > 0xc000);
			joy[INPUT_west] = (joy_event->wXpos < 0x4000);
			joy[INPUT_shoot] = (joy_event->wButtons != 0);
		}
	}
}

int joystick_close(void)
{
	joy_fd[0] = -1;
	joy_fd[1] = -1;
	return(0);
}

/* ---------------------------------------------------------------------- */

/* handle audio */

static char sound_play[SOUND_MAX]; /* should probably be protected by a semaphore */
static int sound_ptr[SOUND_MAX]; /* play position in the sound */

static DWORD WINAPI audio_thread(LPVOID lpParameter);
static DWORD audio_pid;
static HANDLE audio_handle;

int audio_open(void)
{
	char buf[256];

	audio_handle = CreateThread(0, 0, audio_thread, 0, 0, &audio_pid);
	if(audio_handle == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create sound thread");
		goto fail;
	}
	SetThreadPriority(audio_handle, THREAD_PRIORITY_HIGHEST);
	return(0);
fail:
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, GetLastError(), 0, buf + strlen(buf), 128, 0);
	MessageBox(0, buf, 0, 0);
	return(1);
}

void audio_write(struct LOGIC *lev)
{
	int i;
	for(i = 0; i < SOUND_MAX; i++) if(lev->sound[i]) {
		sound_play[i] = 1;
	}
}

int audio_close(void)
{
	if(audio_handle) {
		PostThreadMessage(audio_pid, WM_QUIT, 0, 0);
		CloseHandle(audio_handle);
		audio_handle = 0;
	}
	return(0);
}

/* mix audio data */

static int mix_play[MIXER_MAX]; /* sounds we have chosen to play */
static int mix_count;

#define BUFFER_SIZE 256

static short buf_mix[BUFFER_SIZE]; /* buffer to mix sounds */
static char buf_out[2][BUFFER_SIZE]; /* buffer written to device */

static void audio_mix(char *buf_out)
{
	int i;
	for(i = 0; i < mix_count; i++) {
		register const unsigned char *in = sounds_data[mix_play[i]] + sound_ptr[mix_play[i]];
		register short *out = buf_mix;
		register int len = sounds_length[mix_play[i]] - sound_ptr[mix_play[i]];
		if(len > BUFFER_SIZE) len = BUFFER_SIZE;
		do *out++ += ulaw_to_linear[*in++] / MIXER_MAX; while(--len);
	}
	{
		register const short *in = buf_mix;
		register char *out = buf_out;
		register int len = BUFFER_SIZE;
		do *out++ = (*in++ + 32768) >> 8; while(--len);
	}
}

/* audio thread */

static HWAVEOUT audio_fd;
static WAVEFORMATEX audioInfo;
static WAVEHDR audio_hdr[2]; /* double buffer */

static DWORD WINAPI audio_thread(LPVOID lpParameter)
{
	char buf[256];

	MSG event;
	int i;

	audioInfo.wFormatTag = WAVE_FORMAT_PCM;
	audioInfo.nChannels = 1;
	audioInfo.nSamplesPerSec = 8000;
	audioInfo.nAvgBytesPerSec = 8000;
	audioInfo.nBlockAlign = 1;
	audioInfo.wBitsPerSample = 8;
	audioInfo.cbSize = 0;

	i = waveOutOpen(&audio_fd, WAVE_MAPPER, &audioInfo, GetCurrentThreadId(), 0, CALLBACK_THREAD);
	if(i) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to open audio device");
		waveOutGetErrorText(i, buf + strlen(buf), 128);
		goto fail;
	}
	for(i = 0; i < 2; i++) {
		audio_hdr[i].lpData = buf_out[i];
		audio_hdr[i].dwBufferLength = BUFFER_SIZE;
		audio_hdr[i].dwFlags = 0;
		waveOutPrepareHeader(audio_fd, &audio_hdr[i], sizeof(audio_hdr[i]));
	}
	for(i = 0; i < 2; i++) {
		memset(buf_out[i], 128, sizeof(buf_out[i]));
		waveOutWrite(audio_fd, &audio_hdr[i], sizeof(audio_hdr[i]));
	}

	memset(sound_ptr, -1, sizeof(sound_ptr));

	while(GetMessage(&event, 0, 0, 0)) {
		switch(event.message) {
		case MM_WOM_DONE:

			mix_count = 0;

			for(i = 0; i < SOUND_MAX; i++) if(sound_play[i]) {
				sound_play[i] = 0;
				sound_ptr[i] = 0;
			}
			for(i = 0; i < SOUND_MAX; i++) if(sound_ptr[i] != -1) {
				while(mix_count < MIXER_MAX) {
					mix_play[mix_count++] = i;
					if(i >= SOUND_boom) break;
				}
			}

			for(i = 0; i < 2; i++) {
				if((void *)(event.lParam) == &audio_hdr[i]) {
					memset(buf_mix, 0, sizeof(buf_mix));
					audio_mix(buf_out[i]);
					waveOutWrite(audio_fd, &audio_hdr[i], sizeof(audio_hdr[i]));
				}
			}
			for(i = 0; i < SOUND_MAX; i++) if(sound_ptr[i] != -1) {
				sound_ptr[i] += BUFFER_SIZE;
				if(sound_ptr[i] >= sounds_length[i]) sound_ptr[i] = -1;
			}
			break;
		}
	}

	waveOutReset(audio_fd);
	for(i = 0; i < 2; i++) {
		waveOutUnprepareHeader(audio_fd, &audio_hdr[i], sizeof(audio_hdr[i]));
	}
	waveOutClose(audio_fd); audio_fd = 0;

	return(0);
fail:
	MessageBox(0, buf, 0, 0);
	return(1);
}

/* ---------------------------------------------------------------------- */

/* create device contexts, create pixmaps */

static HDC screenPixmap;
static HDC objectPixmap;
static HDC objectBitmap;
static HDC scorePixmap;
static HDC scoreBitmap;
static HDC spriteBitmap;

static HBITMAP screenPM;
static HBITMAP objectPM;
static HBITMAP objectBM;
static HBITMAP scorePM;
static HBITMAP scoreBM;
static HBITMAP spriteBM;

static HBITMAP screenPMsave;
static HBITMAP objectPMsave;
static HBITMAP objectBMsave;
static HBITMAP scorePMsave;
static HBITMAP scoreBMsave;
static HBITMAP spriteBMsave;

int screen_open(void)
{
	char buf[256];

	screenPixmap = CreateCompatibleDC(window);
	if(screenPixmap == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create screen pixmap device context");
		goto fail;
	}
	screenPM = CreateCompatibleBitmap(window, (VIEW_WIDTH + SCROLL_WIDTH) * scale_x, (VIEW_HEIGHT + SCROLL_HEIGHT) * scale_y);
	if(screenPM == 0) {
		sprintf(buf, "%s[%s] %s (%dx%d): ", global, local, "failed to create screen pixmap", (VIEW_WIDTH + SCROLL_WIDTH) * scale_x, (VIEW_HEIGHT + SCROLL_HEIGHT) * scale_y);
		goto fail;
	}
	screenPMsave = SelectObject(screenPixmap, screenPM);

	objectPixmap = CreateCompatibleDC(window);
	if(objectPixmap == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create graphics pixmap device context");
		goto fail;
	}
	objectPM = CreateCompatibleBitmap(window, GFX_WIDTH * scale_x, GFX_HEIGHT * scale_y);
	if(objectPM == 0) {
		sprintf(buf, "%s[%s] %s (%dx%d): ", global, local, "failed to create graphics pixmap", GFX_WIDTH * scale_x, GFX_HEIGHT * scale_y);
		goto fail;
	}
	objectPMsave = SelectObject(objectPixmap, objectPM);

	objectBitmap = CreateCompatibleDC(window);
	if(objectBitmap == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create graphics bitmap device context");
		goto fail;
	}
	objectBM = CreateBitmap(GFX_WIDTH * scale_x, GFX_HEIGHT * scale_y, 1, 1, 0);
	if(objectBM == 0) {
		sprintf(buf, "%s[%s] %s (%dx%d): ", global, local, "failed to create graphics bitmap", GFX_WIDTH * scale_x, GFX_HEIGHT * scale_y);
		goto fail;
	}
	objectBMsave = SelectObject(objectBitmap, objectBM);

	scorePixmap = CreateCompatibleDC(window);
	if(scorePixmap == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create score pixmap device context");
		goto fail;
	}
	scorePM = CreateCompatibleBitmap(window, SCORE_WIDTH * scale_x, 8 * scale_y);
	if(scorePM == 0) {
		sprintf(buf, "%s[%s] %s (%dx%d): ", global, local, "failed to create score pixmap", SCORE_WIDTH * scale_x, 8 * scale_y);
		goto fail;
	}
	scorePMsave = SelectObject(scorePixmap, scorePM);

	scoreBitmap = CreateCompatibleDC(window);
	if(scoreBitmap == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create score bitmap device context");
		goto fail;
	}
	scoreBM = CreateBitmap(SCORE_WIDTH * scale_x, 8 * scale_y, 1, 1, 0);
	if(scoreBM == 0) {
		sprintf(buf, "%s[%s] %s (%dx%d): ", global, local, "failed to create score bitmap", SCORE_WIDTH * scale_x, 8 * scale_y);
		goto fail;
	}
	scoreBMsave = SelectObject(scoreBitmap, scoreBM);

	spriteBitmap = CreateCompatibleDC(window);
	if(spriteBitmap == 0) {
		sprintf(buf, "%s[%s] %s: ", global, local, "failed to create sprite bitmap device context");
		goto fail;
	}
	spriteBM = CreateBitmap(16 * scale_x, 16 * scale_y, 1, 1, 0);
	if(spriteBM == 0) {
		sprintf(buf, "%s[%s] %s (%dx%d): ", global, local, "failed to create sprite bitmap", 16 * scale_x, 16 * scale_y);
		goto fail;
	}
	spriteBMsave = SelectObject(spriteBitmap, spriteBM);

	return(screen_write());
fail:
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, GetLastError(), 0, buf + strlen(buf), 128, 0);
	MessageBox(0, buf, 0, 0);
	return(1);
}

/* render graphics */

static char *pixmap_image_data;
static char *bitmap_image_data;

static int pixmap_image_bytes;
static int bitmap_image_bytes;

int screen_write(void)
{
	char buf[256];

	int x, in_x, out_x;
	int y, in_y, out_y;

	drawImage.x.bmiHeader.biSize = sizeof(drawImage.x.bmiHeader);
	drawImage.x.bmiHeader.biWidth = GFX_WIDTH * scale_x;
	drawImage.x.bmiHeader.biHeight = GFX_HEIGHT * scale_y;
	drawImage.x.bmiHeader.biPlanes = 1;
	drawImage.x.bmiHeader.biBitCount = 8;
	drawImage.x.bmiHeader.biCompression = BI_RGB;
	drawImage.x.bmiHeader.biClrUsed = COLOUR_MAX;

	pixmap_image_bytes = (drawImage.x.bmiHeader.biWidth * drawImage.x.bmiHeader.biBitCount + 7) / 8;
	pixmap_image_data = malloc(drawImage.x.bmiHeader.biHeight * pixmap_image_bytes);
	if(pixmap_image_data == 0) {
		sprintf(buf, "%s[%s] %s (%d): ", global, local, "failed to allocate memory for pixmap image", drawImage.x.bmiHeader.biHeight * pixmap_image_bytes);
		goto fail;
	}
	bitmap_image_bytes = (drawImage.x.bmiHeader.biWidth * drawImage.x.bmiHeader.biBitCount + 7) / 8;
	bitmap_image_data = malloc(drawImage.x.bmiHeader.biHeight * bitmap_image_bytes);
	if(bitmap_image_data == 0) {
		sprintf(buf, "%s[%s] %s (%d): ", global, local, "failed to allocate memory for bitmap image", drawImage.x.bmiHeader.biHeight * bitmap_image_bytes);
		goto fail;
	}
	drawImage.x.bmiHeader.biHeight = -drawImage.x.bmiHeader.biHeight;

	out_y = 0;
	for(in_y = 0; in_y < GFX_HEIGHT; in_y++) {
		char *pixmap_ptr = pixmap_image_data + out_y * pixmap_image_bytes;
		char *bitmap_ptr = bitmap_image_data + out_y * bitmap_image_bytes;
		memset(bitmap_ptr, COLOUR_black, bitmap_image_bytes);
		out_x = 0;
		for(in_x = 0; in_x < GFX_WIDTH; in_x++) {
			char c = graphics_xpm[in_x][in_y];
			unsigned long pixel = paletteHash[c & 127];
			for(x = 0; x < scale_x; x++) {
				pixmap_ptr[out_x] = pixel;
				if(c != ' ') bitmap_ptr[out_x] = COLOUR_white;
				out_x++;
			}
		}
		for(y = 1; y < scale_y; y++) {
			memcpy(pixmap_ptr + y * pixmap_image_bytes, pixmap_ptr, pixmap_image_bytes);
			memcpy(bitmap_ptr + y * bitmap_image_bytes, bitmap_ptr, bitmap_image_bytes);
		}
		out_y += scale_y;
	}
	SetDIBitsToDevice(objectPixmap, 0, 0, GFX_WIDTH * scale_x, GFX_HEIGHT * scale_y, 0, 0, 0, GFX_HEIGHT * scale_y, pixmap_image_data, &drawImage.x, DIB_RGB_COLORS);
	SetDIBitsToDevice(objectBitmap, 0, 0, GFX_WIDTH * scale_x, GFX_HEIGHT * scale_y, 0, 0, 0, GFX_HEIGHT * scale_y, bitmap_image_data, &drawImage.x, DIB_RGB_COLORS);

	free(pixmap_image_data); pixmap_image_data = 0;
	free(bitmap_image_data); bitmap_image_data = 0;

	return(0);
fail:
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, GetLastError(), 0, buf + strlen(buf), 128, 0);
	MessageBox(0, buf, 0, 0);
	return(1);
}

int screen_close(void)
{
	if(pixmap_image_data) { free(pixmap_image_data); pixmap_image_data = 0; }
	if(bitmap_image_data) { free(bitmap_image_data); bitmap_image_data = 0; }
	if(screenPM) { SelectObject(screenPixmap, screenPMsave); DeleteObject(screenPM); screenPM = 0; }
	if(objectPM) { SelectObject(objectPixmap, objectPMsave); DeleteObject(objectPM); objectPM = 0; }
	if(objectBM) { SelectObject(objectBitmap, objectBMsave); DeleteObject(objectBM); objectBM = 0; }
	if(scorePM) { SelectObject(scorePixmap, scorePMsave); DeleteObject(scorePM); scorePM = 0; }
	if(scoreBM) { SelectObject(scoreBitmap, scoreBMsave); DeleteObject(scoreBM); scoreBM = 0; }
	if(spriteBM) { SelectObject(spriteBitmap, spriteBMsave); DeleteObject(spriteBM); spriteBM = 0; }
	if(screenPixmap) { DeleteDC(screenPixmap); screenPixmap = 0; }
	if(objectPixmap) { DeleteDC(objectPixmap); objectPixmap = 0; }
	if(objectBitmap) { DeleteDC(objectBitmap); objectBitmap = 0; }
	if(scorePixmap) { DeleteDC(scorePixmap); scorePixmap = 0; }
	if(scoreBitmap) { DeleteDC(scoreBitmap); scoreBitmap = 0; }
	if(spriteBitmap) { DeleteDC(spriteBitmap); spriteBitmap = 0; }
	return(0);
}

/* ---------------------------------------------------------------------- */

/* game drawing functions */

static short screen_cache[(VIEW_WIDTH + SCROLL_WIDTH) / 16][(VIEW_HEIGHT + SCROLL_HEIGHT) / 16];

static int scn_pixel_left;
static int scn_pixel_top;

void x11_game_clear(void)
{
	RECT r;
	r.left = 0; r.right = (VIEW_WIDTH + SCROLL_WIDTH) * scale_x;
	r.top = 0; r.bottom = (VIEW_HEIGHT + SCROLL_HEIGHT) * scale_y;
	FillRect(screenPixmap, &r, blackPixel);
	memset(screen_cache, -1, sizeof(screen_cache)); /* mark entire screen as dirty */
}

void x11_game_scroll(struct LOGIC *lev)
{
	scn_pixel_left = lev->pixel_left % SCROLL_WIDTH;
	scn_pixel_top = lev->pixel_top % SCROLL_HEIGHT;
}

void x11_game_object(struct LOGIC *lev)
{
	int tile, tile_x, tile_y;
	int scn_x, lev_x;
	int scn_y, lev_y;
	scn_x = scn_pixel_left / 16;
	for(lev_x = lev->view_left; lev_x <= lev->view_right; lev_x++) {
		scn_y = scn_pixel_top / 16;
		for(lev_y = lev->view_top; lev_y <= lev->view_bottom; lev_y++) {
			tile = object_map[lev->cave[lev_x][lev_y]][lev->frame];
			if(screen_cache[scn_x][scn_y] != tile) {
				screen_cache[scn_x][scn_y] = tile;
				tile_x = (tile / GFX_HEIGHT) * 16;
				tile_y = (tile % GFX_HEIGHT);
				BitBlt(screenPixmap, scn_x * 16 * scale_x, scn_y * 16 * scale_y, 16 * scale_x, 16 * scale_y, objectPixmap, tile_x * scale_x, tile_y * scale_y, SRCCOPY);
			}
			scn_y++;
		}
		scn_x++;
	}
}

void x11_game_player(struct LOGIC *lev)
{
	int tile, tile_x, tile_y;
	int dx, x0, x1;
	int dy, y0, y1;
	int ply;
	for(ply = 0; ply < 2; ply++) {
		if(lev->hide[ply]) continue;

		x0 = lev->pl_pixel_left[ply];
		x1 = lev->pl_pixel_right[ply];
		y0 = lev->pl_pixel_top[ply];
		y1 = lev->pl_pixel_bottom[ply];
		if(!(x0 >= lev->pixel_left && x1 <= lev->pixel_right && y0 >= lev->pixel_top && y1 <= lev->pixel_bottom)) continue;

		dx = x0 % 16; dy = y0 % 16;
		tile = object_map[lev->cave[x0 / 16][y0 / 16]][lev->frame];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		BitBlt(spriteBitmap, 0, 0, (16 - dx) * scale_x, (16 - dy) * scale_y, objectBitmap, (tile_x + dx) * scale_x, (tile_y + dy) * scale_y, SRCCOPY);
		dx = 15 - x1 % 16; dy = 15 - y1 % 16;
		tile = object_map[lev->cave[x1 / 16][y1 / 16]][lev->frame];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		BitBlt(spriteBitmap, dx * scale_x, dy * scale_y, (16 - dx) * scale_x, (16 - dy) * scale_y, objectBitmap, tile_x * scale_x, tile_y * scale_y, SRCCOPY);

		x0 -= lev->pixel_left - scn_pixel_left;
		x1 -= lev->pixel_left - scn_pixel_left;
		y0 -= lev->pixel_top - scn_pixel_top;
		y1 -= lev->pixel_top - scn_pixel_top;

		screen_cache[x0 / 16][y0 / 16] = -1; /* mark screen over player as dirty */
		screen_cache[x1 / 16][y1 / 16] = -1;

		tile = sprite_map[lev->pl_anim[ply]][ply][lev->frame];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		BitBlt(screenPixmap, x0 * scale_x, y0 * scale_y, 16 * scale_x, 16 * scale_y, objectPixmap, tile_x * scale_x, tile_y * scale_y, SRCINVERT);
		BitBlt(screenPixmap, x0 * scale_x, y0 * scale_y, 16 * scale_x, 16 * scale_y, spriteBitmap, 0, 0, SRCAND);
		BitBlt(screenPixmap, x0 * scale_x, y0 * scale_y, 16 * scale_x, 16 * scale_y, objectPixmap, tile_x * scale_x, tile_y * scale_y, SRCINVERT);
	}
}

void x11_game_score(struct LOGIC *lev)
{
	int x = scn_pixel_left, x0, x1;
	int y = scn_pixel_top + 4, y0, y1;

	BitBlt(screenPixmap, x * scale_x, y * scale_y, SCORE_WIDTH * scale_x, 8 * scale_y, scorePixmap, 0, 0, SRCINVERT);
	BitBlt(screenPixmap, x * scale_x, y * scale_y, SCORE_WIDTH * scale_x, 8 * scale_y, scoreBitmap, 0, 0, SRCAND);
	BitBlt(screenPixmap, x * scale_x, y * scale_y, SCORE_WIDTH * scale_x, 8 * scale_y, scorePixmap, 0, 0, SRCINVERT);

	x0 = x / 16; x1 = (x + SCORE_WIDTH - 1) / 16;
	y0 = y / 16; y1 = (y + 8 - 1) / 16;
	for(x = x0; x <= x1; x++) for(y = y0; y <= y1; y++) {
		screen_cache[x][y] = -1; /* mark screen under score as dirty */
	}
}

void x11_game_to_screen(void)
{
	BitBlt(window, center_dx, center_dy, VIEW_WIDTH * scale_x, VIEW_HEIGHT * scale_y, screenPixmap, scn_pixel_left * scale_x, scn_pixel_top * scale_y, SRCCOPY);
}

/* ---------------------------------------------------------------------- */

/* score drawing functions */

static short score_cache[SCORE_WIDTH / 8];

void x11_score_clear(void)
{
	RECT r;
	r.left = 0; r.right = SCORE_WIDTH * scale_x;
	r.top = 0; r.bottom = 8 * scale_y;
	FillRect(scorePixmap, &r, blackPixel);
	FillRect(scoreBitmap, &r, blackPixel);
	memset(score_cache, -1, sizeof(score_cache));
}

void x11_score_print(char str[40])
{
	int tile, tile_x, tile_y;
	int x;
	for(x = 0; x < SCORE_WIDTH / 8; x++) {
		tile = score_map[str[x] & 127];
		if(score_cache[x] != tile) {
			score_cache[x] = tile;
			tile_x = (tile / GFX_HEIGHT) * 16;
			tile_y = (tile % GFX_HEIGHT);
			BitBlt(scorePixmap, x * 8 * scale_x, 0, 8 * scale_x, 8 * scale_y, objectPixmap, tile_x * scale_x, tile_y * scale_y, SRCCOPY);
			BitBlt(scoreBitmap, x * 8 * scale_x, 0, 8 * scale_x, 8 * scale_y, objectBitmap, tile_x * scale_x, tile_y * scale_y, NOTSRCCOPY);
		}
	}
}

/* ---------------------------------------------------------------------- */

/* menu drawing functions */

static int antsPos;

void x11_menu_clear(void)
{
	RECT r;
	r.left = 0; r.right = VIEW_WIDTH * scale_x;
	r.top = 0; r.bottom = VIEW_HEIGHT * scale_y;
	FillRect(screenPixmap, &r, blackPixel);
}

void x11_menu_print(char str[26], int top)
{
	RECT r;
	int tile, tile_x, tile_y;
	int x;
	r.left = 0; r.right = VIEW_WIDTH * scale_x;
	r.top = top * 16 * scale_y; r.bottom = (top + 1) * 16 * scale_y;
	FillRect(screenPixmap, &r, blackPixel);
	for(x = 0; x < 26; x++) {
		tile = font_map[str[x] & 127];
		tile_x = (tile / GFX_HEIGHT) * 16;
		tile_y = (tile % GFX_HEIGHT);
		BitBlt(screenPixmap, (x * 12 + 4) * scale_x, top * 16 * scale_y, 16 * scale_x, 16 * scale_y, objectPixmap, tile_x * scale_x, tile_y * scale_y, SRCCOPY);
	}
}

void x11_menu_ants(int left, int right, int top)
{
	RECT r;
	int x, x0 = left, x1 = (right + 1);
	int y, y0 = top * 16, y1 = (top + 1) * 16;
	int pos, inc;

	antsPos = (antsPos + 1) % 16;
	pos = antsPos;

	for(x = x0; x < x1; x += inc) {
		inc = 8 - pos % 8; if(inc > x1 - x) inc = x1 - x;
		r.left = x * scale_x; r.right = (x + inc) * scale_x;
		r.top = y0 * scale_y; r.bottom = (y0 + 1) * scale_y;
		FillRect(screenPixmap, &r, pos / 8 % 2 ? whitePixel : redPixel);
		pos += inc;
	}
	for(y = y0; y < y1; y += inc) {
		inc = 8 - pos % 8; if(inc > y1 - y) inc = y1 - y;
		r.left = (x1 - 1) * scale_x; r.right = x1 * scale_x;
		r.top = y * scale_y; r.bottom = (y + inc) * scale_y;
		FillRect(screenPixmap, &r, pos / 8 % 2 ? whitePixel : redPixel);
		pos += inc;
	}
	for(x = x1; x > x0; x -= inc) {
		inc = 8 - pos % 8; if(inc > x - x0) inc = x - x0;
		r.left = (x - inc) * scale_x; r.right = x * scale_x;
		r.top = (y1 - 1) * scale_y; r.bottom = y1 * scale_y;
		FillRect(screenPixmap, &r, pos / 8 % 2 ? whitePixel : redPixel);
		pos += inc;
	}
	for(y = y1; y > y0; y -= inc) {
		inc = 8 - pos % 8; if(inc > y - y0) inc = y - y0;
		r.left = x0 * scale_x; r.right = (x0 + 1) * scale_x;
		r.top = (y - inc) * scale_y; r.bottom = y * scale_y;
		FillRect(screenPixmap, &r, pos / 8 % 2 ? whitePixel : redPixel);
		pos += inc;
	}
}

void x11_menu_to_screen(void)
{
	BitBlt(window, center_dx, center_dy, VIEW_WIDTH * scale_x, VIEW_HEIGHT * scale_y, screenPixmap, 0, 0, SRCCOPY);
}
