/* unlimited distribution */

/* 2010-04-30 05:21:11 */

#include "amiga.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

struct HW HW;

unsigned char ulawbyte[16384];

extern int ulaw_to_linear(unsigned char);
extern unsigned char linear_to_ulaw(int);

enum {
	FMT_MU_LAW,
	FMT_S8,
	FMT_S16_BE,
	FMT_S16_LE,
	FMT_U8,
	FMT_U16_BE,
	FMT_U16_LE,
	FMT_MAX
};

const char *arg_fmt_str[FMT_MAX] = {
	"mu",
	"s8",
	"s16be",
	"s16le",
	"u8",
	"u16be",
	"u16le",
};

unsigned char au_hdr[24] = {
	46, 115, 110, 100, /* magic */
	0, 0, 0, 24, /* header length */
	255, 255, 255, 255, /* data length */
	0, 0, 0, 1, /* encoding */
	0, 0, 31, 64, /* rate */
	0, 0, 0, 1, /* channels */
};

char *arg_format;
char *arg_channels;
char *arg_rate;
int arg_header;

#if (defined WIN32)
#include <fcntl.h>
#endif

int main(int argc, char **argv)
{
#if (defined WIN32)
	setmode(STDOUT_FILENO, O_BINARY);
#endif
parse:
	switch(getopt(argc, argv, "f:c:r:a")) {
	case 'f': arg_format = optarg; goto parse;
	case 'c': arg_channels = optarg; goto parse;
	case 'r': arg_rate = optarg; goto parse;
	case 'a': arg_header = 1; goto parse;
	default:
		fputs("\nusage: ./emerald_mine_music [options]\n", stderr);
		fputs("\t-f FORMAT     size and type of output\n", stderr);
		fputs("\t-c CHANNELS   number of channels\n", stderr);
		fputs("\t-r RATE       output rate in hertz\n", stderr);
		fputs("\t-a            write sun au header\n", stderr);
		fputs("\nFORMAT can be one of:\n", stderr);
		fputs("'mu' for mu-law\n", stderr);
		fputs("'u8' 's8' for (un)signed 8bit linear\n", stderr);
		fputs("'u16be' 's16be' for (un)signed 16bit linear big endian\n", stderr);
		fputs("'u16le' 's16le' for (un)signed 16bit linear little endian\n", stderr);
		fputs("CHANNELS can be 1 for mono or 2 for stereo\n", stderr);
		fputs("\ndefault is mu-law mono 8000hz which\ncan usually be fed directly to /dev/audio\n", stderr);
		return(1);
	case -1:
		if(optind >= argc) break;
		fprintf(stderr, "%s: too many arguments\n", argv[0]);
		return(1);
	}

	HW.mixer.fmt = FMT_MU_LAW; /* defaults for sun audio */
	HW.mixer.chan = 1;
	HW.mixer.freq = 8000;

	if(arg_format) {
		for(HW.mixer.fmt = 0; HW.mixer.fmt < FMT_MAX; HW.mixer.fmt++) {
			if(strcmp(arg_format, arg_fmt_str[HW.mixer.fmt]) == 0) break;
		}
		if(HW.mixer.fmt >= FMT_MAX) {
			fprintf(stderr, "%s: unrecognized format type\n", argv[0]);
			return(1);
		}
	}
	if(arg_channels) {
		HW.mixer.chan = strtoul(arg_channels, 0, 10);
		if(HW.mixer.chan < 1) HW.mixer.chan = 1;
		if(HW.mixer.chan > 2) HW.mixer.chan = 2;
	}
	if(arg_rate) {
		HW.mixer.freq = strtoul(arg_rate, 0, 10);
		if(HW.mixer.freq < 1) HW.mixer.freq = 1;
	}
	if(arg_header) {
		int i;
		if(HW.mixer.fmt >= 3) {
			fprintf(stderr, "%s: format not compatible with au header (mu-law, s8, s16be)\n", argv[0]);
			return(1);
		}
		au_hdr[15] = HW.mixer.fmt + 1;
		au_hdr[16] = HW.mixer.freq >> 24; au_hdr[17] = HW.mixer.freq >> 16; au_hdr[18] = HW.mixer.freq >> 8; au_hdr[19] = HW.mixer.freq;
		au_hdr[23] = HW.mixer.chan;

		for(i = 0; i < 24; i++) {
			if(putchar(au_hdr[i]) == EOF) { fputs("broken pipe\n", stderr); exit(1); }
		}
	}

	if(HW.mixer.fmt == FMT_MU_LAW) {
		int i;
		for(i = -8192; i < 8192; i++) {
			ulawbyte[i & 16383] = linear_to_ulaw(i * 4 + 2);
		}
	}

	song();

	return(0);
}

void mixer(void)
{
	struct mixer *mix = &HW.mixer;
	unsigned int i;

	mix->err += mix->freq;
	mix->inc = mix->err / HW.timer.vertb;
	mix->err %= HW.timer.vertb;

	for(i = 0; i < mix->inc; i++) {
		unsigned int i;

		for(i = 0; i < 4; i++) {
			struct channel *ch = &HW.channel[i];

			if(ch->ptr + 1 >= ch->len) ch->dat = 0; /* no loops */

			if(ch->dat) {
				short lower = (short)(ch->dat[ch->ptr]) * (short)(ch->vol) * 4;
				short upper = (short)(ch->dat[ch->ptr + 1]) * (short)(ch->vol) * 4;
				ch->sample = lower + ((long)(upper) - (long)(lower)) * (long)(ch->err) / (long)(mix->freq); /* linear interpolation */

				ch->err += ch->freq;
				ch->inc = ch->err / mix->freq;
				ch->err %= mix->freq;

				ch->ptr += ch->inc;
			} else {
				ch->sample = 0;
			}
		}

		mix->sample[0] = HW.channel[0].sample / 2 + HW.channel[3].sample / 2; /* stereo left */
		mix->sample[1] = HW.channel[1].sample / 2 + HW.channel[2].sample / 2; /* stereo right */
		if(mix->chan == 1) {
			mix->sample[0] = mix->sample[0] / 2 + mix->sample[1] / 2; /* mono */
		}

		for(i = 0; i < mix->chan; i++) {
			int x = mix->sample[i];
			switch(mix->fmt) {
			case FMT_MU_LAW:
				if(putchar(ulawbyte[x >> 2 & 16383]) == EOF) { fputs("broken pipe\n", stderr); exit(1); }
				break;
			case FMT_U8:
				x += 32768;
			case FMT_S8:
				if(putchar(x >> 8 & 255) == EOF) { fputs("broken pipe\n", stderr); exit(1); }
				break;
			case FMT_U16_BE:
				x += 32768;
			case FMT_S16_BE:
				if(putchar(x >> 8 & 255) == EOF || putchar(x & 255) == EOF) { fputs("broken pipe\n", stderr); exit(1); }
				break;
			case FMT_U16_LE:
				x += 32768;
			case FMT_S16_LE:
				if(putchar(x & 255) == EOF || putchar(x >> 8 & 255) == EOF) { fputs("broken pipe\n", stderr); exit(1); }
				break;
			}
		}
	}
}
