/* unlimited distribution */

/* 2007-10-07 22:47:56
 *
 * play emerald mine song
 *
 * 02:46
 */

#include "amiga.h"

#include <stdio.h>
#include <stdlib.h>

extern unsigned char program_dat[];
extern unsigned int program_len;

extern signed char slapbass_dat[], voice_dat[], lead_dat[], bassdrum_dat[];
extern signed char snaredrum_dat[], claps_dat[], whistle_dat[], magic_dat[];

extern unsigned int slapbass_len, voice_len, lead_len, bassdrum_len;
extern unsigned int snaredrum_len, claps_len, whistle_len, magic_len;

struct track {
	unsigned int pc;
	unsigned int sp;

	unsigned int instr;
	unsigned int octave;
	unsigned int volume;

	unsigned int note_time;
	unsigned int vibrato_time;
	unsigned int vibrato_value;
	unsigned int fade_time;
	unsigned int fade_value;

	unsigned int note_cnt;
	unsigned int vibrato_cnt;
	unsigned int fade_cnt;
	unsigned int vibrato_pos;

	unsigned int note[4];

	unsigned int stack[16];
} track[4];

unsigned char byte(unsigned int ptr)
{
	ptr += 0x24;
	if(ptr >= program_len) { fputs("bus error\n", stderr); exit(1); }
	return(program_dat[ptr]);
}
unsigned short half(unsigned int ptr)
{
	return((unsigned short)(byte(ptr)) << 8 | (unsigned short)(byte(ptr + 1)));
}
unsigned long word(unsigned int ptr)
{
	return((unsigned long)(half(ptr)) << 16 | (unsigned long)(half(ptr + 2)));
}

void effect(void)
{
	unsigned int i;
	for(i = 0; i < 4; i++) {
		struct track *tr = &track[i];
		struct channel *ch = &HW.channel[i];

		unsigned int note, octave, period;

		if(++tr->vibrato_cnt >= tr->vibrato_time) {
			tr->vibrato_cnt = 0;

			if(++tr->vibrato_pos >= tr->vibrato_value) tr->vibrato_pos = 0;
		}
		if(++tr->fade_cnt >= tr->fade_time) {
			tr->fade_cnt = 0;

			if(ch->vol >= tr->fade_value) {
				ch->vol -= tr->fade_value;
			} else {
				ch->vol = 0;
			}
		}

		note = tr->note[tr->vibrato_pos];
		octave = tr->octave + (note >> 4 & 15);
		note = byte(0x1a4) + (note & 15);
		octave += note / 12;
		note %= 12;
		period = half(0x25c + note * 2);
		period >>= octave;
		if(period < 1) period = 1;
		ch->freq = HW.timer.clock / period;
		if(ch->freq < 1) ch->freq = 1;
	}
}

void note(void)
{
	unsigned int i;
	for(i = 0; i < 4; i++) {
		struct track *tr = &track[i];
		struct channel *ch = &HW.channel[i];
		unsigned int pc;

		unsigned int cpucycles = 0;

		if(++tr->note_cnt < tr->note_time) continue;
		tr->note_cnt = 0;
		tr->vibrato_cnt = 0;
		tr->fade_cnt = 0;
		tr->vibrato_pos = 0;

		pc = tr->pc;
		for(;;) {
			unsigned int cmd = byte(pc); pc++;

			if(cmd < 0xc0) { /* note */
				unsigned int i;
				tr->note[0] = cmd;
				for(i = 1; i < tr->vibrato_value; i++) {
					cmd = byte(pc); pc++;
					if(cmd >= 0xc0) { fputs("illegal instruction\n", stderr); exit(1); }
					tr->note[i] = cmd;
				}
				break;
			}

			if(++cpucycles > 1000) { fputs("cpu time limit exceeded\n", stderr); exit(1); }

			if(cmd >= 0xc0 && cmd <= 0xef) { /* speed */
				tr->note_time = cmd - 0xc0;
				continue;
			}
			switch(cmd) {
			case 0xf0:
				tr->instr = word(pc) >> 13 & 7;
				tr->octave = half(pc + 12);
				tr->volume = half(pc + 30);
				tr->vibrato_time = half(pc + 16);
				tr->fade_time = half(pc + 18);
				cmd = half(pc + 14);
				tr->vibrato_value = (cmd & 1) ? (cmd & 2) ? 4 : 2 : 1;
				tr->fade_value = (cmd & 4) ? 8 : 0;
				pc += 34;
				continue;
			case 0xf1: /* call */
				if(tr->sp >= 16) { fputs("stack fault\n", stderr); exit(1); }
				tr->stack[tr->sp++] = pc + 4;
			case 0xf3: /* goto */
				pc = word(pc);
				continue;
			case 0xf2: /* ret */
				if(tr->sp <= 0) { fputs("stack fault\n", stderr); exit(1); }
				pc = tr->stack[--tr->sp];
			case 0xf4: /* nop */
				continue;
			}
			fputs("illegal instruction\n", stderr); exit(1);
		}
		tr->pc = pc;

		switch(tr->instr) {
		case 0: ch->dat = slapbass_dat; ch->len = slapbass_len; break;
		case 1: ch->dat = voice_dat; ch->len = voice_len; break;
		case 2: ch->dat = lead_dat; ch->len = lead_len; break;
		case 3: ch->dat = bassdrum_dat; ch->len = bassdrum_len; break;
		case 4: ch->dat = snaredrum_dat; ch->len = snaredrum_len; break;
		case 5: ch->dat = claps_dat; ch->len = claps_len; break;
		case 6: ch->dat = whistle_dat; ch->len = whistle_len; break;
		case 7: ch->dat = magic_dat; ch->len = magic_len; break;
		}
		ch->ptr = 0;
		ch->vol = tr->volume;
	}
}

void song(void)
{
	HW.timer.clock = 3579545; /* NTSC */
	HW.timer.vertb = 60;

	track[0].pc = 0x98c;
	track[1].pc = 0xa24;
	track[2].pc = 0xb24;
	track[3].pc = 0xbfc;

	for(;;) {
		note();
		effect();
		mixer();
		effect();
		mixer();
		effect();
		mixer();
		effect();
		mixer();
	}
}
